@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cli.c 8411 2009-05-15 15:42:13Z charles $
+ * $Id: cli.c 10240 2010-02-20 15:57:34Z charles $
  *
  * Copyright (c) 2005-2006 Transmission authors and contributors
  *
@@ -32,12 +32,12 @@
 #include <libtransmission/bencode.h>
 #include <libtransmission/makemeta.h>
 #include <libtransmission/tr-getopt.h>
-#include <libtransmission/utils.h> /* tr_wait */
+#include <libtransmission/utils.h> /* tr_wait_msec */
 #include <libtransmission/version.h>
 #include <libtransmission/web.h> /* tr_webRun */
 
 #define LINEWIDTH 80
-#define MY_NAME "transmission-cli"
+#define MY_NAME "transmissioncli"
 
 static tr_bool showInfo         = 0;
 static tr_bool showScrape       = 0;
@@ -88,7 +88,7 @@ getUsage( void )
 {
     return "A fast and easy BitTorrent client\n"
            "\n"
-           "Usage: " MY_NAME " [options] <torrent-filename>";
+           "Usage: " MY_NAME " [options] <file|url|magnet>";
 }
 
 static int parseCommandLine( tr_benc*, int argc, const char ** argv );
@@ -145,6 +145,19 @@ torrentCompletenessChanged( tr_torrent       * torrent       UNUSED,
     system( finishCall );
 }
 
+static tr_bool waitingOnWeb;
+
+static void
+onTorrentFileDownloaded( tr_session   * session UNUSED,
+                         long           response_code UNUSED,
+                         const void   * response,
+                         size_t         response_byte_count,
+                         void         * ctor )
+{
+    tr_ctorSetMetainfo( ctor, response, response_byte_count );
+    waitingOnWeb = FALSE;
+}
+
 static int leftToScrape = 0;
 
 static void
@@ -235,8 +248,8 @@ getStatusStr( const tr_stat * st,
     {
         tr_snprintf( buf, buflen,
                      "Verifying local files (%.2f%%, %.2f%% valid)",
-                     100 * st->recheckProgress,
-                     100.0 * st->percentDone );
+                     tr_truncd( 100 * st->recheckProgress, 2 ),
+                     tr_truncd( 100 * st->percentDone, 2 ) );
     }
     else if( st->activity & TR_STATUS_DOWNLOAD )
     {
@@ -246,7 +259,7 @@ getStatusStr( const tr_stat * st,
             buf, buflen,
             "Progress: %.1f%%, dl from %d of %d peers (%.0f KB/s), "
             "ul to %d (%.0f KB/s) [%s]",
-            st->percentDone * 100.0,
+            tr_truncd( 100 * st->percentDone, 1 ),
             st->peersSendingToUs,
             st->peersConnected,
             st->pieceDownloadSpeed,
@@ -298,8 +311,12 @@ main( int     argc,
     tr_session  * h;
     tr_ctor     * ctor;
     tr_torrent  * tor = NULL;
-    tr_benc settings;
-    const char * configDir;
+    tr_benc       settings;
+    const char  * configDir;
+    tr_bool       haveSource;
+    tr_bool       haveAnnounce;
+    uint8_t     * fileContents;
+    size_t        fileLength;
 
     printf( "Transmission %s - http://www.transmissionbt.com/\n",
             LONG_VERSION_STRING );
@@ -332,14 +349,23 @@ main( int     argc,
 
     h = tr_sessionInit( "cli", configDir, FALSE, &settings );
 
-    if( sourceFile && *sourceFile ) /* creating a torrent */
+    haveSource = sourceFile && *sourceFile;
+    haveAnnounce = announceCount > 0;
+
+    if( haveSource && !haveAnnounce )
+        fprintf( stderr, "Did you mean to create a torrent without a tracker's announce URL?\n" );
+
+    if( haveSource ) /* creating a torrent */
     {
-        int                   err;
-        tr_metainfo_builder * b = tr_metaInfoBuilderCreate( sourceFile );
+        int err;
+        tr_metainfo_builder * b;
+        fprintf( stderr, "creating torrent \"%s\"\n", torrentPath );
+
+        b = tr_metaInfoBuilderCreate( sourceFile );
         tr_makeMetaInfo( b, torrentPath, announce, announceCount, comment, isPrivate );
         while( !b->isDone )
         {
-            tr_wait( 1000 );
+            tr_wait_msec( 1000 );
             printf( "." );
         }
 
@@ -349,8 +375,19 @@ main( int     argc,
     }
 
     ctor = tr_ctorNew( h );
-    tr_ctorSetMetainfoFromFile( ctor, torrentPath );
+
+    fileContents = tr_loadFile( torrentPath, &fileLength );
     tr_ctorSetPaused( ctor, TR_FORCE, showScrape );
+    if( fileContents != NULL ) {
+        tr_ctorSetMetainfo( ctor, fileContents, fileLength );
+    } else if( !memcmp( torrentPath, "magnet:?", 8 ) ) {
+        tr_ctorSetMetainfoFromMagnetLink( ctor, torrentPath );
+    } else if( !memcmp( torrentPath, "http", 4 ) ) {
+        tr_webRun( h, torrentPath, NULL, onTorrentFileDownloaded, ctor );
+        waitingOnWeb = TRUE;
+        while( waitingOnWeb ) tr_wait_msec( 1000 );
+    }
+    tr_free( fileContents );
 
     if( showScrape )
     {
@@ -373,8 +410,9 @@ main( int     argc,
                                             strchr( scrape,
                                                     '?' ) ? '&' : '?',
                                             escaped );
-                    tr_httpParseURL( scrape, -1, &host, NULL, NULL );
+                    tr_urlParse( scrape, -1, NULL, &host, NULL, NULL );
                     ++leftToScrape;
+
                     tr_webRun( h, url, NULL, scrapeDoneFunc, host );
                     tr_free( url );
                 }
@@ -383,7 +421,7 @@ main( int     argc,
             fprintf( stderr, "scraping %d trackers:\n", leftToScrape );
 
             while( leftToScrape > 0 && ( ( time( NULL ) - start ) < 20 ) )
-                tr_wait( 250 );
+                tr_wait_msec( 250 );
         }
         goto cleanup;
     }
@@ -428,8 +466,11 @@ main( int     argc,
     {
         char            line[LINEWIDTH];
         const tr_stat * st;
+        const char * messageName[] = { NULL, "Tracker gave a warning:",
+                                             "Tracker gave an error:",
+                                             "Error:" };
 
-        tr_wait( 200 );
+        tr_wait_msec( 200 );
 
         if( gotsig )
         {
@@ -459,8 +500,9 @@ main( int     argc,
 
         getStatusStr( st, line, sizeof( line ) );
         printf( "\r%-*s", LINEWIDTH, line );
-        if( st->error )
-            fprintf( stderr, "\n%s\n", st->errorString );
+
+        if( messageName[st->error] )
+            fprintf( stderr, "\n%s: %s\n", messageName[st->error], st->errorString );
     }
 
 cleanup:
