@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: remote.c 10288 2010-02-25 23:06:05Z livings124 $
+ * $Id: remote.c 10582 2010-05-01 04:36:01Z charles $
  */
 
 #include <ctype.h> /* isspace */
@@ -1840,7 +1840,6 @@ tr_curl_easy_init( struct evbuffer * writebuf )
     curl_easy_setopt( curl, CURLOPT_POST, 1 );
     curl_easy_setopt( curl, CURLOPT_NETRC, CURL_NETRC_OPTIONAL );
     curl_easy_setopt( curl, CURLOPT_HTTPAUTH, CURLAUTH_ANY );
-    curl_easy_setopt( curl, CURLOPT_TIMEOUT, 60L );
     curl_easy_setopt( curl, CURLOPT_VERBOSE, debug );
     curl_easy_setopt( curl, CURLOPT_ENCODING, "" ); /* "" tells curl to fill in the blanks with what it was compiled to support */
     if( netrc )
@@ -1856,6 +1855,14 @@ tr_curl_easy_init( struct evbuffer * writebuf )
     return curl;
 }
 
+static long
+getTimeoutSecs( const char * req )
+{
+    if( strstr( req, "\"method\":\"blocklist-update\"" ) != NULL ) 
+        return 300L; 
+
+    return 60L; /* default value */ 
+} 
 
 static int
 processRequests( const char *  host,
@@ -1881,6 +1888,7 @@ processRequests( const char *  host,
         }
 
         curl_easy_setopt( curl, CURLOPT_POSTFIELDS, reqs[i] );
+        curl_easy_setopt( curl, CURLOPT_TIMEOUT, getTimeoutSecs( reqs[i] ) );
 
         if( debug )
             fprintf( stderr, "posting:\n--------\n%s\n--------\n", reqs[i] );
