@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: main.c 10292 2010-02-27 18:00:24Z charles $
+ * $Id: main.c 10598 2010-05-01 14:06:53Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -227,7 +227,7 @@ static gboolean       updatemodel( gpointer gdata );
 ****
 ***/
 
-#ifdef HAVE_LIBGCONF
+#ifdef HAVE_GCONF2
  #include <gconf/gconf.h>
  #include <gconf/gconf-client.h>
 #endif
@@ -235,7 +235,7 @@ static gboolean       updatemodel( gpointer gdata );
 static void
 registerMagnetLinkHandler( void )
 {
-#ifdef HAVE_LIBGCONF
+#ifdef HAVE_GCONF2
     GError * err;
     GConfValue * value;
     GConfClient * client = gconf_client_get_default( );
