@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,14 +7,14 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent-cell-renderer.c 8389 2009-05-13 15:54:04Z charles $
+ * $Id: torrent-cell-renderer.c 10289 2010-02-26 15:06:26Z charles $
  */
 
-#include "assert.h"
-#include <string.h> /* strcmp */
+#include <string.h> /* strcmp() */
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <libtransmission/transmission.h>
+#include <libtransmission/utils.h> /* tr_truncd() */
 #include "hig.h"
 #include "icons.h"
 #include "torrent-cell-renderer.h"
@@ -26,6 +26,8 @@
 enum
 {
     P_TORRENT = 1,
+    P_UPLOAD_SPEED,
+    P_DOWNLOAD_SPEED,
     P_BAR_HEIGHT,
     P_MINIMAL
 };
@@ -49,7 +51,7 @@ getProgressString( const tr_torrent * tor,
     const int      isSeed = torStat->haveValid >= info->totalSize;
     char           buf1[32], buf2[32], buf3[32], buf4[32];
     char *         str;
-    double         seedRatio; 
+    double         seedRatio;
     gboolean       hasSeedRatio = FALSE;
 
     if( !isDone )
@@ -61,7 +63,7 @@ getProgressString( const tr_torrent * tor,
             _( "%1$s of %2$s (%3$.2f%%)" ),
             tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
             tr_strlsize( buf2, torStat->sizeWhenDone, sizeof( buf2 ) ),
-            torStat->percentDone * 100.0 );
+            tr_truncd( torStat->percentDone * 100.0, 2 ) );
     }
     else if( !isSeed )
     {
@@ -74,7 +76,7 @@ getProgressString( const tr_torrent * tor,
             _( "%1$s of %2$s (%3$.2f%%), uploaded %4$s (Ratio: %5$s)" ),
             tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
             tr_strlsize( buf2, info->totalSize, sizeof( buf2 ) ),
-            torStat->percentComplete * 100.0,
+            tr_truncd( torStat->percentComplete * 100.0, 2 ),
             tr_strlsize( buf3, torStat->uploadedEver, sizeof( buf3 ) ),
             tr_strlratio( buf4, torStat->ratio, sizeof( buf4 ) ) );
     }
@@ -104,7 +106,8 @@ getProgressString( const tr_torrent * tor,
     }
 
     /* add time when downloading */
-    if( hasSeedRatio || ( torStat->activity == TR_STATUS_DOWNLOAD ) )
+    if( ( torStat->activity == TR_STATUS_DOWNLOAD )
+        || ( hasSeedRatio && ( torStat->activity == TR_STATUS_SEED ) ) )
     {
         const int eta = torStat->eta;
         GString * gstr = g_string_new( str );
@@ -126,40 +129,49 @@ getProgressString( const tr_torrent * tor,
 }
 
 static char*
-getShortTransferString( const tr_stat * torStat,
-                        char *          buf,
-                        size_t          buflen )
+getShortTransferString( const tr_torrent  * tor,
+                        const tr_stat     * torStat,
+                        double              uploadSpeed,
+                        double              downloadSpeed,
+                        char              * buf,
+                        size_t              buflen )
 {
-    char      downStr[32], upStr[32];
-    const int haveDown = torStat->peersSendingToUs > 0;
-    const int haveUp = torStat->peersGettingFromUs > 0;
+    char downStr[32], upStr[32];
+    const int haveMeta = tr_torrentHasMetadata( tor );
+    const int haveDown = haveMeta && torStat->peersSendingToUs > 0;
+    const int haveUp = haveMeta && torStat->peersGettingFromUs > 0;
 
     if( haveDown )
-        tr_strlspeed( downStr, torStat->pieceDownloadSpeed, sizeof( downStr ) );
+        tr_strlspeed( downStr, downloadSpeed, sizeof( downStr ) );
     if( haveUp )
-        tr_strlspeed( upStr, torStat->pieceUploadSpeed, sizeof( upStr ) );
+        tr_strlspeed( upStr, uploadSpeed, sizeof( upStr ) );
 
     if( haveDown && haveUp )
-        /* Translators: "speed|" is here for disambiguation.  Please remove it from your translation.
-           %1$s is the download speed
+        /* Translators: "speed|" is here for disambiguation.
+         * Please remove it from your translation.
+         * %1$s is the download speed
            %2$s is the upload speed */
-        g_snprintf( buf, buflen, Q_(
-                        "speed|Down: %1$s, Up: %2$s" ), downStr, upStr );
+        g_snprintf( buf, buflen, Q_( "speed|Down: %1$s, Up: %2$s" ), downStr, upStr );
     else if( haveDown )
         /* download speed */
         g_snprintf( buf, buflen, _( "Down: %s" ), downStr );
     else if( haveUp )
         /* upload speed */
         g_snprintf( buf, buflen, _( "Up: %s" ), upStr );
-    else
+    else if( tr_torrentHasMetadata( tor ) )
         /* the torrent isn't uploading or downloading */
         g_strlcpy( buf, _( "Idle" ), buflen );
+    else
+        *buf = '\0';
 
     return buf;
 }
 
 static char*
-getShortStatusString( const tr_stat * torStat )
+getShortStatusString( const tr_torrent  * tor,
+                      const tr_stat     * torStat,
+                      double              uploadSpeed,
+                      double              downloadSpeed )
 {
     GString * gstr = g_string_new( NULL );
 
@@ -175,22 +187,21 @@ getShortStatusString( const tr_stat * torStat )
 
         case TR_STATUS_CHECK:
             g_string_append_printf( gstr,
-                                    _(
-                                        "Verifying local data (%.1f%% tested)" ),
-                                    torStat->recheckProgress * 100.0 );
+                                    _( "Verifying local data (%.1f%% tested)" ),
+                                    tr_truncd( torStat->recheckProgress * 100.0, 1 ) );
             break;
 
         case TR_STATUS_DOWNLOAD:
         case TR_STATUS_SEED:
         {
-            char buf[128];
+            char buf[512];
             if( torStat->activity != TR_STATUS_DOWNLOAD )
             {
                 tr_strlratio( buf, torStat->ratio, sizeof( buf ) );
                 g_string_append_printf( gstr, _( "Ratio: %s" ), buf );
                 g_string_append( gstr, ", " );
             }
-            getShortTransferString( torStat, buf, sizeof( buf ) );
+            getShortTransferString( tor, torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
             g_string_append( gstr, buf );
             break;
         }
@@ -203,7 +214,10 @@ getShortStatusString( const tr_stat * torStat )
 }
 
 static char*
-getStatusString( const tr_stat * torStat )
+getStatusString( const tr_torrent  * tor,
+                 const tr_stat     * torStat,
+                 const double        uploadSpeed,
+                 const double        downloadSpeed )
 {
     const int isActive = torStat->activity != TR_STATUS_STOPPED;
     const int isChecking = torStat->activity == TR_STATUS_CHECK
@@ -213,21 +227,27 @@ getStatusString( const tr_stat * torStat )
 
     if( torStat->error )
     {
-        g_string_assign( gstr, torStat->errorString );
+        const char * fmt[] = { NULL, N_( "Tracker gave a warning: \"%s\"" ),
+                                     N_( "Tracker gave an error: \"%s\"" ),
+                                     N_( "Error: %s" ) };
+        g_string_append_printf( gstr, _( fmt[torStat->error] ), torStat->errorString );
     }
     else switch( torStat->activity )
+    {
+        case TR_STATUS_STOPPED:
+        case TR_STATUS_CHECK_WAIT:
+        case TR_STATUS_CHECK:
         {
-            case TR_STATUS_STOPPED:
-            case TR_STATUS_CHECK_WAIT:
-            case TR_STATUS_CHECK:
-            {
-                char * pch = getShortStatusString( torStat );
-                g_string_assign( gstr, pch );
-                g_free( pch );
-                break;
-            }
+            char * pch = getShortStatusString( tor, torStat, uploadSpeed, downloadSpeed );
+            g_string_assign( gstr, pch );
+            g_free( pch );
+            break;
+        }
 
-            case TR_STATUS_DOWNLOAD:
+        case TR_STATUS_DOWNLOAD:
+        {
+            if( tr_torrentHasMetadata( tor ) )
+            {
                 g_string_append_printf( gstr,
                     ngettext( "Downloading from %1$'d of %2$'d connected peer",
                               "Downloading from %1$'d of %2$'d connected peers",
@@ -236,23 +256,35 @@ getStatusString( const tr_stat * torStat )
                     torStat->webseedsSendingToUs,
                     torStat->peersConnected +
                     torStat->webseedsSendingToUs );
-                break;
-
-            case TR_STATUS_SEED:
+            }
+            else
+            {
                 g_string_append_printf( gstr,
-                    ngettext( "Seeding to %1$'d of %2$'d connected peer",
-                              "Seeding to %1$'d of %2$'d connected peers",
+                    ngettext( "Downloading metadata from %1$'d peer (%2$d%% done)",
+                              "Downloading metadata from %1$'d peers (%2$d%% done)",
                               torStat->peersConnected ),
-                    torStat->peersGettingFromUs,
-                    torStat->peersConnected );
-                break;
+                    torStat->peersConnected + torStat->webseedsSendingToUs,
+                    (int)(100.0*torStat->metadataPercentComplete) );
+            }
+            break;
         }
 
+        case TR_STATUS_SEED:
+            g_string_append_printf( gstr,
+                ngettext( "Seeding to %1$'d of %2$'d connected peer",
+                          "Seeding to %1$'d of %2$'d connected peers",
+                          torStat->peersConnected ),
+                torStat->peersGettingFromUs,
+                torStat->peersConnected );
+                break;
+    }
+
     if( isActive && !isChecking )
     {
         char buf[256];
-        getShortTransferString( torStat, buf, sizeof( buf ) );
-        g_string_append_printf( gstr, " - %s", buf );
+        getShortTransferString( tor, torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
+        if( *buf )
+            g_string_append_printf( gstr, " - %s", buf );
     }
 
     return g_string_free( gstr, FALSE );
@@ -271,8 +303,18 @@ struct TorrentCellRendererPrivate
     GtkCellRenderer  * text_renderer_err;
     GtkCellRenderer  * progress_renderer;
     GtkCellRenderer  * icon_renderer;
-    int                bar_height;
-    gboolean           minimal;
+    int bar_height;
+
+    /* use this instead of tr_stat.pieceUploadSpeed so that the model can
+       control when the speed displays get updated.  this is done to keep
+       the individual torrents' speeds and the status bar's overall speed
+       in sync even if they refresh at slightly different times */
+    double upload_speed;
+
+    /* @see upload_speed */
+    double download_speed;
+
+    gboolean minimal;
 };
 
 /***
@@ -285,7 +327,11 @@ get_icon( const tr_torrent * tor, GtkIconSize icon_size, GtkWidget * for_widget
     const char * mime_type;
     const tr_info * info = tr_torrentInfo( tor );
 
-    if( info->fileCount > 1 )
+    if( info->fileCount == 0  )
+        mime_type = UNKNOWN_MIME_TYPE;
+    else if( info->fileCount > 1 )
+        mime_type = DIRECTORY_MIME_TYPE;
+    else if( strchr( info->files[0].name, '/' ) != NULL )
         mime_type = DIRECTORY_MIME_TYPE;
     else
         mime_type = get_mime_type_from_filename( info->files[0].name );
@@ -324,7 +370,7 @@ get_size_minimal( TorrentCellRenderer * cell,
 
     icon = get_icon( tor, MINIMAL_ICON_SIZE, widget );
     name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( st );
+    status = getShortStatusString( tor, st, p->upload_speed, p->download_speed );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
@@ -340,7 +386,7 @@ get_size_minimal( TorrentCellRenderer * cell,
     gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
     stat_area.width = w;
     stat_area.height = h;
-    
+
     /**
     *** LAYOUT
     **/
@@ -381,7 +427,7 @@ get_size_full( TorrentCellRenderer * cell,
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
     name = inf->name;
-    status = getStatusString( st );
+    status = getStatusString( tor, st, p->upload_speed, p->download_speed );
     progress = getProgressString( tor, inf, st );
 
     /* get the idealized cell dimensions */
@@ -402,7 +448,7 @@ get_size_full( TorrentCellRenderer * cell,
     gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
     stat_area.width = w;
     stat_area.height = h;
-    
+
     /**
     *** LAYOUT
     **/
@@ -481,10 +527,11 @@ render_minimal( TorrentCellRenderer   * cell,
     const tr_stat * st = tr_torrentStatCached( (tr_torrent*)tor );
     const gboolean active = st->activity != TR_STATUS_STOPPED;
     const double percentDone = MAX( 0.0, st->percentDone );
+    const gboolean sensitive = active || st->error;
 
     icon = get_icon( tor, MINIMAL_ICON_SIZE, widget );
     name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( st );
+    status = getShortStatusString( tor, st, p->upload_speed, p->download_speed );
 
     /* get the cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
@@ -533,14 +580,14 @@ render_minimal( TorrentCellRenderer   * cell,
     /**
     *** RENDER
     **/
-   
-    g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", active, NULL );
+
+    g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, &icon_area, &icon_area, flags );
-    g_object_set( text_renderer, "text", status, "scale", SMALL_SCALE, "sensitive", active, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
+    g_object_set( text_renderer, "text", status, "scale", SMALL_SCALE, "sensitive", sensitive, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
     gtk_cell_renderer_render( text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
     g_object_set( text_renderer, "text", name, "scale", 1.0, NULL );
     gtk_cell_renderer_render( text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
-    g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", "", "sensitive", active, NULL );
+    g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", "", "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->progress_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
 
     /* cleanup */
@@ -576,10 +623,11 @@ render_full( TorrentCellRenderer   * cell,
     const tr_info * inf = tr_torrentInfo( tor );
     const gboolean active = st->activity != TR_STATUS_STOPPED;
     const double percentDone = MAX( 0.0, st->percentDone );
+    const gboolean sensitive = active || st->error;
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
     name = inf->name;
-    status = getStatusString( st );
+    status = getStatusString( tor, st, p->upload_speed, p->download_speed );
     progress = getProgressString( tor, inf, st );
 
     /* get the idealized cell dimensions */
@@ -600,7 +648,7 @@ render_full( TorrentCellRenderer   * cell,
     gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
     stat_area.width = w;
     stat_area.height = h;
-    
+
     /**
     *** LAYOUT
     **/
@@ -639,14 +687,14 @@ render_full( TorrentCellRenderer   * cell,
     /**
     *** RENDER
     **/
-   
-    g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", active, NULL );
+
+    g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, &icon_area, &icon_area, flags );
-    g_object_set( text_renderer, "text", name, "scale", 1.0, "sensitive", active, "ellipsize", PANGO_ELLIPSIZE_END, "weight", PANGO_WEIGHT_BOLD, NULL );
+    g_object_set( text_renderer, "text", name, "scale", 1.0, "sensitive", sensitive, "ellipsize", PANGO_ELLIPSIZE_END, "weight", PANGO_WEIGHT_BOLD, NULL );
     gtk_cell_renderer_render( text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
     g_object_set( text_renderer, "text", progress, "scale", SMALL_SCALE, "weight", PANGO_WEIGHT_NORMAL, NULL );
     gtk_cell_renderer_render( text_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
-    g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", "", "sensitive", active, NULL );
+    g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", "", "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->progress_renderer, window, widget, &prct_area, &prct_area, &prct_area, flags );
     g_object_set( text_renderer, "text", status, NULL );
     gtk_cell_renderer_render( text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
@@ -688,56 +736,42 @@ torrent_cell_renderer_render( GtkCellRenderer       * cell,
 }
 
 static void
-torrent_cell_renderer_set_property( GObject *      object,
+torrent_cell_renderer_set_property( GObject      * object,
                                     guint          property_id,
                                     const GValue * v,
-                                    GParamSpec *   pspec )
+                                    GParamSpec   * pspec )
 {
-    TorrentCellRenderer *               self = TORRENT_CELL_RENDERER(
-        object );
+    TorrentCellRenderer * self = TORRENT_CELL_RENDERER( object );
     struct TorrentCellRendererPrivate * p = self->priv;
 
     switch( property_id )
     {
-        case P_TORRENT:
-            p->tor = g_value_get_pointer( v ); break;
-
-        case P_BAR_HEIGHT:
-            p->bar_height = g_value_get_int( v ); break;
-
-        case P_MINIMAL:
-            p->minimal  = g_value_get_boolean( v ); break;
-
-        default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec );
-            break;
+        case P_TORRENT:        p->tor            = g_value_get_pointer( v ); break;
+        case P_UPLOAD_SPEED:   p->upload_speed   = g_value_get_double( v ); break;
+        case P_DOWNLOAD_SPEED: p->download_speed = g_value_get_double( v ); break;
+        case P_BAR_HEIGHT:     p->bar_height     = g_value_get_int( v ); break;
+        case P_MINIMAL:        p->minimal        = g_value_get_boolean( v ); break;
+        default: G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec ); break;
     }
 }
 
 static void
-torrent_cell_renderer_get_property( GObject *    object,
-                                    guint        property_id,
-                                    GValue *     v,
-                                    GParamSpec * pspec )
+torrent_cell_renderer_get_property( GObject     * object,
+                                    guint         property_id,
+                                    GValue      * v,
+                                    GParamSpec  * pspec )
 {
-    const TorrentCellRenderer *         self = TORRENT_CELL_RENDERER(
-        object );
+    const TorrentCellRenderer * self = TORRENT_CELL_RENDERER( object );
     struct TorrentCellRendererPrivate * p = self->priv;
 
     switch( property_id )
     {
-        case P_TORRENT:
-            g_value_set_pointer( v, p->tor ); break;
-
-        case P_BAR_HEIGHT:
-            g_value_set_int( v, p->bar_height ); break;
-
-        case P_MINIMAL:
-            g_value_set_boolean( v, p->minimal ); break;
-
-        default:
-            G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec );
-            break;
+        case P_TORRENT:        g_value_set_pointer( v, p->tor ); break;
+        case P_UPLOAD_SPEED:   g_value_set_double( v, p->upload_speed ); break;
+        case P_DOWNLOAD_SPEED: g_value_set_double( v, p->download_speed ); break;
+        case P_BAR_HEIGHT:     g_value_set_int( v, p->bar_height ); break;
+        case P_MINIMAL:        g_value_set_boolean( v, p->minimal ); break;
+        default: G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec ); break;
     }
 }
 
@@ -782,6 +816,18 @@ torrent_cell_renderer_class_init( TorrentCellRendererClass * klass )
                                                           "tr_torrent*",
                                                           G_PARAM_READWRITE ) );
 
+    g_object_class_install_property( gobject_class, P_UPLOAD_SPEED,
+                                    g_param_spec_double( "piece-upload-speed", NULL,
+                                                         "tr_stat.pieceUploadSpeed",
+                                                         0, INT_MAX, 0,
+                                                         G_PARAM_READWRITE ) );
+
+    g_object_class_install_property( gobject_class, P_DOWNLOAD_SPEED,
+                                    g_param_spec_double( "piece-download-speed", NULL,
+                                                         "tr_stat.pieceDownloadSpeed",
+                                                         0, INT_MAX, 0,
+                                                         G_PARAM_READWRITE ) );
+
     g_object_class_install_property( gobject_class, P_BAR_HEIGHT,
                                     g_param_spec_int( "bar-height", NULL,
                                                       "Bar Height",
