@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-torrent.c 8122 2009-04-02 17:30:29Z charles $
+ * $Id: tr-torrent.c 10453 2010-04-06 02:53:14Z charles $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -29,6 +29,7 @@
 #include <glib/gi18n.h>
 
 #include <libtransmission/transmission.h>
+#include <libtransmission/utils.h> /* tr_truncd() */
 
 #include "tr-prefs.h"
 #include "tr-torrent.h"
@@ -167,7 +168,7 @@ completenessChangedCallback( tr_torrent       * tor,
                              void *             user_data )
 {
     if( ( completeness != TR_LEECH ) && ( tr_torrentStat( tor )->sizeWhenDone != 0 ) )
-        g_idle_add( notifyInMainThread, user_data );
+        gtr_idle_add( notifyInMainThread, user_data );
 }
 
 static TrTorrent *
@@ -214,66 +215,6 @@ tr_torrent_new_ctor( tr_session   * session,
     return tor ? maketorrent( tor ) : NULL;
 }
 
-char *
-tr_torrent_status_str( TrTorrent * gtor )
-{
-    char *          top = NULL;
-
-    const tr_stat * st = tr_torrent_stat( gtor );
-
-    const int       tpeers = MAX ( st->peersConnected, 0 );
-    const int       upeers = MAX ( st->peersGettingFromUs, 0 );
-    const int       eta = st->eta;
-
-    switch( st->activity )
-    {
-        case TR_STATUS_CHECK_WAIT:
-            top =
-                g_strdup_printf( _(
-                                     "Waiting to verify local data (%.1f%% tested)" ),
-                                 st->recheckProgress * 100.0 );
-            break;
-
-        case TR_STATUS_CHECK:
-            top =
-                g_strdup_printf( _(
-                                     "Verifying local data (%.1f%% tested)" ),
-                                 st->recheckProgress * 100.0 );
-            break;
-
-        case TR_STATUS_DOWNLOAD:
-
-            if( eta < 0 )
-                top = g_strdup_printf( _( "Remaining time unknown" ) );
-            else
-            {
-                char timestr[128];
-                tr_strltime( timestr, eta, sizeof( timestr ) );
-                /* %s is # of minutes */
-                top = g_strdup_printf( _( "%1$s remaining" ), timestr );
-            }
-            break;
-
-        case TR_STATUS_SEED:
-            top = g_strdup_printf(
-                ngettext( "Seeding to %1$'d of %2$'d connected peer",
-                          "Seeding to %1$'d of %2$'d connected peers",
-                          tpeers ),
-                upeers, tpeers );
-            break;
-
-        case TR_STATUS_STOPPED:
-            top = g_strdup( _( "Stopped" ) );
-            break;
-
-        default:
-            top = g_strdup( "???" );
-            break;
-    }
-
-    return top;
-}
-
 void
 tr_torrent_set_remove_flag( TrTorrent * gtor,
                             gboolean    do_remove )
@@ -291,15 +232,11 @@ tr_torrent_delete_files( TrTorrent * gtor )
 void
 tr_torrent_open_folder( TrTorrent * gtor )
 {
-    tr_torrent *    tor = tr_torrent_handle( gtor );
+    const tr_torrent * tor =  tr_torrent_handle( gtor );
     const tr_info * info = tr_torrent_info( gtor );
-    char *          path = info->fileCount == 1
-                           ? g_build_filename( tr_torrentGetDownloadDir(
-                                                   tor ), NULL )
-                           : g_build_filename( tr_torrentGetDownloadDir(
-                                                   tor ), info->name, NULL );
-
+    const gboolean single = info->fileCount == 1;
+    char * path = single ? g_build_filename( tr_torrentGetCurrentDir( tor ), NULL )
+                         : g_build_filename( tr_torrentGetCurrentDir( tor ), info->name, NULL );
     gtr_open_file( path );
     g_free( path );
 }
-
