@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-torrent.h 6978 2008-10-28 19:49:33Z charles $
+ * $Id: tr-torrent.h 10453 2010-04-06 02:53:14Z charles $
  *
  * Copyright (c) 2006-2008 Transmission authors and contributors
  *
@@ -65,8 +65,6 @@ const tr_stat *tr_torrent_stat( TrTorrent *tor );
 
 const tr_info *tr_torrent_info( TrTorrent *tor );
 
-char*          tr_torrent_status_str( TrTorrent * tor );
-
 void           tr_torrent_delete_files( TrTorrent * tor );
 
 void           tr_torrent_open_folder( TrTorrent * tor );
@@ -75,7 +73,7 @@ TrTorrent *    tr_torrent_new_preexisting( tr_torrent * tor );
 
 TrTorrent *    tr_torrent_new_ctor( tr_session  * session,
                                     tr_ctor     * ctor,
-                                    char       ** err );
+                                    int         * errcode );
 
 void           tr_torrent_set_remove_flag( TrTorrent *,
                                            gboolean );
