@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-window.c 8673 2009-06-11 22:36:45Z charles $
+ * $Id: tr-window.c 10108 2010-02-06 05:21:25Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -27,12 +27,18 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
+#if !GTK_CHECK_VERSION( 2,16,0 )
+ /* FIXME: when 2.16 has been out long enough, it would be really nice to
+  * get rid of this libsexy usage because of its makefile strangeness */
+ #define USE_SEXY
+ #include "sexy-icon-entry.h"
+#endif
+
 #include <libtransmission/transmission.h>
 
 #include "actions.h"
 #include "conf.h"
 #include "hig.h"
-#include "sexy-icon-entry.h"
 #include "torrent-cell-renderer.h"
 #include "tr-prefs.h"
 #include "tr-torrent.h"
@@ -83,13 +89,11 @@ typedef struct
     GtkWidget *           filter;
     GtkWidget *           status;
     GtkWidget *           status_menu;
-    GtkWidget *           ul_hbox;
-    GtkWidget *           dl_hbox;
     GtkWidget *           ul_lb;
     GtkWidget *           dl_lb;
     GtkWidget *           stats_lb;
     GtkWidget *           gutter_lb;
-    GtkWidget *           alt_speed_image[2]; /* 0==off, 1==on */
+    GtkWidget *           alt_speed_image;
     GtkWidget *           alt_speed_button;
     GtkWidget *           options_menu;
     GtkTreeSelection *    selection;
@@ -181,7 +185,7 @@ makeview( PrivateData * p,
 
     p->selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
 
-    p->column = col = GTK_TREE_VIEW_COLUMN (g_object_new (GTK_TYPE_TREE_VIEW_COLUMN,		      
+    p->column = col = GTK_TREE_VIEW_COLUMN (g_object_new (GTK_TYPE_TREE_VIEW_COLUMN,
         "title", _("Torrent"),
         "resizable", TRUE,
         "sizing", GTK_TREE_VIEW_COLUMN_FIXED,
@@ -190,7 +194,9 @@ makeview( PrivateData * p,
     p->renderer = r = torrent_cell_renderer_new( );
     gtk_tree_view_column_pack_start( col, r, FALSE );
     gtk_tree_view_column_add_attribute( col, r, "torrent", MC_TORRENT_RAW );
-    
+    gtk_tree_view_column_add_attribute( col, r, "piece-upload-speed", MC_SPEED_UP );
+    gtk_tree_view_column_add_attribute( col, r, "piece-download-speed", MC_SPEED_DOWN );
+
     gtk_tree_view_append_column( GTK_TREE_VIEW( view ), col );
     g_object_set( r, "xpad", GUI_PAD_SMALL, "ypad", GUI_PAD_SMALL, NULL );
 
@@ -264,7 +270,9 @@ prefsChanged( TrCore * core UNUSED,
     {
         tr_window_update( (TrWindow*)wind );
     }
-    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_ENABLED ) )
+    else if( !strcmp( key, TR_PREFS_KEY_ALT_SPEED_ENABLED ) ||
+             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_UP ) ||
+             !strcmp( key, TR_PREFS_KEY_ALT_SPEED_DOWN ) )
     {
         syncAltSpeedButton( p );
     }
@@ -274,7 +282,6 @@ static void
 privateFree( gpointer vprivate )
 {
     PrivateData * p = vprivate;
-
     g_signal_handler_disconnect( p->core, p->pref_handler_id );
     g_free( p->filter_text );
     g_free( p );
@@ -318,18 +325,26 @@ status_menu_toggled_cb( GtkCheckMenuItem * menu_item,
 static void
 syncAltSpeedButton( PrivateData * p )
 {
-    const char * tip;
+    char u[32];
+    char d[32];
+    char * str;
+    const char * fmt;
     const gboolean b = pref_flag_get( TR_PREFS_KEY_ALT_SPEED_ENABLED );
+    const char * stock = b ? "alt-speed-on" : "alt-speed-off";
     GtkWidget * w = p->alt_speed_button;
 
+    tr_strlspeed( u, pref_int_get( TR_PREFS_KEY_ALT_SPEED_UP ), sizeof( u ) );
+    tr_strlspeed( d, pref_int_get( TR_PREFS_KEY_ALT_SPEED_DOWN ), sizeof( d ) );
+    fmt = b ? _( "Click to disable Temporary Speed Limits\n(%1$s down, %2$s up)" )
+            : _( "Click to enable Temporary Speed Limits\n(%1$s down, %2$s up)" );
+    str = g_strdup_printf( fmt, d, u );
+
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), b );
-
-    gtk_button_set_image( GTK_BUTTON( w ), p->alt_speed_image[b?1:0] );
+    gtk_image_set_from_stock( GTK_IMAGE( p->alt_speed_image ), stock, -1 );
     gtk_button_set_alignment( GTK_BUTTON( w ), 0.5, 0.5 );
+    gtr_widget_set_tooltip_text( w, str );
 
-    tip = b ? _( "Click to disable Speed Limit Mode" )
-            : _( "Click to enable Speed Limit Mode" );
-    gtr_widget_set_tooltip_text( w, tip );
+    g_free( str );
 }
 
 static void
@@ -339,14 +354,14 @@ alt_speed_toggled_cb( GtkToggleButton * button, gpointer vprivate )
     const gboolean b = gtk_toggle_button_get_active( button );
     tr_core_set_pref_bool( p->core, TR_PREFS_KEY_ALT_SPEED_ENABLED,  b );
 }
-    
+
 /***
 ****  FILTER
 ***/
 
 static int
 checkFilterText( filter_text_mode_t    filter_text_mode,
-                 const tr_info       * torInfo,
+                 const tr_info       * inf,
                  const char          * text )
 {
     tr_file_index_t i;
@@ -356,24 +371,31 @@ checkFilterText( filter_text_mode_t    filter_text_mode,
     switch( filter_text_mode )
     {
         case FILTER_TEXT_MODE_FILES:
-            for( i = 0; i < torInfo->fileCount && !ret; ++i )
+            for( i = 0; i < inf->fileCount && !ret; ++i )
             {
-                pch = g_utf8_casefold( torInfo->files[i].name, -1 );
+                pch = g_utf8_casefold( inf->files[i].name, -1 );
                 ret = !text || strstr( pch, text ) != NULL;
                 g_free( pch );
             }
             break;
 
         case FILTER_TEXT_MODE_TRACKER:
-            pch = g_utf8_casefold( torInfo->trackers[0].announce, -1 );
-            ret = !text || ( strstr( pch, text ) != NULL );
-            g_free( pch );
+            if( inf->trackerCount > 0 )
+            {
+                pch = g_utf8_casefold( inf->trackers[0].announce, -1 );
+                ret = !text || ( strstr( pch, text ) != NULL );
+                g_free( pch );
+            }
             break;
 
         default: /* NAME */
-            pch = g_utf8_casefold( torInfo->name, -1 );
-            ret = !text || ( strstr( pch, text ) != NULL );
-            g_free( pch );
+            if( !inf->name )
+                ret = TRUE;
+            else {
+                pch = g_utf8_casefold( inf->name, -1 );
+                ret = !text || ( strstr( pch, text ) != NULL );
+                g_free( pch );
+            }
             break;
     }
 
@@ -473,7 +495,7 @@ setFilter( PrivateData * p, int mode )
             gtk_toggle_button_set_active( p->filter_toggles[i], i==mode );
     }
 }
-  
+
 
 static void
 filter_toggled_cb( GtkToggleButton * toggle, gpointer vprivate )
@@ -500,17 +522,33 @@ filter_entry_changed( GtkEditable * e,
     g_free( pch );
 }
 
+
+#ifdef USE_SEXY
 static void
-entry_icon_released( SexyIconEntry         * entry  UNUSED,
-                     SexyIconEntryPosition          icon_pos,
-                     int                     button UNUSED,
-                     gpointer                       menu )
+entry_icon_released( SexyIconEntry           * entry  UNUSED,
+                     SexyIconEntryPosition     icon_pos,
+                     int                       button UNUSED,
+                     gpointer                  menu )
 {
     if( icon_pos == SEXY_ICON_ENTRY_PRIMARY )
-        gtk_menu_popup ( GTK_MENU(
-                            menu ), NULL, NULL, NULL, NULL, 0,
+        gtk_menu_popup( GTK_MENU( menu ), NULL, NULL, NULL, NULL, 0,
                         gtk_get_current_event_time( ) );
 }
+#else
+static void
+entry_icon_release( GtkEntry              * entry  UNUSED,
+                    GtkEntryIconPosition    icon_pos,
+                    GdkEventButton        * event  UNUSED,
+                    gpointer                menu )
+{
+    if( icon_pos == GTK_ENTRY_ICON_SECONDARY )
+        gtk_entry_set_text( entry, "" );
+
+    if( icon_pos == GTK_ENTRY_ICON_PRIMARY )
+        gtk_menu_popup( GTK_MENU( menu ), NULL, NULL, NULL, NULL, 0,
+                        gtk_get_current_event_time( ) );
+}
+#endif
 
 #if GTK_CHECK_VERSION( 2, 12, 0 )
 
@@ -550,7 +588,7 @@ onAskTrackerQueryTooltip( GtkWidget *            widget UNUSED,
     }
     else
     {
-        char      buf[128];
+        char      buf[512];
         char      timebuf[64];
         const int seconds = maxTime - now;
 
@@ -577,7 +615,7 @@ onAltSpeedToggledIdle( gpointer vp )
 static void
 onAltSpeedToggled( tr_session * s UNUSED, tr_bool isEnabled UNUSED, tr_bool byUser UNUSED, void * p )
 {
-    g_idle_add( onAltSpeedToggledIdle, p );
+    gtr_idle_add( onAltSpeedToggledIdle, p );
 }
 
 /***
@@ -589,7 +627,7 @@ onAltSpeedToggled( tr_session * s UNUSED, tr_bool isEnabled UNUSED, tr_bool byUs
 #define SPEED_KEY "speed-key"
 
 static void
-onSpeedToggled( GtkCheckMenuItem * check, gpointer vp ) 
+onSpeedToggled( GtkCheckMenuItem * check, gpointer vp )
 {
     PrivateData * p = vp;
     GObject * o = G_OBJECT( check );
@@ -603,7 +641,7 @@ onSpeedToggled( GtkCheckMenuItem * check, gpointer vp )
 }
 
 static void
-onSpeedSet( GtkCheckMenuItem * check, gpointer vp ) 
+onSpeedSet( GtkCheckMenuItem * check, gpointer vp )
 {
     const char * key;
     PrivateData * p = vp;
@@ -667,7 +705,7 @@ createSpeedMenu( PrivateData * p, tr_direction dir )
 static const double stockRatios[] = { 0.25, 0.5, 0.75, 1, 1.5, 2, 3 };
 
 static void
-onRatioToggled( GtkCheckMenuItem * check, gpointer vp ) 
+onRatioToggled( GtkCheckMenuItem * check, gpointer vp )
 {
     PrivateData * p = vp;
     if( gtk_check_menu_item_get_active( check ) )
@@ -677,7 +715,7 @@ onRatioToggled( GtkCheckMenuItem * check, gpointer vp )
     }
 }
 static void
-onRatioSet( GtkCheckMenuItem * check, gpointer vp ) 
+onRatioSet( GtkCheckMenuItem * check, gpointer vp )
 {
     PrivateData * p = vp;
     int i = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( check ), RATIO_KEY ) );
@@ -794,12 +832,12 @@ GtkWidget *
 tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 {
     int           i, n;
-    const char *  pch;
+    const char  * pch;
     PrivateData * p;
-    GtkWidget   *mainmenu, *toolbar, *filter, *list, *status;
-    GtkWidget *   vbox, *w, *self, *h, *s, *hbox, *image, *menu;
-    GtkWindow *   win;
-    GSList *      l;
+    GtkWidget   * mainmenu, *toolbar, *filter, *list, *status;
+    GtkWidget   * vbox, *w, *self, *h, *s, *hbox, *menu;
+    GtkWindow   * win;
+    GSList      * l;
 
     const char *  filter_names[FILTER_MODE_QTY] = {
         /* show all torrents */
@@ -842,7 +880,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
     gtk_container_add ( GTK_CONTAINER( self ), vbox );
 
     /* main menu */
-    w = mainmenu = action_get_widget( "/main-window-menu" );
+    mainmenu = action_get_widget( "/main-window-menu" );
     w = action_get_widget( "/main-window-menu/torrent-menu/update-tracker" );
 #if GTK_CHECK_VERSION( 2, 12, 0 )
     g_signal_connect( w, "query-tooltip",
@@ -850,7 +888,9 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 #endif
 
     /* toolbar */
-    w = toolbar = p->toolbar = action_get_widget( "/main-window-toolbar" );
+    toolbar = p->toolbar = action_get_widget( "/main-window-toolbar" );
+    action_set_important( "add-torrent-toolbar", TRUE );
+    action_set_important( "show-torrent-properties", TRUE );
 
     /* filter */
     h = filter = p->filter = gtk_hbox_new( FALSE, 0 );
@@ -867,15 +907,25 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
         gtk_box_pack_start( GTK_BOX( h ), w, FALSE, FALSE, 0 );
     }
 
+#ifdef USE_SEXY
     s = sexy_icon_entry_new( );
     sexy_icon_entry_add_clear_button( SEXY_ICON_ENTRY( s ) );
-    image = gtk_image_new_from_stock( GTK_STOCK_FIND, GTK_ICON_SIZE_MENU );
-    sexy_icon_entry_set_icon( SEXY_ICON_ENTRY(
-                                 s ), SEXY_ICON_ENTRY_PRIMARY,
-                             GTK_IMAGE( image ) );
-    sexy_icon_entry_set_icon_highlight( SEXY_ICON_ENTRY(
-                                            s ), SEXY_ICON_ENTRY_PRIMARY,
-                                        TRUE );
+    w = gtk_image_new_from_stock( GTK_STOCK_FIND, GTK_ICON_SIZE_MENU );
+    sexy_icon_entry_set_icon( SEXY_ICON_ENTRY( s ),
+                              SEXY_ICON_ENTRY_PRIMARY,
+                              GTK_IMAGE( w ) );
+    g_object_unref( w );
+    sexy_icon_entry_set_icon_highlight( SEXY_ICON_ENTRY( s ),
+                                        SEXY_ICON_ENTRY_PRIMARY, TRUE );
+#else
+    s = gtk_entry_new( );
+    gtk_entry_set_icon_from_stock( GTK_ENTRY( s ),
+                                   GTK_ENTRY_ICON_PRIMARY,
+                                   GTK_STOCK_FIND);
+   gtk_entry_set_icon_from_stock( GTK_ENTRY( s ),
+                                  GTK_ENTRY_ICON_SECONDARY,
+                                  GTK_STOCK_CLEAR );
+#endif
     gtk_box_pack_end( GTK_BOX( h ), s, FALSE, FALSE, 0 );
     g_signal_connect( s, "changed", G_CALLBACK( filter_entry_changed ), p );
 
@@ -905,41 +955,40 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 
         w = gtk_button_new( );
         gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( "options", GTK_ICON_SIZE_SMALL_TOOLBAR ) );
+        gtr_widget_set_tooltip_text( w, _( "Options" ) );
         gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
         gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
         p->options_menu = createOptionsMenu( p );
         g_signal_connect( w, "clicked", G_CALLBACK(onOptionsClicked), p );
 
-        p->alt_speed_image[0] = gtk_image_new_from_stock( "alt-speed-off", -1 );
-        p->alt_speed_image[1]  = gtk_image_new_from_stock( "alt-speed-on", -1 );
+        p->alt_speed_image = gtk_image_new( );
         w = p->alt_speed_button = gtk_toggle_button_new( );
+        gtk_button_set_image( GTK_BUTTON( w ), p->alt_speed_image );
         gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
-        g_object_ref( G_OBJECT( p->alt_speed_image[0] ) );
-        g_object_ref( G_OBJECT( p->alt_speed_image[1] ) );
         g_signal_connect( w, "toggled", G_CALLBACK(alt_speed_toggled_cb ), p );
         gtk_box_pack_start( GTK_BOX( h ), w, 0, 0, 0 );
 
         w = p->gutter_lb = gtk_label_new( "N Torrents" );
         gtk_box_pack_start( GTK_BOX( h ), w, 1, 1, GUI_PAD_BIG );
 
-        hbox = p->ul_hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
+        hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
             w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
             gtk_widget_set_size_request( w, GUI_PAD, 0u );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = gtk_image_new_from_stock( GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->ul_lb = gtk_label_new( NULL );
-            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 ); 
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
         gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
 
-        hbox = p->dl_hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
+        hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
             w = gtk_alignment_new( 0.0f, 0.0f, 0.0f, 0.0f );
             gtk_widget_set_size_request( w, GUI_PAD, 0u );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = gtk_image_new_from_stock( GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = p->dl_lb = gtk_label_new( NULL );
-            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 ); 
+            gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
         gtk_box_pack_end( GTK_BOX( h ), hbox, FALSE, FALSE, 0 );
 
         hbox = gtk_hbox_new( FALSE, GUI_PAD_SMALL );
@@ -947,7 +996,8 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
             gtk_widget_set_size_request( w, GUI_PAD, 0u );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
             w = gtk_button_new( );
-            gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU ) );
+            gtr_widget_set_tooltip_text( w, _( "Statistics" ) );
+            gtk_container_add( GTK_CONTAINER( w ), gtk_image_new_from_stock( "ratio", GTK_ICON_SIZE_SMALL_TOOLBAR ) );
             gtk_button_set_relief( GTK_BUTTON( w ), GTK_RELIEF_NONE );
             g_signal_connect( w, "clicked", G_CALLBACK( onYinYangReleased ), p );
             gtk_box_pack_start( GTK_BOX( hbox ), w, FALSE, FALSE, 0 );
@@ -958,7 +1008,7 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
 
     menu = gtk_menu_new( );
     l = NULL;
-    for( i = 0; i < FILTER_TEXT_MODE_QTY; ++i )
+    for( i=0; i<FILTER_TEXT_MODE_QTY; ++i )
     {
         const char * name = _( filter_text_names[i] );
         GtkWidget *  w = gtk_radio_menu_item_new_with_label ( l, name );
@@ -970,8 +1020,13 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
         gtk_menu_shell_append( GTK_MENU_SHELL( menu ), w );
         gtk_widget_show( w );
     }
-    g_signal_connect( s, "icon-released",
-                      G_CALLBACK( entry_icon_released ), menu );
+
+#ifdef USE_SEXY
+    g_signal_connect( s, "icon-released", G_CALLBACK( entry_icon_released ), menu );
+#else
+    g_signal_connect( s, "icon-release", G_CALLBACK( entry_icon_release ), menu );
+
+#endif
 
     /* workarea */
     p->view = makeview( p, core );
@@ -1004,6 +1059,19 @@ tr_window_new( GtkUIManager * ui_mgr, TrCore * core )
         g_strfreev( tokens );
     }
 
+    {
+        int w=0, h=0;
+        /* this is to determine the maximum width/height for the label */
+        PangoLayout * pango_layout =
+            gtk_widget_create_pango_layout( p->ul_lb, _( "999.9 KB/s" ) );
+        pango_layout_get_pixel_size( pango_layout, &w, &h );
+        gtk_widget_set_size_request( p->ul_lb, w, h );
+        gtk_widget_set_size_request( p->dl_lb, w, h );
+        gtk_misc_set_alignment( GTK_MISC( p->ul_lb ), 1.0, 0.5 );
+        gtk_misc_set_alignment( GTK_MISC( p->dl_lb ), 1.0, 0.5 );
+        g_object_unref( G_OBJECT( pango_layout ) );
+    }
+
     /* show all but the window */
     gtk_widget_show_all( vbox );
 
@@ -1030,7 +1098,7 @@ updateTorrentCount( PrivateData * p )
 {
     if( p && p->core )
     {
-        char      buf[128];
+        char      buf[512];
         const int torrentCount = gtk_tree_model_iter_n_children(
             tr_core_model( p->core ), NULL );
         const int visibleCount = gtk_tree_model_iter_n_children(
@@ -1056,7 +1124,7 @@ static void
 updateStats( PrivateData * p )
 {
     const char *            pch;
-    char                    up[32], down[32], ratio[32], buf[128];
+    char                    up[32], down[32], ratio[32], buf[512];
     struct tr_session_stats stats;
     tr_session *            session = tr_core_session( p->core );
 
@@ -1107,17 +1175,26 @@ updateSpeeds( PrivateData * p )
     if( session != NULL )
     {
         char buf[128];
-        double d;
+        double up=0, down=0;
+        GtkTreeIter iter;
+        GtkTreeModel * model = tr_core_model( p->core );
 
-        d = tr_sessionGetPieceSpeed( session, TR_DOWN );
-        tr_strlspeed( buf, d, sizeof( buf ) );
+        if( gtk_tree_model_get_iter_first( model, &iter ) ) do
+        {
+            double u, d;
+            gtk_tree_model_get( model, &iter, MC_SPEED_UP, &u,
+                                              MC_SPEED_DOWN, &d,
+                                              -1 );
+            up += u;
+            down += d;
+        }
+        while( gtk_tree_model_iter_next( model, &iter ) );
+
+        tr_strlspeed( buf, down, sizeof( buf ) );
         gtk_label_set_text( GTK_LABEL( p->dl_lb ), buf );
-        g_object_set( p->dl_hbox, "visible", d>=0.01, NULL );
 
-        d = tr_sessionGetPieceSpeed( session, TR_UP );
-        tr_strlspeed( buf, d, sizeof( buf ) );
+        tr_strlspeed( buf, up, sizeof( buf ) );
         gtk_label_set_text( GTK_LABEL( p->ul_lb ), buf );
-        g_object_set( p->ul_hbox, "visible", d>=0.01, NULL );
     }
 }
 
