@@ -1,3 +1,7 @@
+#ifndef __TRANSMISSION__
+#error only libtransmission should #include this header.
+#endif
+
 #ifndef JSON_PARSER_H
 #define JSON_PARSER_H
 
@@ -18,15 +22,9 @@
 #endif
 
 /* Determine the integer type use to parse non-floating point numbers */
-#if __STDC_VERSION__ >= 199901L || HAVE_LONG_LONG == 1
-typedef long long JSON_int_t;
-#define JSON_PARSER_INTEGER_SSCANF_TOKEN "%lld"
-#define JSON_PARSER_INTEGER_SPRINTF_TOKEN "%lld"
-#else 
-typedef long JSON_int_t;
-#define JSON_PARSER_INTEGER_SSCANF_TOKEN "%ld"
-#define JSON_PARSER_INTEGER_SPRINTF_TOKEN "%ld"
-#endif
+#include <stdarg.h> /* some 1.4.x versions of evutil.h need this */
+#include <evutil.h>
+typedef ev_int64_t JSON_int_t;
 
 
 #ifdef __cplusplus
