@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bandwidth.h 8709 2009-06-16 19:31:17Z charles $
+ * $Id: bandwidth.h 9868 2010-01-04 21:00:47Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -76,7 +76,7 @@ struct tr_band
  *   tr_session also owns a tr_handshake's bandwidths, so that the handshake
  *   I/O can be counted in the global raw totals.  When the handshake is done,
  *   the bandwidth's ownership passes to a tr_peer.
- * 
+ *
  * MEASURING
  *
  *   When you ask a bandwidth object for its speed, it gives the speed of the
@@ -85,17 +85,17 @@ struct tr_band
  *   tr_torrent's bandwidth, and per-peer speeds by asking tr_peer's bandwidth.
  *
  * CONSTRAINING
- * 
+ *
  *   Call tr_bandwidthAllocate() periodically.  tr_bandwidth knows its current
  *   speed and will decide how many bytes to make available over the
  *   user-specified period to reach the user-specified desired speed.
  *   If appropriate, it notifies its peer-ios that new bandwidth is available.
- * 
- *   tr_bandwidthAllocate() operates on the tr_bandwidth subtree, so usually 
+ *
+ *   tr_bandwidthAllocate() operates on the tr_bandwidth subtree, so usually
  *   you'll only need to invoke it for the top-level tr_session bandwidth.
  *
  *   The peer-ios all have a pointer to their associated tr_bandwidth object,
- *   and call tr_bandwidthClamp() before performing I/O to see how much 
+ *   and call tr_bandwidthClamp() before performing I/O to see how much
  *   bandwidth they can safely use.
  */
 typedef struct tr_bandwidth
@@ -123,7 +123,7 @@ tr_bandwidth* tr_bandwidthConstruct( tr_bandwidth * bandwidth,
                                      tr_bandwidth * parent );
 
 /** @brief create a new tr_bandwidth object */
-static TR_INLINE tr_bandwidth* tr_bandwidthNew( tr_session * session, tr_bandwidth * parent )
+static inline tr_bandwidth* tr_bandwidthNew( tr_session * session, tr_bandwidth * parent )
 {
     return tr_bandwidthConstruct( tr_new0( tr_bandwidth, 1 ), session, parent );
 }
@@ -131,13 +131,13 @@ static TR_INLINE tr_bandwidth* tr_bandwidthNew( tr_session * session, tr_bandwid
 tr_bandwidth* tr_bandwidthDestruct( tr_bandwidth * bandwidth );
 
 /** @brief free a tr_bandwidth object */
-static TR_INLINE void tr_bandwidthFree( tr_bandwidth * bandwidth )
+static inline void tr_bandwidthFree( tr_bandwidth * bandwidth )
 {
     tr_free( tr_bandwidthDestruct( bandwidth ) );
 }
 
 /** @brief test to see if the pointer refers to a live bandwidth object */
-static TR_INLINE tr_bool tr_isBandwidth( const tr_bandwidth  * b )
+static inline tr_bool tr_isBandwidth( const tr_bandwidth  * b )
 {
     return ( b != NULL ) && ( b->magicNumber == MAGIC_NUMBER );
 }
@@ -151,18 +151,21 @@ static TR_INLINE tr_bool tr_isBandwidth( const tr_bandwidth  * b )
  * @see tr_bandwidthAllocate
  * @see tr_bandwidthGetDesiredSpeed
  */
-static TR_INLINE void tr_bandwidthSetDesiredSpeed( tr_bandwidth        * bandwidth,
-                                                tr_direction          dir,
-                                                double                desiredSpeed )
+static inline tr_bool tr_bandwidthSetDesiredSpeed( tr_bandwidth        * bandwidth,
+                                                      tr_direction          dir,
+                                                      double                desiredSpeed )
 {
-    bandwidth->band[dir].desiredSpeed = desiredSpeed;
+    double * value = &bandwidth->band[dir].desiredSpeed;
+    const tr_bool didChange = (int)(desiredSpeed*1024.0) != (int)(*value*1024.0);
+    *value = desiredSpeed;
+    return didChange;
 }
 
 /**
  * @brief Get the desired speed (in KiB/s) for ths bandwidth subtree.
  * @see tr_bandwidthSetDesiredSpeed
  */
-static TR_INLINE double
+static inline double
 tr_bandwidthGetDesiredSpeed( const tr_bandwidth  * bandwidth,
                              tr_direction          dir )
 {
@@ -172,17 +175,20 @@ tr_bandwidthGetDesiredSpeed( const tr_bandwidth  * bandwidth,
 /**
  * @brief Set whether or not this bandwidth should throttle its peer-io's speeds
  */
-static TR_INLINE void tr_bandwidthSetLimited( tr_bandwidth        * bandwidth,
-                                              tr_direction          dir,
-                                              tr_bool               isLimited )
+static inline tr_bool tr_bandwidthSetLimited( tr_bandwidth        * bandwidth,
+                                                 tr_direction          dir,
+                                                 tr_bool               isLimited )
 {
-    bandwidth->band[dir].isLimited = isLimited;
+    tr_bool * value = &bandwidth->band[dir].isLimited;
+    const tr_bool didChange = isLimited != *value;
+    *value = isLimited;
+    return didChange;
 }
 
 /**
  * @return nonzero if this bandwidth throttles its peer-ios speeds
  */
-static TR_INLINE tr_bool tr_bandwidthIsLimited( const tr_bandwidth  * bandwidth,
+static inline tr_bool tr_bandwidthIsLimited( const tr_bandwidth  * bandwidth,
                                                 tr_direction          dir )
 {
     return bandwidth->band[dir].isLimited;
@@ -238,17 +244,17 @@ void    tr_bandwidthSetParent         ( tr_bandwidth        * bandwidth,
  * But when we set a torrent's speed mode to TR_SPEEDLIMIT_UNLIMITED, then
  * in that particular case we want to ignore the global speed limit...
  */
-static TR_INLINE void tr_bandwidthHonorParentLimits ( tr_bandwidth        * bandwidth,
-                                                      tr_direction          direction,
-                                                      tr_bool               isEnabled )
+static inline tr_bool tr_bandwidthHonorParentLimits ( tr_bandwidth        * bandwidth,
+                                                         tr_direction          direction,
+                                                         tr_bool               isEnabled )
 {
-    assert( tr_isBandwidth( bandwidth ) );
-    assert( tr_isDirection( direction ) );
-
-    bandwidth->band[direction].honorParentLimits = isEnabled;
+    tr_bool * value = &bandwidth->band[direction].honorParentLimits;
+    const tr_bool didChange = isEnabled != *value;
+    *value = isEnabled;
+    return didChange;
 }
 
-static TR_INLINE tr_bool tr_bandwidthAreParentLimitsHonored( tr_bandwidth  * bandwidth,
+static inline tr_bool tr_bandwidthAreParentLimitsHonored( tr_bandwidth  * bandwidth,
                                                              tr_direction    direction )
 {
     assert( tr_isBandwidth( bandwidth ) );
