@@ -1,18 +1,18 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
- * so that the bulk of its code can remain under the MIT license. 
+ * so that the bulk of its code can remain under the MIT license.
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: clients.c 8599 2009-06-03 15:47:08Z charles $
+ * $Id: clients.c 10016 2010-01-25 05:06:54Z livings124 $
  */
 
 /* thanks amc1! */
 
-#include <ctype.h> /* isprint, tolower */
+#include <ctype.h> /* isprint */
 #include <stdio.h>
 #include <stdlib.h> /* strtol */
 #include <string.h>
@@ -32,16 +32,16 @@ charint( char ch )
     return 0;
 }
 
-static int 
-getShadowInt( char ch, int * setme ) 
-{ 
-    const char * str = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.-"; 
-    const char * pch = strchr( str, ch ); 
-    if( !pch ) 
-    return 0; 
-        *setme = pch - str; 
-    return 1; 
-} 
+static int
+getShadowInt( char ch, int * setme )
+{
+    const char * str = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.-";
+    const char * pch = strchr( str, ch );
+    if( !pch )
+        return 0;
+    *setme = pch - str;
+    return 1;
+}
 
 static int
 strint( const void * pch, int span )
@@ -110,7 +110,7 @@ isMainlineStyle( const uint8_t * peer_id )
      * One of the following styles will be used:
      *   Mx-y-z--
      *   Mx-yy-z-
-     */ 
+     */
     return peer_id[2]=='-'
         && peer_id[7]=='-'
         && ( peer_id[4]=='-' || peer_id[5]=='-' );
@@ -146,18 +146,6 @@ decodeBitCometClient( char * buf, size_t buflen, const uint8_t * id )
     return TRUE;
 }
 
-static int
-decodeBitSpiritClient( char * buf, size_t buflen, const uint8_t * id )
-{
-    const int isBS = !memcmp( id+2, "BS", 2 );
-    if( isBS )
-    {
-        const int version = id[1] ? id[1] : 1;
-        tr_snprintf( buf, buflen, "BitSpirit v%d", version );
-    }
-    return isBS;
-}
-
 void
 tr_clientForId( char * buf, size_t buflen, const void * id_in )
 {
@@ -181,7 +169,7 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
                 tr_snprintf( buf, buflen, "Transmission %d.%02d%s", strint(id+3,1), strint(id+4,2),
                           id[6]=='Z' || id[6]=='X' ? "+" : "" );
         }
-        
+
         else if( !memcmp( id+1, "UT", 2 ) )
         {
             tr_snprintf( buf, buflen, "\xc2\xb5Torrent %d.%d.%d%s",
@@ -192,7 +180,7 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
             tr_snprintf( buf, buflen, "\xc2\xb5Torrent Mac %d.%d.%d%s",
                          strint(id+3,1), strint(id+4,1), strint(id+5,1), getMnemonicEnd(id[6]) );
         }
-        
+
         else if( !memcmp( id+1, "AZ", 2 ) )
         {
             if( id[3] > '3' || ( id[3] == '3' && id[4] >= '1' ) ) /* Vuze starts at version 3.1.0.0 */
@@ -200,7 +188,7 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
             else
                 four_digits( buf, buflen, "Azureus", id+3 );
         }
-        
+
         else if( !memcmp( id+1, "KT", 2 ) )
         {
             if( id[5] == 'D' )
@@ -214,6 +202,7 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
         else if( !memcmp( id+1, "AR", 2 ) ) four_digits( buf, buflen, "Ares", id+3 );
         else if( !memcmp( id+1, "AT", 2 ) ) four_digits( buf, buflen, "Artemis", id+3 );
         else if( !memcmp( id+1, "AV", 2 ) ) four_digits( buf, buflen, "Avicora", id+3 );
+        else if( !memcmp( id+1, "BE", 2 ) ) four_digits( buf, buflen, "BitTorrent SDK", id+3 );
         else if( !memcmp( id+1, "BG", 2 ) ) four_digits( buf, buflen, "BTGetit", id+3 );
         else if( !memcmp( id+1, "BM", 2 ) ) four_digits( buf, buflen, "BitMagnet", id+3 );
         else if( !memcmp( id+1, "BP", 2 ) ) four_digits( buf, buflen, "BitTorrent Pro (Azureus + Spyware)", id+3 );
@@ -229,19 +218,21 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
         else if( !memcmp( id+1, "FT", 2 ) ) four_digits( buf, buflen, "FoxTorrent/RedSwoosh", id+3 );
         else if( !memcmp( id+1, "GR", 2 ) ) four_digits( buf, buflen, "GetRight", id+3 );
         else if( !memcmp( id+1, "HN", 2 ) ) four_digits( buf, buflen, "Hydranode", id+3 );
+        else if( !memcmp( id+1, "KG", 2 ) ) four_digits( buf, buflen, "KGet", id+3 );
         else if( !memcmp( id+1, "LC", 2 ) ) four_digits( buf, buflen, "LeechCraft", id+3 );
         else if( !memcmp( id+1, "LH", 2 ) ) four_digits( buf, buflen, "LH-ABC", id+3 );
         else if( !memcmp( id+1, "NX", 2 ) ) four_digits( buf, buflen, "Net Transport", id+3 );
         else if( !memcmp( id+1, "MO", 2 ) ) four_digits( buf, buflen, "MonoTorrent", id+3 );
         else if( !memcmp( id+1, "MR", 2 ) ) four_digits( buf, buflen, "Miro", id+3 );
         else if( !memcmp( id+1, "MT", 2 ) ) four_digits( buf, buflen, "Moonlight", id+3 );
+        else if( !memcmp( id+1, "OS", 2 ) ) four_digits( buf, buflen, "OneSwarm", id+3 );
         else if( !memcmp( id+1, "OT", 2 ) ) four_digits( buf, buflen, "OmegaTorrent", id+3 );
         else if( !memcmp( id+1, "PD", 2 ) ) four_digits( buf, buflen, "Pando", id+3 );
         else if( !memcmp( id+1, "QD", 2 ) ) four_digits( buf, buflen, "QQDownload", id+3 );
         else if( !memcmp( id+1, "RS", 2 ) ) four_digits( buf, buflen, "Rufus", id+3 );
         else if( !memcmp( id+1, "RT", 2 ) ) four_digits( buf, buflen, "Retriever", id+3 );
         else if( !memcmp( id+1, "RZ", 2 ) ) four_digits( buf, buflen, "RezTorrent", id+3 );
-        else if( !memcmp( id+1, "SD", 2 ) ) four_digits( buf, buflen, "Xunlei", id+3 );
+        else if( !memcmp( id+1, "SD", 2 ) ) four_digits( buf, buflen, "Thunder", id+3 );
         else if( !memcmp( id+1, "SS", 2 ) ) four_digits( buf, buflen, "SwarmScope", id+3 );
         else if( !memcmp( id+1, "SZ", 2 ) ) four_digits( buf, buflen, "Shareaza", id+3 );
         else if( !memcmp( id+1, "S~", 2 ) ) four_digits( buf, buflen, "Shareaza", id+3 );
@@ -250,7 +241,6 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
         else if( !memcmp( id+1, "TS", 2 ) ) four_digits( buf, buflen, "TorrentStorm", id+3 );
         else if( !memcmp( id+1, "UL", 2 ) ) four_digits( buf, buflen, "uLeecher!", id+3 );
         else if( !memcmp( id+1, "VG", 2 ) ) four_digits( buf, buflen, "Vagaa", id+3 );
-        else if( !memcmp( id+1, "WT", 2 ) ) four_digits( buf, buflen, "BitLet", id+3 );
         else if( !memcmp( id+1, "WY", 2 ) ) four_digits( buf, buflen, "Wyzo", id+3 );
         else if( !memcmp( id+1, "XL", 2 ) ) four_digits( buf, buflen, "Xunlei", id+3 );
         else if( !memcmp( id+1, "XT", 2 ) ) four_digits( buf, buflen, "XanTorrent", id+3 );
@@ -286,7 +276,7 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
         {
             tr_snprintf( buf, buflen, "CTorrent %d.%d.%02d", charint(id[3]), charint(id[4]), strint(id+5,2) );
         }
-        else if( !memcmp( id+1, "XX", 2 ) )
+        else if( !memcmp( id+1, "XC", 2 ) || !memcmp( id+1, "XX", 2 ) )
         {
             tr_snprintf( buf, buflen, "Xtorrent %d.%d (%d)", charint(id[3]), charint(id[4]), strint(id+5,2) );
         }
@@ -311,8 +301,6 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
 
     if( decodeBitCometClient( buf, buflen, id ) )
         return;
-    if( decodeBitSpiritClient( buf, buflen, id ) )
-        return;
 
     /* Clients with no version */
          if( !memcmp( id, "AZ2500BT", 8 ) )  no_version( buf, buflen, "BitTyrant (Azureus Mod)" );
@@ -326,9 +314,10 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
     else if( !memcmp( id, "346-", 4 ) )      no_version( buf, buflen, "TorrentTopia" );
     else if( !memcmp( id, "eX", 2 ) )        no_version( buf, buflen, "eXeem" );
     else if( !memcmp( id, "-aria2-", 7 ) )   no_version( buf, buflen, "aria2" );
+    else if( !memcmp( id, "-WT-", 4 ) )      no_version( buf, buflen, "BitLet" );
     else if( !memcmp( id, "-FG", 3 ) )       two_major_two_minor( buf, buflen, "FlashGet", id+3 );
-   
-    /* Everything else */ 
+
+    /* Everything else */
     else if( !memcmp( id, "S3", 2 ) && id[2] == '-' && id[4] == '-' && id[6] == '-' )
     {
         tr_snprintf( buf, buflen, "Amazon S3 %c.%c.%c", id[3], id[5], id[7] );
@@ -367,7 +356,11 @@ tr_clientForId( char * buf, size_t buflen, const void * id_in )
     {
         tr_snprintf( buf, buflen, "Blizzard Downloader %d.%d", id[3]+1, id[4] );
     }
-    else if( '\0' == id[0] && !memcmp( &id[1], "BS", 2 ) )
+    else if( !memcmp( id, "-SP", 3 ) )
+    {
+        three_digits( buf, buflen, "BitSpirit", id+3 );
+    }
+    else if( '\0' == id[0] && !memcmp( id+2, "BS", 2 ) )
     {
         tr_snprintf( buf, buflen, "BitSpirit %u", ( id[1] == 0 ? 1 : id[1] ) );
     }
