@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: completion.c 8218 2009-04-12 15:01:29Z charles $
+ * $Id: completion.c 9868 2010-01-04 21:00:47Z charles $
  */
 
 #include <assert.h>
@@ -16,6 +16,7 @@
 #include "transmission.h"
 #include "completion.h"
 #include "torrent.h"
+#include "torrent-magnet.h"
 #include "utils.h"
 
 static void
@@ -240,6 +241,7 @@ tr_cpBlockBitfieldSet( tr_completion * cp, tr_bitfield * blockBitfield )
 tr_completeness
 tr_cpGetStatus( const tr_completion * cp )
 {
+    if( !tr_torrentHasMetadata( cp->tor ) ) return TR_LEECH;
     if( cp->sizeNow == cp->tor->info.totalSize ) return TR_SEED;
     if( cp->sizeNow == tr_cpSizeWhenDone( cp ) ) return TR_PARTIAL_SEED;
     return TR_LEECH;
@@ -255,6 +257,9 @@ calculateHaveValid( const tr_completion * ccp )
     const uint64_t            lastPieceSize  = tor->lastPieceSize;
     const tr_piece_index_t    lastPiece      = tor->info.pieceCount - 1;
 
+    if( !tr_torrentHasMetadata( tor ) )
+        return 0;
+
     for( i=0; i!=lastPiece; ++i )
         if( tr_cpPieceIsComplete( ccp, i ) )
             b += pieceSize;
@@ -325,21 +330,8 @@ tr_cpFileIsComplete( const tr_completion * cp, tr_file_index_t fileIndex )
     const tr_block_index_t firstBlock = file->offset / tor->blockSize;
     const tr_block_index_t lastBlock = file->length ? ( ( file->offset + file->length - 1 ) / tor->blockSize ) : firstBlock;
 
-    tr_assert( tr_torBlockPiece( tor, firstBlock ) == file->firstPiece,
-               "file->offset %"PRIu64"; file->length %"PRIu64"; "
-               "pieceSize %"PRIu32"; blockSize %"PRIu32"; "
-               "firstBlock %"PRIu64"; lastBlock %"PRIu64,
-               file->offset, file->length,
-               tor->info.pieceSize, tor->blockSize,
-               firstBlock, lastBlock );
-
-    tr_assert( tr_torBlockPiece( tor, lastBlock ) == file->lastPiece,
-               "file->offset %"PRIu64"; file->length %"PRIu64"; "
-               "pieceSize %"PRIu32"; blockSize %"PRIu32"; "
-               "firstBlock %"PRIu64"; lastBlock %"PRIu64,
-               file->offset, file->length,
-               tor->info.pieceSize, tor->blockSize,
-               firstBlock, lastBlock );
+    assert( tr_torBlockPiece( tor, firstBlock ) == file->firstPiece );
+    assert( tr_torBlockPiece( tor, lastBlock ) == file->lastPiece );
 
     for( block=firstBlock; block<=lastBlock; ++block )
         if( !tr_cpBlockIsCompleteFast( cp, block ) )
