@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: fdlimit.h 8561 2009-05-29 19:17:12Z charles $
+ * $Id: fdlimit.h 9931 2010-01-14 14:20:49Z charles $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,7 +23,7 @@
  *****************************************************************************/
 
 #ifndef __TRANSMISSION__
-#error only libtransmission should #include this header.
+ #error only libtransmission should #include this header.
 #endif
 
 #include "transmission.h"
@@ -34,8 +34,15 @@
  * @{
  */
 
-void tr_fdInit( size_t openFileLimit,
-                size_t globalPeerLimit );
+void tr_fdSetFileLimit( tr_session * session, int limit );
+
+int tr_fdGetFileLimit( const tr_session * session );
+
+void tr_fdSetGlobalPeerLimit( tr_session * session, int limit );
+
+/***
+****
+***/
 
 int tr_open_file_for_scanning( const char * filename );
 
@@ -46,6 +53,9 @@ void tr_close_file( int fd );
 tr_bool tr_preallocate_file( const char * filename, uint64_t length );
 
 int64_t tr_lseek( int fd, int64_t offset, int whence );
+ssize_t tr_pread(int fd, void *buf, size_t count, off_t offset);
+ssize_t tr_pwrite(int fd, void *buf, size_t count, off_t offset);
+int tr_prefetch(int fd, off_t offset, size_t count);
 
 
 /**
@@ -53,33 +63,28 @@ int64_t tr_lseek( int fd, int64_t offset, int whence );
  *
  * A small pool of open files is kept to avoid the overhead of
  * continually opening and closing the same files when downloading
- * piece data.    It's also used to ensure only one caller can
- * write to the file at a time.  Callers check out a file, use it,
- * and then check it back in via tr_fdFileReturn() when done.
+ * piece data.
  *
- * - if `folder' doesn't exist, errno is set to ENOENT.
  * - if doWrite is true, subfolders in torrentFile are created if necessary.
  * - if doWrite is true, the target file is created if necessary.
  *
  * on success, a file descriptor >= 0 is returned.
  * on failure, a -1 is returned and errno is set.
  *
- * @see tr_fdFileReturn
  * @see tr_fdFileClose
  */
-int  tr_fdFileCheckout( const char             * folder,
-                        const char             * torrentFile,
+int  tr_fdFileCheckout( tr_session             * session,
+                        int                      torrentId,
+                        tr_file_index_t          fileNum,
+                        const char             * fileName,
                         tr_bool                  doWrite,
                         tr_preallocation_mode    preallocationMode,
                         uint64_t                 desiredFileSize );
 
-/**
- * Returns an fd from tr_fdFileCheckout() so that other clients may borrow it.
- *
- * @see tr_fdFileCheckout
- * @see tr_fdFileClose
- */
-void tr_fdFileReturn( int file );
+int tr_fdFileGetCached( tr_session             * session,
+                        int                      torrentId,
+                        tr_file_index_t          fileNum,
+                        tr_bool                  doWrite );
 
 /**
  * Closes a file that's being held by our file repository.
@@ -88,32 +93,40 @@ void tr_fdFileReturn( int file );
  * If the file is currently checked out, it will be closed upon its return.
  *
  * @see tr_fdFileCheckout
- * @see tr_fdFileReturn
  */
-void     tr_fdFileClose( const char * filename );
+void tr_fdFileClose( tr_session        * session,
+                     const tr_torrent  * tor,
+                     tr_file_index_t     fileNo );
+
+
+/**
+ * Closes all the files associated with a given torrent id
+ */
+void tr_fdTorrentClose( tr_session * session, int torrentId );
 
 
 /***********************************************************************
  * Sockets
  **********************************************************************/
-int      tr_fdSocketCreate( int domain, int type );
+int      tr_fdSocketCreate( tr_session * session, int domain, int type );
 
-int      tr_fdSocketAccept( int           b,
+int      tr_fdSocketAccept( tr_session  * session,
+                            int           b,
                             tr_address  * addr,
                             tr_port     * port );
 
-void     tr_fdSocketClose( int s );
+void     tr_fdSocketClose( tr_session * session, int s );
 
 /***********************************************************************
  * tr_fdClose
  ***********************************************************************
  * Frees resources allocated by tr_fdInit.
  **********************************************************************/
-void     tr_fdClose( void );
+void     tr_fdClose( tr_session * session );
 
 
-void     tr_fdSetPeerLimit( uint16_t n );
+void     tr_fdSetPeerLimit( tr_session * session, int n );
 
-uint16_t tr_fdGetPeerLimit( void );
+int      tr_fdGetPeerLimit( const tr_session * );
 
 /* @} */
