@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: handshake.c 8698 2009-06-15 03:24:40Z charles $
+ * $Id: handshake.c 10302 2010-03-06 14:56:15Z charles $
  */
 
 #include <assert.h>
@@ -20,13 +20,13 @@
 #include <event.h>
 
 #include "transmission.h"
-#include "session.h"
 #include "bencode.h"
 #include "clients.h"
 #include "crypto.h"
 #include "handshake.h"
 #include "peer-io.h"
 #include "peer-mgr.h"
+#include "session.h"
 #include "torrent.h"
 #include "tr-dht.h"
 #include "utils.h"
@@ -64,7 +64,7 @@ enum
     CRYPTO_PROVIDE_CRYPTO          = 2,
 
     /* how long to wait before giving up on a handshake */
-    HANDSHAKE_TIMEOUT_SEC          = 60
+    HANDSHAKE_TIMEOUT_SEC          = 30
 };
 
 
@@ -76,13 +76,13 @@ enum
  #define HANDSHAKE_SET_LTEP( bits ) ( (void)0 )
 #endif
 
-#ifdef ENABLE_FAST 
- #define HANDSHAKE_HAS_FASTEXT( bits ) ( ( ( bits )[7] & 0x04 ) ? 1 : 0 ) 
- #define HANDSHAKE_SET_FASTEXT( bits ) ( ( bits )[7] |= 0x04 ) 
-#else 
- #define HANDSHAKE_HAS_FASTEXT( bits ) ( 0 ) 
- #define HANDSHAKE_SET_FASTEXT( bits ) ( (void)0 ) 
-#endif 
+#ifdef ENABLE_FAST
+ #define HANDSHAKE_HAS_FASTEXT( bits ) ( ( ( bits )[7] & 0x04 ) ? 1 : 0 )
+ #define HANDSHAKE_SET_FASTEXT( bits ) ( ( bits )[7] |= 0x04 )
+#else
+ #define HANDSHAKE_HAS_FASTEXT( bits ) ( 0 )
+ #define HANDSHAKE_SET_FASTEXT( bits ) ( (void)0 )
+#endif
 
 #ifdef ENABLE_DHT
  #define HANDSHAKE_HAS_DHT( bits ) ( ( ( bits )[7] & 0x01 ) ? 1 : 0 )
@@ -90,7 +90,7 @@ enum
 #else
  #define HANDSHAKE_HAS_DHT( bits ) ( 0 )
  #define HANDSHAKE_SET_DHT( bits ) ( (void)0 )
-#endif 
+#endif
 
 /* http://www.azureuswiki.com/index.php/Extension_negotiation_protocol
    these macros are to be used if both extended messaging and the
@@ -225,10 +225,10 @@ buildHandshakeMessage( tr_handshake * handshake, uint8_t * buf )
     HANDSHAKE_SET_LTEP( walk );
     HANDSHAKE_SET_FASTEXT( walk );
 
-    /* Note that this doesn't depend on whether the torrent is private.  We
-       don't accept DHT peers for a private torrent, but we participate in
-       the DHT regardless. */
-    if(tr_dhtEnabled(handshake->session))
+    /* Note that this doesn't depend on whether the torrent is private.
+     * We don't accept DHT peers for a private torrent,
+     * but we participate in the DHT regardless. */
+    if( tr_dhtEnabled( handshake->session ) )
         HANDSHAKE_SET_DHT( walk );
 
     walk += HANDSHAKE_FLAGS_LEN;
@@ -312,9 +312,7 @@ parseHandshake( tr_handshake *    handshake,
 
     tr_peerIoEnableFEXT( handshake->io, HANDSHAKE_HAS_FASTEXT( reserved ) );
 
-    /* This doesn't depend on whether the torrent is private. */
-    if( tor && tor->session->isDHTEnabled )
-        tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
+    tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
 
     return HANDSHAKE_OK;
 }
@@ -374,7 +372,7 @@ static uint32_t
 getCryptoSelect( const tr_handshake * handshake,
                  uint32_t             crypto_provide )
 {
-    uint32_t choices[4];
+    uint32_t choices[2];
     int      i, nChoices = 0;
 
     switch( handshake->encryptionMode )
@@ -674,6 +672,8 @@ readHandshake( tr_handshake *    handshake,
 
     tr_peerIoEnableFEXT( handshake->io, HANDSHAKE_HAS_FASTEXT( reserved ) );
 
+    tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
+
     /* torrent hash */
     tr_peerIoReadBytes( handshake->io, inbuf, hash, sizeof( hash ) );
     if( tr_peerIoIsIncoming( handshake->io ) )
@@ -862,7 +862,7 @@ readCryptoProvide( tr_handshake *    handshake,
         const tr_bool clientIsSeed = tr_torrentIsSeed( tor );
         const tr_bool peerIsSeed = tr_peerMgrPeerIsSeed( tor, tr_peerIoGetAddress( handshake->io, NULL ) );
         dbgmsg( handshake, "got INCOMING connection's encrypted handshake for torrent [%s]",
-                tor->info.name );
+                tr_torrentName( tor ) );
         tr_peerIoSetTorrentHash( handshake->io, tor->info.hash );
 
         if( clientIsSeed && peerIsSeed )
@@ -1026,6 +1026,8 @@ canRead( struct tr_peerIo * io, void * arg, size_t * piece )
     ReadState         ret;
     tr_bool           readyForMore = TRUE;
 
+    assert( tr_isPeerIo( io ) );
+
     /* no piece data in handshake */
     *piece = 0;
 
@@ -1127,7 +1129,7 @@ tr_handshakeDone( tr_handshake * handshake,
 
     success = fireDoneFunc( handshake, isOK );
 
-    tr_handshakeFree( handshake );    
+    tr_handshakeFree( handshake );
 
     return success ? READ_LATER : READ_ERR;
 }
@@ -1141,9 +1143,9 @@ tr_handshakeAbort( tr_handshake * handshake )
 static void
 gotError( tr_peerIo  * io UNUSED,
           short        what,
-          void       * arg )
+          void       * vhandshake )
 {
-    tr_handshake * handshake = (tr_handshake *) arg;
+    tr_handshake * handshake = vhandshake;
 
     /* if the error happened while we were sending a public key, we might
      * have encountered a peer that doesn't do encryption... reconnect and
