@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,9 +7,13 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: inout.c 8296 2009-04-26 16:14:47Z charles $
+ * $Id: inout.c 10276 2010-02-23 07:20:57Z charles $
  */
 
+#ifdef HAVE_LSEEK64
+ #define _LARGEFILE64_SOURCE
+#endif
+
 #include <assert.h>
 #include <errno.h>
 #include <stdlib.h> /* realloc */
@@ -39,21 +43,24 @@
   #undef read
  #endif
  #define read  _read
- 
+
  #if defined(write)
   #undef write
  #endif
  #define write _write
 #endif
 
-enum { TR_IO_READ, TR_IO_WRITE };
+enum { TR_IO_READ, TR_IO_PREFETCH,
+       /* Any operations that require write access must follow TR_IO_WRITE. */
+       TR_IO_WRITE
+};
 
 int64_t
 tr_lseek( int fd, int64_t offset, int whence )
 {
-#if defined(_LARGEFILE_SOURCE)
+#if defined( HAVE_LSEEK64 )
     return lseek64( fd, (off64_t)offset, whence );
-#elif defined(WIN32)
+#elif defined( WIN32 )
     return _lseeki64( fd, offset, whence );
 #else
     return lseek( fd, (off_t)offset, whence );
@@ -62,7 +69,8 @@ tr_lseek( int fd, int64_t offset, int whence )
 
 /* returns 0 on success, or an errno on failure */
 static int
-readOrWriteBytes( const tr_torrent * tor,
+readOrWriteBytes( tr_session       * session,
+                  const tr_torrent * tor,
                   int                ioMode,
                   tr_file_index_t    fileIndex,
                   uint64_t           fileOffset,
@@ -71,50 +79,96 @@ readOrWriteBytes( const tr_torrent * tor,
 {
     const tr_info * info = &tor->info;
     const tr_file * file = &info->files[fileIndex];
-    tr_preallocation_mode preallocationMode;
 
-    typedef size_t ( *iofunc )( int, void *, size_t );
-    iofunc          func = ioMode == TR_IO_READ ? (iofunc)read : (iofunc)write;
-    struct stat     sb;
     int             fd = -1;
-    int             err;
-    int             fileExists;
+    int             err = 0;
+    const tr_bool doWrite = ioMode >= TR_IO_WRITE;
 
-    assert( tor->downloadDir && *tor->downloadDir );
     assert( fileIndex < info->fileCount );
     assert( !file->length || ( fileOffset < file->length ) );
     assert( fileOffset + buflen <= file->length );
 
-    {
-        char path[MAX_PATH_LENGTH];
-        tr_snprintf( path, sizeof( path ), "%s%c%s", tor->downloadDir, TR_PATH_DELIMITER, file->name );
-        fileExists = !stat( path, &sb );
-    }
-
     if( !file->length )
         return 0;
 
-    if( ( file->dnd ) || ( ioMode != TR_IO_WRITE ) )
-        preallocationMode = TR_PREALLOCATE_NONE;
-    else
-        preallocationMode = tor->session->preallocationMode;
+    fd = tr_fdFileGetCached( session, tr_torrentId( tor ), fileIndex, doWrite );
 
-    if( ( ioMode == TR_IO_READ ) && !fileExists ) /* does file exist? */
-        err = errno;
-    else if( ( fd = tr_fdFileCheckout ( tor->downloadDir, file->name, ioMode == TR_IO_WRITE, preallocationMode, file->length ) ) < 0 )
-        err = errno;
-    else if( tr_lseek( fd, (int64_t)fileOffset, SEEK_SET ) == -1 )
-        err = errno;
-    else if( func( fd, buf, buflen ) != buflen )
-        err = errno;
-    else
-        err = 0;
+    if( fd < 0 )
+    {
+        /* the fd cache doesn't have this file...
+         * we'll need to open it and maybe create it */
+        char * subpath;
+        const char * base;
+        tr_bool fileExists;
+        tr_preallocation_mode preallocationMode;
 
-    if( ( !err ) && ( !fileExists ) && ( ioMode == TR_IO_WRITE ) )
-        tr_statsFileCreated( tor->session );
+        fileExists = tr_torrentFindFile2( tor, fileIndex, &base, &subpath );
 
-    if( fd >= 0 )
-        tr_fdFileReturn( fd );
+        if( !fileExists )
+        {
+            base = tr_torrentGetCurrentDir( tor );
+
+            if( tr_sessionIsIncompleteFileNamingEnabled( tor->session ) )
+                subpath = tr_torrentBuildPartial( tor, fileIndex );
+            else
+                subpath = tr_strdup( file->name );
+        }
+
+        if( ( file->dnd ) || ( ioMode < TR_IO_WRITE ) )
+            preallocationMode = TR_PREALLOCATE_NONE;
+        else
+            preallocationMode = tor->session->preallocationMode;
+
+        if( ( ioMode < TR_IO_WRITE ) && !fileExists ) /* does file exist? */
+        {
+            err = ENOENT;
+        }
+        else
+        {
+            char * filename = tr_buildPath( base, subpath, NULL );
+
+            if( ( fd = tr_fdFileCheckout( session, tor->uniqueId, fileIndex, filename,
+                                          doWrite, preallocationMode, file->length ) ) < 0 )
+            {
+                err = errno;
+                tr_torerr( tor, "tr_fdFileCheckout failed for \"%s\": %s", filename, tr_strerror( err ) );
+            }
+
+            tr_free( filename );
+        }
+
+        if( doWrite && !err )
+            tr_statsFileCreated( tor->session );
+
+        tr_free( subpath );
+    }
+
+    if( !err )
+    {
+        if( ioMode == TR_IO_READ ) {
+            const int rc = tr_pread( fd, buf, buflen, fileOffset );
+            if( rc < 0 ) {
+                err = errno;
+                tr_torerr( tor, "read failed for \"%s\": %s",
+                           file->name, tr_strerror( err ) );
+            }
+        } else if( ioMode == TR_IO_PREFETCH ) {
+            const int rc = tr_prefetch( fd, fileOffset, buflen );
+            if( rc < 0 ) {
+                tr_tordbg( tor, "prefetch failed for \"%s\": %s",
+                           file->name, tr_strerror( err ) );
+            }
+        } else if( ioMode == TR_IO_WRITE ) {
+            const int rc = tr_pwrite( fd, buf, buflen, fileOffset );
+            if( rc < 0 ) {
+                err = errno;
+                tr_torerr( tor, "write failed for \"%s\": %s",
+                           file->name, tr_strerror( err ) );
+            }
+        } else {
+            abort();
+        }
+    }
 
     return err;
 }
@@ -155,11 +209,11 @@ tr_ioFindFileLocation( const tr_torrent * tor,
 
 /* returns 0 on success, or an errno on failure */
 static int
-readOrWritePiece( const tr_torrent * tor,
+readOrWritePiece( tr_torrent       * tor,
                   int                ioMode,
                   tr_piece_index_t   pieceIndex,
                   uint32_t           pieceOffset,
-                  uint8_t *          buf,
+                  uint8_t          * buf,
                   size_t             buflen )
 {
     int             err = 0;
@@ -178,34 +232,51 @@ readOrWritePiece( const tr_torrent * tor,
     while( buflen && !err )
     {
         const tr_file * file = &info->files[fileIndex];
-        const uint64_t  bytesThisPass = MIN( buflen, file->length - fileOffset );
+        const uint64_t bytesThisPass = MIN( buflen, file->length - fileOffset );
 
-        err = readOrWriteBytes( tor, ioMode, fileIndex, fileOffset, buf, bytesThisPass );
+        err = readOrWriteBytes( tor->session, tor, ioMode, fileIndex, fileOffset, buf, bytesThisPass );
         buf += bytesThisPass;
         buflen -= bytesThisPass;
         ++fileIndex;
         fileOffset = 0;
+
+        if( ( err != 0 ) && (ioMode == TR_IO_WRITE ) )
+        {
+            char * path = tr_buildPath( tor->downloadDir, file->name, NULL );
+            tr_torrentSetLocalError( tor, "%s (%s)", tr_strerror( err ), path );
+            tr_free( path );
+        }
     }
 
     return err;
 }
 
 int
-tr_ioRead( const tr_torrent * tor,
+tr_ioRead( tr_torrent       * tor,
            tr_piece_index_t   pieceIndex,
            uint32_t           begin,
            uint32_t           len,
-           uint8_t *          buf )
+           uint8_t          * buf )
 {
     return readOrWritePiece( tor, TR_IO_READ, pieceIndex, begin, buf, len );
 }
 
 int
-tr_ioWrite( const tr_torrent * tor,
+tr_ioPrefetch( tr_torrent       * tor,
+               tr_piece_index_t   pieceIndex,
+               uint32_t           begin,
+               uint32_t           len)
+{
+    return readOrWritePiece( tor, TR_IO_PREFETCH, pieceIndex, begin,
+                             NULL, len );
+}
+
+int
+tr_ioWrite( tr_torrent       * tor,
             tr_piece_index_t   pieceIndex,
             uint32_t           begin,
             uint32_t           len,
-            const uint8_t *    buf )
+            const uint8_t    * buf )
 {
     return readOrWritePiece( tor, TR_IO_WRITE, pieceIndex, begin,
                              (uint8_t*)buf,
@@ -217,25 +288,17 @@ tr_ioWrite( const tr_torrent * tor,
 ****/
 
 static tr_bool
-recalculateHash( const tr_torrent * tor,
+recalculateHash( tr_torrent       * tor,
                  tr_piece_index_t   pieceIndex,
-                 void             * buffer,
-                 size_t             buflen,
-                 uint8_t *          setme )
+                 uint8_t          * setme )
 {
     size_t   bytesLeft;
     uint32_t offset = 0;
     tr_bool  success = TRUE;
-    uint8_t  stackbuf[MAX_STACK_ARRAY_SIZE];
+    uint8_t * buffer = tr_sessionGetBuffer( tor->session );
+    const size_t buflen = SESSION_BUFFER_SIZE;
     SHA_CTX  sha;
 
-    /* fallback buffer */
-    if( ( buffer == NULL ) || ( buflen < 1 ) )
-    {
-        buffer = stackbuf;
-        buflen = sizeof( stackbuf );
-    }
-
     assert( tor != NULL );
     assert( pieceIndex < tor->info.pieceCount );
     assert( buffer != NULL );
@@ -259,17 +322,15 @@ recalculateHash( const tr_torrent * tor,
     if( success )
         SHA1_Final( setme, &sha );
 
+    tr_sessionReleaseBuffer( tor->session );
     return success;
 }
 
 tr_bool
-tr_ioTestPiece( const tr_torrent  * tor,
-                tr_piece_index_t    pieceIndex,
-                void              * buffer,
-                size_t              buflen )
+tr_ioTestPiece( tr_torrent * tor, tr_piece_index_t piece )
 {
     uint8_t hash[SHA_DIGEST_LENGTH];
 
-    return recalculateHash( tor, pieceIndex, buffer, buflen, hash )
-           && !memcmp( hash, tor->info.pieces[pieceIndex].hash, SHA_DIGEST_LENGTH );
+    return recalculateHash( tor, piece, hash )
+           && !memcmp( hash, tor->info.pieces[piece].hash, SHA_DIGEST_LENGTH );
 }
