@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,12 +7,12 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: json.c 7664 2009-01-11 17:46:51Z charles $
+ * $Id: json.c 9979 2010-01-20 23:58:09Z charles $
  */
 
 #include <assert.h>
 #include <ctype.h>
-#include <errno.h>
+#include <errno.h> /* EILSEQ, EINVAL */
 #include <string.h>
 #include <stdio.h> /* printf */
 
@@ -28,9 +28,10 @@
 
 struct json_benc_data
 {
-    tr_benc *      top;
+    tr_bool        hasContent;
+    tr_benc      * top;
     tr_ptrArray    stack;
-    char *         key;
+    char         * key;
 };
 
 static tr_benc*
@@ -69,6 +70,7 @@ callback( void *             vdata,
     switch( type )
     {
         case JSON_T_ARRAY_BEGIN:
+            data->hasContent = TRUE;
             node = getNode( data );
             tr_bencInitList( node, 0 );
             tr_ptrArrayAppend( &data->stack, node );
@@ -79,6 +81,7 @@ callback( void *             vdata,
             break;
 
         case JSON_T_OBJECT_BEGIN:
+            data->hasContent = TRUE;
             node = getNode( data );
             tr_bencInitDict( node, 0 );
             tr_ptrArrayAppend( &data->stack, node );
@@ -89,37 +92,39 @@ callback( void *             vdata,
             break;
 
         case JSON_T_FLOAT:
-        {
-            char buf[128];
-            tr_snprintf( buf, sizeof( buf ), "%f",
-                         (double)value->vu.float_value );
-            tr_bencInitStr( getNode( data ), buf, -1 );
+            data->hasContent = TRUE;
+            tr_bencInitReal( getNode( data ), (double)value->vu.float_value );
             break;
-        }
 
         case JSON_T_NULL:
+            data->hasContent = TRUE;
             tr_bencInitStr( getNode( data ), "", 0 );
             break;
 
         case JSON_T_INTEGER:
+            data->hasContent = TRUE;
             tr_bencInitInt( getNode( data ), value->vu.integer_value );
             break;
 
         case JSON_T_TRUE:
-            tr_bencInitInt( getNode( data ), 1 );
+            data->hasContent = TRUE;
+            tr_bencInitBool( getNode( data ), 1 );
             break;
 
         case JSON_T_FALSE:
-            tr_bencInitInt( getNode( data ), 0 );
+            data->hasContent = TRUE;
+            tr_bencInitBool( getNode( data ), 0 );
             break;
 
         case JSON_T_STRING:
+            data->hasContent = TRUE;
             tr_bencInitStr( getNode( data ),
                             value->vu.str.value,
                             value->vu.str.length );
             break;
 
         case JSON_T_KEY:
+            data->hasContent = TRUE;
             assert( !data->key );
             data->key = tr_strdup( value->vu.str.value );
             break;
@@ -129,15 +134,18 @@ callback( void *             vdata,
 }
 
 int
-tr_jsonParse( const void *     vbuf,
+tr_jsonParse( const char     * source,
+              const void     * vbuf,
               size_t           len,
-              tr_benc *        setme_benc,
+              tr_benc        * setme_benc,
               const uint8_t ** setme_end )
 {
+    int                         line = 1;
+    int                         column = 1;
     int                         err = 0;
-    const unsigned char *       buf = vbuf;
-    const void *                bufend = buf + len;
-    struct JSON_config_struct   config;
+    const unsigned char       * buf = vbuf;
+    const void                * bufend = buf + len;
+    JSON_config                 config;
     struct JSON_parser_struct * checker;
     struct json_benc_data       data;
 
@@ -146,16 +154,32 @@ tr_jsonParse( const void *     vbuf,
     config.callback_ctx = &data;
     config.depth = -1;
 
+    data.hasContent = FALSE;
     data.key = NULL;
     data.top = setme_benc;
     data.stack = TR_PTR_ARRAY_INIT;
 
     checker = new_JSON_parser( &config );
-    while( ( buf != bufend ) && JSON_parser_char( checker, *buf ) )
+    while( ( buf != bufend ) && JSON_parser_char( checker, *buf ) ) {
+        if( *buf != '\n' )
+            ++column;
+        else {
+            ++line;
+            column = 1;
+        }
         ++buf;
+    }
 
-    if( buf != bufend )
+    if( buf != bufend ) {
+        if( source )
+            tr_err( "JSON parser failed in %s at line %d, column %d: \"%.16s\"", source, line, column, buf );
+        else
+            tr_err( "JSON parser failed at line %d, column %d: \"%.16s\"", line, column, buf );
         err = EILSEQ;
+    }
+
+    if( !data.hasContent )
+        err = EINVAL;
 
     if( setme_end )
         *setme_end = (const uint8_t*) buf;
