@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-io.c 10300 2010-03-02 04:51:40Z charles $
+ * $Id: peer-io.c 10603 2010-05-01 14:53:37Z charles $
  */
 
 #include <assert.h>
@@ -75,7 +75,6 @@ struct tr_datatype
 {
     tr_bool  isPieceData;
     size_t   length;
-    struct __tr_list head;
 };
 
 /***
@@ -87,7 +86,8 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
 {
      while( bytes_transferred && tr_isPeerIo( io ) )
      {
-        struct tr_datatype * next = __tr_list_entry( io->outbuf_datatypes.next, struct tr_datatype, head );
+        struct tr_datatype * next = io->outbuf_datatypes->data;
+
         const size_t payload = MIN( next->length, bytes_transferred );
         const size_t overhead = guessPacketOverhead( payload );
 
@@ -104,7 +104,7 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
             bytes_transferred -= payload;
             next->length -= payload;
             if( !next->length ) {
-                __tr_list_remove( io->outbuf_datatypes.next );
+                tr_list_pop_front( &io->outbuf_datatypes );
                 tr_free( next );
             }
         }
@@ -398,8 +398,6 @@ tr_peerIoNew( tr_session       * session,
     event_set( &io->event_read, io->socket, EV_READ, event_read_cb, io );
     event_set( &io->event_write, io->socket, EV_WRITE, event_write_cb, io );
 
-    __tr_list_init( &io->outbuf_datatypes );
-
     return io;
 }
 
@@ -451,6 +449,9 @@ event_enable( tr_peerIo * io, short event )
     assert( event_initialized( &io->event_read ) );
     assert( event_initialized( &io->event_write ) );
 
+    if( io->socket < 0 )
+        return;
+
     if( ( event & EV_READ ) && ! ( io->pendingEvents & EV_READ ) )
     {
         dbgmsg( io, "enabling libevent ready-to-read polling" );
@@ -512,13 +513,6 @@ tr_peerIoSetEnabled( tr_peerIo    * io,
 ****
 ***/
 
-static void
-trDatatypeFree( void * data )
-{
-    struct tr_datatype * dt = __tr_list_entry( data, struct tr_datatype, head );
-    tr_free(dt);
-}
-
 static void
 io_dtor( void * vio )
 {
@@ -535,7 +529,7 @@ io_dtor( void * vio )
     evbuffer_free( io->inbuf );
     tr_netClose( io->session, io->socket );
     tr_cryptoFree( io->crypto );
-    __tr_list_destroy( &io->outbuf_datatypes, trDatatypeFree );
+    tr_list_free( &io->outbuf_datatypes, tr_free );
 
     memset( io, ~0, sizeof( tr_peerIo ) );
     tr_free( io );
@@ -641,10 +635,10 @@ tr_peerIoReconnect( tr_peerIo * io )
     io->socket = tr_netOpenPeerSocket( session, &io->addr, io->port, io->isSeed );
     event_set( &io->event_read, io->socket, EV_READ, event_read_cb, io );
     event_set( &io->event_write, io->socket, EV_WRITE, event_write_cb, io );
-    event_enable( io, pendingEvents );
 
     if( io->socket >= 0 )
     {
+        event_enable( io, pendingEvents );
         tr_netSetTOS( io->socket, session->peerSocketTOS );
         return 0;
     }
@@ -800,9 +794,7 @@ tr_peerIoWrite( tr_peerIo   * io,
     datatype = tr_new( struct tr_datatype, 1 );
     datatype->isPieceData = isPieceData != 0;
     datatype->length = byteCount;
-
-    __tr_list_init( &datatype->head );
-    __tr_list_append( &io->outbuf_datatypes, &datatype->head );
+    tr_list_append( &io->outbuf_datatypes, datatype );
 
     switch( io->encryptionMode )
     {
@@ -981,15 +973,17 @@ int
 tr_peerIoFlushOutgoingProtocolMsgs( tr_peerIo * io )
 {
     size_t byteCount = 0;
-    struct __tr_list * walk;
-    struct __tr_list * fencepost = &io->outbuf_datatypes;
+    tr_list * it;
 
     /* count up how many bytes are used by non-piece-data messages
        at the front of our outbound queue */
-    for( walk=fencepost->next; walk!=fencepost; walk=walk->next ) {
-        struct tr_datatype * d = __tr_list_entry( walk, struct tr_datatype, head );
+    for( it=io->outbuf_datatypes; it!=NULL; it=it->next )
+    {
+        struct tr_datatype * d = it->data;
+
         if( d->isPieceData )
             break;
+
         byteCount += d->length;
     }
 
