@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-io.h 9868 2010-01-04 21:00:47Z charles $
+ * $Id: peer-io.h 10583 2010-05-01 04:37:52Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -27,7 +27,7 @@
 
 #include "transmission.h"
 #include "bandwidth.h"
-#include "list.h" /* __tr_list */
+#include "list.h" /* tr_list */
 #include "net.h" /* tr_address */
 
 struct evbuffer;
@@ -106,7 +106,7 @@ typedef struct tr_peerIo
 
     struct evbuffer     * inbuf;
     struct evbuffer     * outbuf;
-    struct __tr_list      outbuf_datatypes; /* struct tr_datatype */
+    struct tr_list      * outbuf_datatypes; /* struct tr_datatype */
 
     struct event          event_read;
     struct event          event_write;
