@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: platform.c 8163 2009-04-06 16:10:37Z charles $
+ * $Id: platform.c 9987 2010-01-21 03:14:46Z charles $
  */
 
 #ifdef WIN32
@@ -17,7 +17,9 @@
  #ifdef SYS_DARWIN
   #include <CoreFoundation/CoreFoundation.h>
  #endif
-
+ #ifdef __HAIKU__
+  #include <FindDirectory.h>
+ #endif
  #define _XOPEN_SOURCE 600  /* needed for recursive locks. */
  #ifndef __USE_UNIX98
   #define __USE_UNIX98 /* some older Linuxes need it spelt out for them */
@@ -76,6 +78,7 @@ tr_areThreadsEqual( tr_thread_id a,
 #endif
 }
 
+/** @brief portability wrapper around OS-dependent threads */
 struct tr_thread
 {
     void            ( * func )( void * );
@@ -105,6 +108,7 @@ ThreadFunc( void * _t )
 
     t->func( t->arg );
 
+    tr_free( t );
 #ifdef WIN32
     _endthreadex( 0 );
     return 0;
@@ -140,6 +144,7 @@ tr_threadNew( void   ( *func )(void *),
 ****  LOCKS
 ***/
 
+/** @brief portability wrapper around OS-dependent thread mutexes */
 struct tr_lock
 {
     int                 depth;
@@ -271,6 +276,10 @@ getOldConfigDir( void )
         char appdata[MAX_PATH]; /* SHGetFolderPath() requires MAX_PATH */
         SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, appdata );
         path = tr_buildPath( appdata, "Transmission", NULL );
+#elif defined( __HAIKU__ )
+        char buf[MAX_PATH_LENGTH];
+        find_directory( B_USER_SETTINGS_DIRECTORY, -1, true, buf, sizeof(buf) );
+        path = tr_buildPath( buf, "Transmission", NULL );
 #else
         path = tr_buildPath( getHomeDir( ), ".transmission", NULL );
 #endif
@@ -372,8 +381,7 @@ migrateFiles( const tr_session * session )
 }
 
 void
-tr_setConfigDir( tr_session * session,
-                 const char * configDir )
+tr_setConfigDir( tr_session * session, const char * configDir )
 {
     char * path;
 
@@ -431,6 +439,10 @@ tr_getDefaultConfigDir( const char * appname )
             char appdata[MAX_PATH]; /* SHGetFolderPath() requires MAX_PATH */
             SHGetFolderPath( NULL, CSIDL_APPDATA, NULL, 0, appdata );
             s = tr_buildPath( appdata, appname, NULL );
+#elif defined( __HAIKU__ )
+            char buf[MAX_PATH_LENGTH];
+            find_directory( B_USER_SETTINGS_DIRECTORY, -1, true, buf, sizeof(buf) );
+            s = tr_buildPath( buf, appname, NULL );
 #else
             if( ( s = getenv( "XDG_CONFIG_HOME" ) ) )
                 s = tr_buildPath( s, appname, NULL );
@@ -486,7 +498,11 @@ tr_getDefaultDownloadDir( void )
         }
 
         if( user_dir == NULL )
+#ifdef __HAIKU__
+            user_dir = tr_buildPath( getHomeDir( ), "Desktop", NULL );
+#else
             user_dir = tr_buildPath( getHomeDir( ), "Downloads", NULL );
+#endif
 
         tr_free( content );
         tr_free( config_file );
@@ -508,7 +524,6 @@ isClutchDir( const char * path )
     tr_inf( _( "Searching for web interface file \"%s\"" ), tmp );
     tr_free( tmp );
     return ret;
-    
 }
 
 const char *
@@ -540,10 +555,10 @@ tr_getClutchDir( const tr_session * session UNUSED )
             CFRelease( appRef );
 
             s = tr_buildPath( appString, "Contents", "Resources", "web", NULL );
-            
+
             if( !isClutchDir( s ) ) {
                 tr_free( s );
-                
+
                 /* Fallback to the Application Support folder */
                 s = tr_buildPath( tr_sessionGetConfigDir( session ), "web", NULL );
                 if( !isClutchDir( s ) ) {
@@ -556,11 +571,11 @@ tr_getClutchDir( const tr_session * session UNUSED )
 
             /* SHGetFolderPath explicitly requires MAX_PATH length */
             char dir[MAX_PATH];
-            
+
             /* Generally, Web interface should be stored in a Web subdir of
              * calling executable dir. */
 
-            if( s == NULL ) { 
+            if( s == NULL ) {
                 /* First, we should check personal AppData/Transmission/Web */
                 SHGetFolderPath( NULL, CSIDL_COMMON_APPDATA, NULL, 0, dir );
                 s = tr_buildPath( dir, "Transmission", "Web", NULL );
@@ -650,53 +665,6 @@ tr_getClutchDir( const tr_session * session UNUSED )
 ****
 ***/
 
-tr_lockfile_state_t
-tr_lockfile( const char * filename )
-{
-    tr_lockfile_state_t ret;
-
-#ifdef WIN32
-
-    HANDLE              file = CreateFile(
-        filename,
-        GENERIC_READ | GENERIC_WRITE,
-        FILE_SHARE_READ |
-        FILE_SHARE_WRITE,
-        NULL,
-        OPEN_ALWAYS,
-        FILE_ATTRIBUTE_NORMAL,
-        NULL );
-    if( file == INVALID_HANDLE_VALUE )
-        ret = TR_LOCKFILE_EOPEN;
-    else if( !LockFile( file, 0, 0, 1, 1 ) )
-        ret = TR_LOCKFILE_ELOCK;
-    else
-        ret = TR_LOCKFILE_SUCCESS;
-
-#else
-
-    int fd = open( filename, O_RDWR | O_CREAT, 0666 );
-    if( fd < 0 )
-        ret = TR_LOCKFILE_EOPEN;
-    else
-    {
-        struct flock lk;
-        memset( &lk, 0,  sizeof( lk ) );
-        lk.l_start = 0;
-        lk.l_len = 0;
-        lk.l_type = F_WRLCK;
-        lk.l_whence = SEEK_SET;
-        if( -1 == fcntl( fd, F_SETLK, &lk ) )
-            ret = TR_LOCKFILE_ELOCK;
-        else
-            ret = TR_LOCKFILE_SUCCESS;
-    }
-
-#endif
-
-    return ret;
-}
-
 #ifdef WIN32
 
 /* The following mmap functions are by Joerg Walter, and were taken from
