@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: ptrarray.c 6953 2008-10-25 02:15:37Z charles $
+ * $Id: ptrarray.c 10348 2010-03-10 16:35:11Z charles $
  */
 
 #include <assert.h>
@@ -17,38 +17,22 @@
 #include "ptrarray.h"
 #include "utils.h"
 
-#define GROW 32
+#define FLOOR 32
 
-struct tr_ptrArray
+const tr_ptrArray TR_PTR_ARRAY_INIT = { NULL, 0, 0 };
+
+void
+tr_ptrArrayDestruct( tr_ptrArray * p, PtrArrayForeachFunc func )
 {
-    void ** items;
-    int     n_items;
-    int     n_alloc;
-};
+    assert( p );
+    assert( p->items || !p->n_items );
 
-tr_ptrArray*
-tr_ptrArrayNew( void )
-{
-    tr_ptrArray * p;
+    if( func )
+        tr_ptrArrayForeach( p, func );
 
-    p = tr_new( tr_ptrArray, 1 );
-    p->n_items = 0;
-    p->n_alloc = 0;
-    p->items = NULL;
+    tr_free( p->items );
 
-    return p;
-}
-
-tr_ptrArray*
-tr_ptrArrayDup( tr_ptrArray* in )
-{
-    tr_ptrArray * out;
-
-    out = tr_new( tr_ptrArray, 1 );
-    out->n_items = out->n_alloc = in->n_items;
-    out->items = tr_memdup( in->items, out->n_items * sizeof( void* ) );
-
-    return out;
+    memset( p, ~0, sizeof( tr_ptrArray ) );
 }
 
 void
@@ -65,20 +49,6 @@ tr_ptrArrayForeach( tr_ptrArray *       t,
         func( t->items[i] );
 }
 
-void
-tr_ptrArrayFree( tr_ptrArray *       t,
-                 PtrArrayForeachFunc func )
-{
-    assert( t );
-    assert( t->items || !t->n_items );
-
-    if( func )
-        tr_ptrArrayForeach( t, func );
-
-    tr_free( t->items );
-    tr_free( t );
-}
-
 void**
 tr_ptrArrayPeek( tr_ptrArray * t,
                  int *         size )
@@ -87,12 +57,6 @@ tr_ptrArrayPeek( tr_ptrArray * t,
     return t->items;
 }
 
-void**
-tr_ptrArrayBase( tr_ptrArray * t )
-{
-    return t->items;
-}
-
 void*
 tr_ptrArrayNth( tr_ptrArray* t,
                 int          i )
@@ -104,62 +68,29 @@ tr_ptrArrayNth( tr_ptrArray* t,
     return t->items[i];
 }
 
-void*
-tr_ptrArrayBack( tr_ptrArray* t )
-{
-    assert( t->n_items > 0 );
-
-    return tr_ptrArrayNth( t, t->n_items - 1 );
-}
-
-int
-tr_ptrArraySize( const tr_ptrArray * t )
-{
-    return t->n_items;
-}
-
-int
-tr_ptrArrayEmpty( const tr_ptrArray * t )
-{
-    return t->n_items == 0;
-}
-
-void
-tr_ptrArrayClear( tr_ptrArray * t )
-{
-    t->n_items = 0;
-}
-
 int
 tr_ptrArrayInsert( tr_ptrArray * t,
-                   void *        ptr,
+                   void        * ptr,
                    int           pos )
 {
-    if( pos < 0 || pos > t->n_items )
-        pos = t->n_items;
-
     if( t->n_items >= t->n_alloc )
     {
-        t->n_alloc = t->n_items + GROW;
+        t->n_alloc = MAX( FLOOR, t->n_alloc * 2 );
         t->items = tr_renew( void*, t->items, t->n_alloc );
     }
 
-    memmove( t->items + pos + 1,
-            t->items + pos,
-            sizeof( void* ) * ( t->n_items - pos ) );
+    if( pos < 0 || pos > t->n_items )
+        pos = t->n_items;
+    else
+        memmove( t->items + pos + 1,
+                 t->items + pos,
+                 sizeof( void* ) * ( t->n_items - pos ) );
 
     t->items[pos] = ptr;
     t->n_items++;
     return pos;
 }
 
-int
-tr_ptrArrayAppend( tr_ptrArray * t,
-                   void *        ptr )
-{
-    return tr_ptrArrayInsert( t, ptr, -1 );
-}
-
 void*
 tr_ptrArrayPop( tr_ptrArray* t )
 {
@@ -171,14 +102,14 @@ tr_ptrArrayPop( tr_ptrArray* t )
     return ret;
 }
 
-void
+static void
 tr_ptrArrayErase( tr_ptrArray * t,
                   int           begin,
                   int           end )
 {
     assert( begin >= 0 );
     if( end < 0 ) end = t->n_items;
-    assert( end - begin > 0 );
+    assert( begin < end );
     assert( end <= t->n_items );
 
     memmove( t->items + begin,
@@ -197,7 +128,7 @@ tr_ptrArrayLowerBound( const tr_ptrArray *                t,
                        const void *                       ptr,
                        int                 compare( const void *,
                                                     const void * ),
-                       int *                              exact_match )
+                       tr_bool *                    exact_match )
 {
     int len = t->n_items;
     int first = 0;
@@ -215,7 +146,7 @@ tr_ptrArrayLowerBound( const tr_ptrArray *                t,
         else if( !c )
         {
             if( exact_match )
-                *exact_match = 1;
+                *exact_match = TRUE;
             return middle;
             break;
         }
@@ -226,25 +157,29 @@ tr_ptrArrayLowerBound( const tr_ptrArray *                t,
     }
 
     if( exact_match )
-        *exact_match = 0;
+        *exact_match = FALSE;
 
     return first;
 }
 
+#ifdef NDEBUG
+#define assertSortedAndUnique(a,b)
+#else
 static void
 assertSortedAndUnique( const tr_ptrArray * t,
-                        int compare(const void*, const void*) )
+                    int compare(const void*, const void*) )
 {
     int i;
 
     for( i = 0; i < t->n_items - 2; ++i )
-        assert( compare( t->items[i], t->items[i + 1] ) < 0 );
+        assert( compare( t->items[i], t->items[i + 1] ) <= 0 );
 }
+#endif
 
 int
 tr_ptrArrayInsertSorted( tr_ptrArray * t,
                          void *        ptr,
-                          int            compare(const void*, const void*) )
+                         int           compare(const void*, const void*) )
 {
     const int pos = tr_ptrArrayLowerBound( t, ptr, compare, NULL );
     const int ret = tr_ptrArrayInsert( t, ptr, pos );
@@ -256,9 +191,9 @@ tr_ptrArrayInsertSorted( tr_ptrArray * t,
 void*
 tr_ptrArrayFindSorted( tr_ptrArray * t,
                        const void *  ptr,
-                        int            compare(const void*, const void*) )
+                       int           compare(const void*, const void*) )
 {
-    int       match;
+    tr_bool   match;
     const int pos = tr_ptrArrayLowerBound( t, ptr, compare, &match );
 
     return match ? t->items[pos] : NULL;
@@ -266,11 +201,11 @@ tr_ptrArrayFindSorted( tr_ptrArray * t,
 
 void*
 tr_ptrArrayRemoveSorted( tr_ptrArray * t,
-                         void *        ptr,
-                          int            compare(const void*, const void*) )
+                         const void  * ptr,
+                         int           compare(const void*, const void*) )
 {
     void *    ret = NULL;
-    int       match;
+    tr_bool   match;
     const int pos = tr_ptrArrayLowerBound( t, ptr, compare, &match );
 
     if( match )
