@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: session.c 10303 2010-03-06 15:05:05Z charles $
+ * $Id: session.c 10602 2010-05-01 14:19:45Z charles $
  */
 
 #include <assert.h>
@@ -214,12 +214,16 @@ open_incoming_peer_port( tr_session * session )
 const tr_address*
 tr_sessionGetPublicAddress( const tr_session * session, int tr_af_type )
 {
+    const struct tr_bindinfo * bindinfo;
+
     switch( tr_af_type )
     {
-        case TR_AF_INET: return &session->public_ipv4->addr;
-        case TR_AF_INET6: return &session->public_ipv6->addr; break;
-        default: return NULL;
+        case TR_AF_INET:  bindinfo = session->public_ipv4; break;
+        case TR_AF_INET6: bindinfo = session->public_ipv6; break;
+        default:          bindinfo = NULL;                 break;
     }
+
+    return bindinfo ? &bindinfo->addr : NULL;
 }
 
 /***
@@ -233,10 +237,8 @@ tr_sessionGetPublicAddress( const tr_session * session, int tr_af_type )
 #endif
 
 void
-tr_sessionGetDefaultSettings( const char * configDir, tr_benc * d )
+tr_sessionGetDefaultSettings( const char * configDir UNUSED, tr_benc * d )
 {
-    char * incompleteDir = tr_buildPath( configDir, "Incomplete", NULL );
-
     assert( tr_bencIsDict( d ) );
 
     tr_bencDictReserve( d, 35 );
@@ -246,7 +248,7 @@ tr_sessionGetDefaultSettings( const char * configDir, tr_benc * d )
     tr_bencDictAddInt ( d, TR_PREFS_KEY_DSPEED,                   100 );
     tr_bencDictAddBool( d, TR_PREFS_KEY_DSPEED_ENABLED,           FALSE );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_ENCRYPTION,               TR_DEFAULT_ENCRYPTION );
-    tr_bencDictAddStr ( d, TR_PREFS_KEY_INCOMPLETE_DIR,           incompleteDir );
+    tr_bencDictAddStr ( d, TR_PREFS_KEY_INCOMPLETE_DIR,           tr_getDefaultDownloadDir( ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_INCOMPLETE_DIR_ENABLED,   FALSE );
     tr_bencDictAddBool( d, TR_PREFS_KEY_LAZY_BITFIELD,            TRUE );
     tr_bencDictAddInt ( d, TR_PREFS_KEY_MSGLEVEL,                 TR_MSG_INF );
@@ -260,11 +262,7 @@ tr_sessionGetDefaultSettings( const char * configDir, tr_benc * d )
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PEER_SOCKET_TOS,          atoi( TR_DEFAULT_PEER_SOCKET_TOS_STR ) );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PEX_ENABLED,              TRUE );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PORT_FORWARDING,          TRUE );
-#ifdef HAVE_FALLOCATE64
-    tr_bencDictAddInt ( d, TR_PREFS_KEY_PREALLOCATION,            TR_PREALLOCATE_FULL );
-#else
     tr_bencDictAddInt ( d, TR_PREFS_KEY_PREALLOCATION,            TR_PREALLOCATE_SPARSE );
-#endif
     tr_bencDictAddStr ( d, TR_PREFS_KEY_PROXY,                    "" );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PROXY_AUTH_ENABLED,       FALSE );
     tr_bencDictAddBool( d, TR_PREFS_KEY_PROXY_ENABLED,            FALSE );
@@ -277,7 +275,7 @@ tr_sessionGetDefaultSettings( const char * configDir, tr_benc * d )
     tr_bencDictAddBool( d, TR_PREFS_KEY_RENAME_PARTIAL_FILES,     TRUE );
     tr_bencDictAddBool( d, TR_PREFS_KEY_RPC_AUTH_REQUIRED,        FALSE );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_RPC_BIND_ADDRESS,         "0.0.0.0" );
-    tr_bencDictAddBool( d, TR_PREFS_KEY_RPC_ENABLED,              TRUE );
+    tr_bencDictAddBool( d, TR_PREFS_KEY_RPC_ENABLED,              FALSE );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_RPC_PASSWORD,             "" );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_RPC_USERNAME,             "" );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_RPC_WHITELIST,            TR_DEFAULT_RPC_WHITELIST );
@@ -296,8 +294,6 @@ tr_sessionGetDefaultSettings( const char * configDir, tr_benc * d )
     tr_bencDictAddInt ( d, TR_PREFS_KEY_UPLOAD_SLOTS_PER_TORRENT, 14 );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_BIND_ADDRESS_IPV4,        TR_DEFAULT_BIND_ADDRESS_IPV4 );
     tr_bencDictAddStr ( d, TR_PREFS_KEY_BIND_ADDRESS_IPV6,        TR_DEFAULT_BIND_ADDRESS_IPV6 );
-
-    tr_free( incompleteDir );
 }
 
 void
@@ -524,7 +520,7 @@ tr_sessionInit( const char  * tag,
     return session;
 }
 
-static void turtleCheckClock( tr_session * session, struct tr_turtle_info * t, tr_bool byUser );
+static void turtleCheckClock( tr_session * s, struct tr_turtle_info * t );
 
 static void
 onNowTimer( int foo UNUSED, short bar UNUSED, void * vsession )
@@ -548,7 +544,8 @@ onNowTimer( int foo UNUSED, short bar UNUSED, void * vsession )
 
     /* tr_session things to do once per second */
     tr_timeUpdate( tv.tv_sec );
-    turtleCheckClock( session, &session->turtle, FALSE );
+    if( session->turtle.isClockEnabled )
+        turtleCheckClock( session, &session->turtle );
 }
 
 static void loadBlocklists( tr_session * session );
@@ -628,6 +625,7 @@ tr_sessionInitImpl( void * vdata )
 }
 
 static void turtleBootstrap( tr_session *, struct tr_turtle_info * );
+static void setPeerPort( tr_session * session, tr_port port );
 
 static void
 sessionSetImpl( void * vdata )
@@ -713,7 +711,6 @@ sessionSetImpl( void * vdata )
         b.addr = tr_inaddr_any;
     b.socket = -1;
     session->public_ipv4 = tr_memdup( &b, sizeof( struct tr_bindinfo ) );
-    tr_webSetInterface( session, &session->public_ipv4->addr );
 
     str = TR_PREFS_KEY_BIND_ADDRESS_IPV6;
     tr_bencDictFindStr( settings, TR_PREFS_KEY_BIND_ADDRESS_IPV6, &str );
@@ -731,7 +728,7 @@ sessionSetImpl( void * vdata )
         tr_sessionSetPeerPortRandomOnStart( session, boolVal );
     if( !tr_bencDictFindInt( settings, TR_PREFS_KEY_PEER_PORT, &i ) )
         i = session->peerPort;
-    tr_sessionSetPeerPort( session, boolVal ? getRandomPort( session ) : i );
+    setPeerPort( session, boolVal ? getRandomPort( session ) : i );
     if( tr_bencDictFindBool( settings, TR_PREFS_KEY_PORT_FORWARDING, &boolVal ) )
         tr_sessionSetPortForwardingEnabled( session, boolVal );
 
@@ -942,7 +939,7 @@ tr_sessionIsLocked( const tr_session * session )
  **********************************************************************/
 
 static void
-setPeerPort( void * session )
+peerPortChanged( void * session )
 {
     tr_torrent * tor = NULL;
 
@@ -956,6 +953,14 @@ setPeerPort( void * session )
         tr_torrentChangeMyPort( tor );
 }
 
+static void
+setPeerPort( tr_session * session, tr_port port )
+{
+    session->peerPort = port;
+
+    tr_runInEventThread( session, peerPortChanged, session );
+}
+
 void
 tr_sessionSetPeerPort( tr_session * session, tr_port port )
 {
@@ -963,9 +968,7 @@ tr_sessionSetPeerPort( tr_session * session, tr_port port )
 
     if( session->peerPort != port )
     {
-        session->peerPort = port;
-
-        tr_runInEventThread( session, setPeerPort, session );
+        setPeerPort( session, port );
     }
 }
 
@@ -1094,51 +1097,35 @@ updateBandwidth( tr_session * session, tr_direction dir )
     tr_bandwidthSetDesiredSpeed( session->bandwidth, dir, limit );
 }
 
+enum
+{
+    MINUTES_PER_HOUR = 60,
+    MINUTES_PER_DAY = MINUTES_PER_HOUR * 24,
+    MINUTES_PER_WEEK = MINUTES_PER_DAY * 7
+};
+
 static void
-turtleFindNextChange( struct tr_turtle_info * t )
+turtleUpdateTable( struct tr_turtle_info * t )
 {
     int day;
-    struct tm tm;
-    time_t today_began_at;
-    time_t next_begin;
-    time_t next_end;
-    const time_t now = tr_time( );
-    const int SECONDS_PER_DAY = 86400;
+    tr_bitfield * b = &t->minutes;
 
-    tr_localtime_r( &now, &tm );
-    tm.tm_hour = tm.tm_min = tm.tm_sec = 0;
-    today_began_at = mktime( &tm );
+    tr_bitfieldClear( b );
 
-    next_begin = today_began_at + ( t->beginMinute * 60 );
-    if( next_begin <= now )
-        next_begin += SECONDS_PER_DAY;
+    for( day=0; day<7; ++day )
+    {
+        if( t->days & (1<<day) )
+        {
+            int i;
+            const time_t begin = t->beginMinute;
+            time_t end = t->endMinute;
 
-    next_end = today_began_at + ( t->endMinute * 60 );
-    if( next_end <= now )
-        next_end += SECONDS_PER_DAY;
+            if( end <= begin )
+                end += MINUTES_PER_DAY;
 
-    if( next_begin < next_end ) {
-        t->_nextChangeAt = next_begin;
-        t->_nextChangeValue = TRUE;
-    } else {
-        t->_nextChangeAt = next_end;
-        t->_nextChangeValue = FALSE;
-    }
-
-    /* if the next change is today, look for today in t->days.
-       if the next change is tomorrow to turn limits OFF, look for today in t->days.
-       if the next change is tomorrow to turn limits ON, look for tomorrow in t->days. */
-    if( t->_nextChangeValue && (( t->_nextChangeAt >= today_began_at + SECONDS_PER_DAY )))
-        day = ( tm.tm_wday + 1 ) % 7;
-    else
-        day = tm.tm_wday;
-    t->_nextChangeAllowed = ( t->days & (1<<day) ) != 0;
-
-    if( t->isClockEnabled && t->_nextChangeAllowed ) {
-        char buf[128];
-        tr_localtime_r( &t->_nextChangeAt, &tm );
-        strftime( buf, sizeof( buf ), "%a %b %d %T %Y", &tm );
-        tr_inf( "Turtle clock updated: at %s we'll turn limits %s", buf, (t->_nextChangeValue?"on":"off") );
+            for( i=begin; i<end; ++i )
+                tr_bitfieldAdd( b, (i+day*MINUTES_PER_DAY) % MINUTES_PER_WEEK );
+        }
     }
 }
 
@@ -1152,14 +1139,14 @@ altSpeedToggled( void * vsession )
 
     updateBandwidth( session, TR_UP );
     updateBandwidth( session, TR_DOWN );
-    turtleFindNextChange( t );
 
     if( t->callback != NULL )
         (*t->callback)( session, t->isEnabled, t->changedByUser, t->callbackUserData );
 }
 
 static void
-useAltSpeed( tr_session * s, struct tr_turtle_info * t, tr_bool enabled, tr_bool byUser )
+useAltSpeed( tr_session * s, struct tr_turtle_info * t,
+             tr_bool enabled, tr_bool byUser )
 {
     assert( tr_isSession( s ) );
     assert( t != NULL );
@@ -1174,25 +1161,54 @@ useAltSpeed( tr_session * s, struct tr_turtle_info * t, tr_bool enabled, tr_bool
     }
 }
 
+/**
+ * @param enabled whether turtle should be on/off according to the scheduler
+ * @param changed whether that's different from the previous minute
+ */
 static void
-turtleCheckClock( tr_session * session, struct tr_turtle_info * t, tr_bool byUser )
+testTurtleTime( const struct tr_turtle_info * t,
+                tr_bool * enabled,
+                tr_bool * changed )
 {
+    tr_bool e;
+    struct tm tm;
+    size_t minute_of_the_week;
     const time_t now = tr_time( );
-    const tr_bool hit = ( t->testedAt < t->_nextChangeAt ) && ( t->_nextChangeAt <= tr_time( ));
 
-    t->testedAt = now;
+    tr_localtime_r( &now, &tm );
 
-    if( hit )
+    minute_of_the_week = tm.tm_wday * MINUTES_PER_DAY
+                       + tm.tm_hour * MINUTES_PER_HOUR
+                       + tm.tm_min;
+    if( minute_of_the_week >= MINUTES_PER_WEEK ) /* leap minutes? */
+        minute_of_the_week = MINUTES_PER_WEEK - 1;
+
+    e = tr_bitfieldHasFast( &t->minutes, minute_of_the_week );
+    if( enabled != NULL )
+        *enabled = e;
+
+    if( changed != NULL )
     {
-        const tr_bool enabled = t->_nextChangeValue;
+        const size_t prev = minute_of_the_week > 0 ? minute_of_the_week - 1
+                                                   : MINUTES_PER_WEEK - 1;
+        *changed = e != tr_bitfieldHasFast( &t->minutes, prev );
+    }
+}
 
-        if( t->isClockEnabled && t->_nextChangeAllowed )
-        {
-            tr_inf( "Time to turn %s turtle mode!", (enabled?"on":"off") );
-            useAltSpeed( session, t, enabled, byUser );
-        }
+static void
+turtleCheckClock( tr_session * s, struct tr_turtle_info * t )
+{
+    tr_bool enabled;
+    tr_bool changed;
 
-        turtleFindNextChange( t );
+    assert( t->isClockEnabled );
+
+    testTurtleTime( t, &enabled, &changed );
+
+    if( changed )
+    {
+        tr_inf( "Time to turn %s turtle mode!", (enabled?"on":"off") );
+        useAltSpeed( s, t, enabled, FALSE );
     }
 }
 
@@ -1202,12 +1218,14 @@ turtleCheckClock( tr_session * session, struct tr_turtle_info * t, tr_bool byUse
 static void
 turtleBootstrap( tr_session * session, struct tr_turtle_info * turtle )
 {
-    turtleFindNextChange( turtle );
-
     turtle->changedByUser = FALSE;
 
+    tr_bitfieldConstruct( &turtle->minutes, MINUTES_PER_WEEK );
+
+    turtleUpdateTable( turtle );
+
     if( turtle->isClockEnabled )
-        turtle->isEnabled = !turtle->_nextChangeValue;
+        testTurtleTime( turtle, &turtle->isEnabled, NULL );
 
     altSpeedToggled( session );
 }
@@ -1288,11 +1306,14 @@ userPokedTheClock( tr_session * s, struct tr_turtle_info * t )
 {
     tr_dbg( "Refreshing the turtle mode clock due to user changes" );
 
-    t->testedAt = 0;
-    turtleFindNextChange( t );
+    turtleUpdateTable( t );
 
-    if( t->isClockEnabled && t->_nextChangeAllowed )
-        useAltSpeed( s, t, !t->_nextChangeValue, TRUE );
+    if( t->isClockEnabled )
+    {
+        tr_bool enabled, changed;
+        testTurtleTime( t, &enabled, &changed );
+        useAltSpeed( s, t, enabled, TRUE );
+    }
 }
 
 void
@@ -1592,6 +1613,7 @@ tr_sessionClose( tr_session * session )
     /* free the session memory */
     tr_bencFree( &session->removedTorrents );
     tr_bandwidthFree( session->bandwidth );
+    tr_bitfieldDestruct( &session->turtle.minutes );
     tr_lockFree( session->lock );
     if( session->metainfoLookup ) {
         tr_bencFree( session->metainfoLookup );
