@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: session.h 8661 2009-06-09 21:59:25Z charles $
+ * $Id: session.h 10593 2010-05-01 13:40:42Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -28,6 +28,7 @@
 #endif
 
 #include "bencode.h"
+#include "bitfield.h"
 
 typedef enum { TR_NET_OK, TR_NET_ERROR, TR_NET_WAIT } tr_tristate_t;
 
@@ -36,9 +37,48 @@ uint8_t*       tr_peerIdNew( void );
 const uint8_t* tr_getPeerId( void );
 
 struct tr_address;
+struct tr_announcer;
 struct tr_bandwidth;
 struct tr_bindsockets;
+struct tr_fdInfo;
 
+struct tr_turtle_info
+{
+    /* TR_UP and TR_DOWN speed limits */
+    int speedLimit[2];
+
+    /* is turtle mode on right now? */
+    tr_bool isEnabled;
+
+    /* does turtle mode turn itself on and off at given times? */
+    tr_bool isClockEnabled;
+
+    /* when clock mode is on, minutes after midnight to turn on turtle mode */
+    int beginMinute;
+
+    /* when clock mode is on, minutes after midnight to turn off turtle mode */
+    int endMinute;
+
+    /* only use clock mode on these days of the week */
+    tr_sched_day days;
+
+    /* called when isEnabled changes */
+    tr_altSpeedFunc * callback;
+
+    /* the callback's user_data argument */
+    void * callbackUserData;
+
+    /* the callback's changedByUser argument.
+     * indicates whether the change came from the user or from the clock. */
+    tr_bool changedByUser;
+
+    /* bitfield of all the minutes in a week.
+     * Each bit's value indicates whether the scheduler wants turtle
+     * limits on or off at that given minute in the week. */
+    tr_bitfield minutes;
+};
+
+/** @brief handle to an active libtransmission session */
 struct tr_session
 {
     tr_bool                      isPortRandom;
@@ -48,27 +88,21 @@ struct tr_session
     tr_bool                      isProxyEnabled;
     tr_bool                      isProxyAuthEnabled;
     tr_bool                      isClosed;
-    tr_bool                      isWaiting;
     tr_bool                      useLazyBitfield;
+    tr_bool                      isIncompleteFileNamingEnabled;
     tr_bool                      isRatioLimited;
+    tr_bool                      isIncompleteDirEnabled;
 
     tr_benc                      removedTorrents;
-    
+
+    int                          umask;
+
     int                          speedLimit[2];
     tr_bool                      speedLimitEnabled[2];
 
-    int                          altSpeed[2];
-    tr_bool                      altSpeedEnabled;
-
-    int                          altSpeedTimeBegin;
-    int                          altSpeedTimeEnd;
-    tr_sched_day                 altSpeedTimeDay;
-    tr_bool                      altSpeedTimeEnabled;
-    tr_bool                      altSpeedChangedByUser;
-
-    tr_altSpeedFunc            * altCallback;
-    void                       * altCallbackUserData;
+    struct tr_turtle_info        turtle;
 
+    struct tr_fdInfo           * fdInfo;
 
     int                          magicNumber;
 
@@ -79,7 +113,6 @@ struct tr_session
     struct tr_event_handle *     events;
 
     uint16_t                     peerLimitPerTorrent;
-    uint16_t                     openFileLimit;
 
     int                          uploadSlotsPerTorrent;
 
@@ -98,6 +131,7 @@ struct tr_session
     char *                       downloadDir;
     char *                       resumeDir;
     char *                       torrentDir;
+    char *                       incompleteDir;
 
     tr_proxy_type                proxyType;
     char *                       proxy;
@@ -117,17 +151,13 @@ struct tr_session
     void *                       rpc_func_user_data;
 
     struct tr_stats_handle     * sessionStats;
-    struct tr_tracker_handle   * tracker;
+
+    struct tr_announcer        * announcer;
 
     tr_benc                    * metainfoLookup;
 
-    struct event               * altTimer;
-
-    /* the size of the output buffer for peer connections */
-    int so_sndbuf;
-
-    /* the size of the input buffer for peer connections */
-    int so_rcvbuf;
+    struct event               * nowTimer;
+    struct event               * saveTimer;
 
     /* monitors the "global pool" speeds */
     struct tr_bandwidth        * bandwidth;
@@ -136,11 +166,15 @@ struct tr_session
 
     struct tr_bindinfo         * public_ipv4;
     struct tr_bindinfo         * public_ipv6;
+
+    /* a page-aligned buffer for use by the libtransmission thread.
+     * @see SESSION_BUFFER_SIZE */
+    void * buffer;
+
+    tr_bool bufferInUse;
 };
 
-tr_bool      tr_sessionGetActiveSpeedLimit( const tr_session  * session,
-                                            tr_direction        dir,
-                                            int               * setme );
+tr_bool      tr_sessionAllowsDHT( const tr_session * session );
 
 const char * tr_sessionFindTorrentFile( const tr_session * session,
                                         const char *       hashString );
@@ -152,11 +186,11 @@ void         tr_sessionSetTorrentFile( tr_session * session,
 tr_bool      tr_sessionIsAddressBlocked( const tr_session        * session,
                                          const struct tr_address * addr );
 
-void         tr_globalLock( tr_session * );
+void         tr_sessionLock( tr_session * );
 
-void         tr_globalUnlock( tr_session * );
+void         tr_sessionUnlock( tr_session * );
 
-tr_bool      tr_globalIsLocked( const tr_session * );
+tr_bool      tr_sessionIsLocked( const tr_session * );
 
 const struct tr_address*  tr_sessionGetPublicAddress( const tr_session *, int tr_af_type );
 
@@ -164,29 +198,36 @@ struct tr_bindsockets * tr_sessionGetBindSockets( tr_session * );
 
 enum
 {
-    SESSION_MAGIC_NUMBER = 3845
+    SESSION_MAGIC_NUMBER = 3845,
+
+    /* @see tr_session.buffer */
+    SESSION_BUFFER_SIZE = (16*1024)
 };
 
-static TR_INLINE tr_bool tr_isSession( const tr_session * session )
+void* tr_sessionGetBuffer( tr_session * session );
+
+void tr_sessionReleaseBuffer( tr_session * session );
+
+static inline tr_bool tr_isSession( const tr_session * session )
 {
     return ( session != NULL ) && ( session->magicNumber == SESSION_MAGIC_NUMBER );
 }
 
-static TR_INLINE tr_bool tr_isPreallocationMode( tr_preallocation_mode m  )
+static inline tr_bool tr_isPreallocationMode( tr_preallocation_mode m  )
 {
     return ( m == TR_PREALLOCATE_NONE )
         || ( m == TR_PREALLOCATE_SPARSE )
         || ( m == TR_PREALLOCATE_FULL );
 }
 
-static TR_INLINE tr_bool tr_isEncryptionMode( tr_encryption_mode m )
+static inline tr_bool tr_isEncryptionMode( tr_encryption_mode m )
 {
     return ( m == TR_CLEAR_PREFERRED )
         || ( m == TR_ENCRYPTION_PREFERRED )
         || ( m == TR_ENCRYPTION_REQUIRED );
 }
 
-static TR_INLINE tr_bool tr_isPriority( tr_priority_t p )
+static inline tr_bool tr_isPriority( tr_priority_t p )
 {
     return ( p == TR_PRI_LOW )
         || ( p == TR_PRI_NORMAL )
