@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: trevent.c 7813 2009-01-29 18:18:24Z charles $
+ * $Id: trevent.c 9992 2010-01-22 03:39:21Z charles $
  */
 
 #include <assert.h>
@@ -18,28 +18,31 @@
 
 #include <signal.h>
 
+#include <event.h>
+
 #include "transmission.h"
+#include "net.h"
 #include "session.h"
 
 #ifdef WIN32
 
-#include <WinSock2.h> 
- 
-static int 
-pgpipe( int handles[2] ) 
+#include <WinSock2.h>
+
+static int
+pgpipe( int handles[2] )
 {
         SOCKET s;
         struct sockaddr_in serv_addr;
         int len = sizeof( serv_addr );
- 
+
         handles[0] = handles[1] = INVALID_SOCKET;
- 
+
         if ( ( s = socket( AF_INET, SOCK_STREAM, 0 ) ) == INVALID_SOCKET )
         {
 /*              ereport(LOG, (errmsg_internal("pgpipe failed to create socket: %ui", WSAGetLastError()))); */
                 return -1;
         }
- 
+
         memset( &serv_addr, 0, sizeof( serv_addr ) );
         serv_addr.sin_family = AF_INET;
         serv_addr.sin_port = htons(0);
@@ -68,7 +71,7 @@ pgpipe( int handles[2] )
                 closesocket(s);
                 return -1;
         }
- 
+
         if (connect(handles[1], (SOCKADDR *) & serv_addr, len) == SOCKET_ERROR)
         {
 /*              ereport(LOG, (errmsg_internal("pgpipe failed to connect socket: %ui", WSAGetLastError()))); */
@@ -86,27 +89,27 @@ pgpipe( int handles[2] )
         closesocket(s);
         return 0;
 }
- 
-static int 
-piperead( int s, char *buf, int len ) 
-{ 
-        int ret = recv(s, buf, len, 0); 
- 
-        if (ret < 0 && WSAGetLastError() == WSAECONNRESET) 
-                /* EOF on the pipe! (win32 socket based implementation) */ 
-                ret = 0; 
-        return ret; 
-} 
- 
-#define pipe(a) pgpipe(a) 
-#define pipewrite(a,b,c) send(a,(char*)b,c,0) 
+
+static int
+piperead( int s, char *buf, int len )
+{
+        int ret = recv(s, buf, len, 0);
+
+        if (ret < 0 && WSAGetLastError() == WSAECONNRESET)
+                /* EOF on the pipe! (win32 socket based implementation) */
+                ret = 0;
+        return ret;
+}
+
+#define pipe(a) pgpipe(a)
+#define pipewrite(a,b,c) send(a,(char*)b,c,0)
 
 #else
-#define piperead(a,b,c) read(a,b,c) 
-#define pipewrite(a,b,c) write(a,b,c) 
+#define piperead(a,b,c) read(a,b,c)
+#define pipewrite(a,b,c) write(a,b,c)
 #endif
 
-#include <unistd.h> 
+#include <unistd.h>
 
 #include <event.h>
 
@@ -131,18 +134,6 @@ typedef struct tr_event_handle
 }
 tr_event_handle;
 
-typedef int timer_func ( void* );
-
-struct tr_timer
-{
-    tr_bool                   inCallback;
-    timer_func *              func;
-    void *                    user_data;
-    struct tr_event_handle *  eh;
-    struct timeval            tv;
-    struct event              event;
-};
-
 struct tr_run_data
 {
     void    ( *func )( void * );
@@ -191,19 +182,6 @@ readFromPipe( int    fd,
             break;
         }
 
-        case 't': /* create timer */
-        {
-            tr_timer *    timer;
-            const size_t  nwant = sizeof( timer );
-            const ssize_t ngot = piperead( fd, &timer, nwant );
-            if( !eh->die && ( ngot == (ssize_t)nwant ) )
-            {
-                dbgmsg( "adding timer in libevent thread" );
-                evtimer_add( &timer->event, &timer->tv );
-            }
-            break;
-        }
-
         case '\0': /* eof */
         {
             dbgmsg( "pipe eof reached... removing event listener" );
@@ -248,7 +226,8 @@ libeventThreadFunc( void * veh )
     event_set_log_callback( logFunc );
 
     /* loop until all the events are done */
-    event_dispatch( );
+    while( !eh->die )
+        event_dispatch( );
 
     /* shut down the thread */
     tr_lockFree( eh->lock );
@@ -273,7 +252,7 @@ tr_eventInit( tr_session * session )
 
     /* wait until the libevent thread is running */
     while( session->events == NULL )
-        tr_wait( 100 );
+        tr_wait_msec( 100 );
 }
 
 void
@@ -283,7 +262,7 @@ tr_eventClose( tr_session * session )
 
     session->events->die = TRUE;
     tr_deepLog( __FILE__, __LINE__, NULL, "closing trevent pipe" );
-    EVUTIL_CLOSESOCKET( session->events->fds[1] );
+    tr_netCloseSocket( session->events->fds[1] );
 }
 
 /**
@@ -291,10 +270,10 @@ tr_eventClose( tr_session * session )
 **/
 
 tr_bool
-tr_amInEventThread( tr_session * session )
+tr_amInEventThread( const tr_session * session )
 {
     assert( tr_isSession( session ) );
-    assert( session->events );
+    assert( session->events != NULL );
 
     return tr_amInThread( session->events->thread );
 }
@@ -303,80 +282,6 @@ tr_amInEventThread( tr_session * session )
 ***
 **/
 
-static void
-timerCallback( int    fd UNUSED,
-               short  event UNUSED,
-               void * vtimer )
-{
-    int               more;
-    struct tr_timer * timer = vtimer;
-
-    timer->inCallback = 1;
-    more = ( *timer->func )( timer->user_data );
-    timer->inCallback = 0;
-
-    if( more )
-        evtimer_add( &timer->event, &timer->tv );
-    else
-        tr_timerFree( &timer );
-}
-
-void
-tr_timerFree( tr_timer ** ptimer )
-{
-    tr_timer * timer;
-
-    /* zero out the argument passed in */
-    assert( ptimer );
-    timer = *ptimer;
-    *ptimer = NULL;
-
-    /* destroy the timer directly or via the command queue */
-    if( timer && !timer->inCallback )
-    {
-        assert( tr_amInEventThread( timer->eh->session ) );
-        event_del( &timer->event );
-        tr_free( timer );
-    }
-}
-
-tr_timer*
-tr_timerNew( tr_session * session,
-             timer_func   func,
-             void       * user_data,
-             uint64_t     interval_milliseconds )
-{
-    tr_timer * timer;
-
-    assert( tr_isSession( session ) );
-    assert( session->events != NULL );
-
-    timer = tr_new0( tr_timer, 1 );
-    tr_timevalMsec( interval_milliseconds, &timer->tv );
-    timer->func = func;
-    timer->user_data = user_data;
-    timer->eh = session->events;
-    evtimer_set( &timer->event, timerCallback, timer );
-
-    if( tr_amInThread( session->events->thread ) )
-    {
-        evtimer_add( &timer->event,  &timer->tv );
-    }
-    else
-    {
-        const char ch = 't';
-        int        fd = session->events->fds[1];
-        tr_lock *  lock = session->events->lock;
-
-        tr_lockLock( lock );
-        pipewrite( fd, &ch, 1 );
-        pipewrite( fd, &timer, sizeof( timer ) );
-        tr_lockUnlock( lock );
-    }
-
-    return timer;
-}
-
 void
 tr_runInEventThread( tr_session * session,
                      void func( void* ), void * user_data )
