@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: upnp.c 9868 2010-01-04 21:00:47Z charles $
+ * $Id: upnp.c 10581 2010-05-01 04:32:15Z charles $
  */
 
 #include <assert.h>
@@ -137,7 +137,7 @@ tr_upnpPulse( tr_upnp * handle,
 
         tr_snprintf( portStr, sizeof( portStr ), "%d", handle->port );
         i = UPNP_GetSpecificPortMappingEntry( handle->urls.controlURL,
-                                              handle->data.servicetype, portStr,
+                                              handle->data.first.servicetype, portStr,
                                               "TCP", intClient, intPort );
         if( i != UPNPCOMMAND_SUCCESS )
         {
@@ -151,12 +151,12 @@ tr_upnpPulse( tr_upnp * handle,
         char portStr[16];
         tr_snprintf( portStr, sizeof( portStr ), "%d", handle->port );
         UPNP_DeletePortMapping( handle->urls.controlURL,
-                                handle->data.servicetype,
+                                handle->data.first.servicetype,
                                 portStr, "TCP", NULL );
         tr_ninf( getKey( ),
                  _(
                      "Stopping port forwarding through \"%s\", service \"%s\"" ),
-                 handle->urls.controlURL, handle->data.servicetype );
+                 handle->urls.controlURL, handle->data.first.servicetype );
         handle->isMapped = 0;
         handle->state = TR_UPNP_IDLE;
         handle->port = -1;
@@ -173,7 +173,7 @@ tr_upnpPulse( tr_upnp * handle,
         int  err = -1;
         errno = 0;
 
-        if( !handle->urls.controlURL || !handle->data.servicetype )
+        if( !handle->urls.controlURL || !handle->data.first.servicetype )
             handle->isMapped = 0;
         else
         {
@@ -182,7 +182,7 @@ tr_upnpPulse( tr_upnp * handle,
             tr_snprintf( portStr, sizeof( portStr ), "%d", port );
             tr_snprintf( desc, sizeof( desc ), "%s at %d", TR_NAME, port );
             err = UPNP_AddPortMapping( handle->urls.controlURL,
-                                       handle->data.servicetype,
+                                       handle->data.first.servicetype,
                                        portStr, portStr, handle->lanaddr,
                                        desc, "TCP", NULL );
             handle->isMapped = !err;
@@ -190,7 +190,7 @@ tr_upnpPulse( tr_upnp * handle,
         tr_ninf( getKey( ),
                  _(
                      "Port forwarding through \"%s\", service \"%s\".  (local address: %s:%d)" ),
-                 handle->urls.controlURL, handle->data.servicetype,
+                 handle->urls.controlURL, handle->data.first.servicetype,
                  handle->lanaddr, port );
         if( handle->isMapped )
         {
