@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,16 +7,23 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: utils.c 8698 2009-06-15 03:24:40Z charles $
+ * $Id: utils.c 10305 2010-03-06 19:33:04Z charles $
  */
 
 #ifdef HAVE_MEMMEM
  #define _GNU_SOURCE /* glibc's string.h needs this to pick up memmem */
 #endif
 
+#if defined(SYS_DARWIN)
+ #define HAVE_GETPAGESIZE
+ #define HAVE_VALLOC
+ #undef HAVE_POSIX_MEMALIGN /* not supported on OS X 10.5 and lower */
+#endif
+
 #include <assert.h>
 #include <ctype.h> /* isalpha, tolower */
 #include <errno.h>
+#include <math.h> /* pow */
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,7 +33,7 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h> /* usleep, stat, getcwd */
+#include <unistd.h> /* usleep, stat, getcwd, getpagesize */
 
 #include "event.h"
 
@@ -39,43 +46,56 @@
 #include "fdlimit.h"
 #include "ConvertUTF.h"
 #include "list.h"
+#include "net.h"
 #include "utils.h"
 #include "platform.h"
 #include "version.h"
 
+
+int                   messageLevel = TR_MSG_INF;
 static tr_lock *      messageLock = NULL;
-static int            messageLevel = 0;
 static tr_bool        messageQueuing = FALSE;
 static tr_msg_list *  messageQueue = NULL;
 static tr_msg_list ** messageQueueTail = &messageQueue;
+static int            messageQueueCount = 0;
 
 #ifndef WIN32
     /* make null versions of these win32 functions */
-    static int IsDebuggerPresent( void ) { return FALSE; }
-    static void OutputDebugString( const void * unused UNUSED ) { }
+    static inline int IsDebuggerPresent( void ) { return FALSE; }
+    static inline void OutputDebugString( const void * unused UNUSED ) { }
 #endif
 
-static void
+/***
+****
+***/
+
+time_t transmission_now = 0;
+
+void
+tr_timeUpdate( time_t now )
+{
+    transmission_now = now;
+}
+
+/***
+****
+***/
+
+void
 tr_msgInit( void )
 {
-    static tr_bool initialized = FALSE;
-
-    if( !initialized )
-    {
-        char * env = getenv( "TR_DEBUG" );
-        messageLevel = ( env ? atoi( env ) : 0 ) + 1;
-        messageLevel = MAX( 1, messageLevel );
+    const char * env = getenv( "TR_DEBUG" );
+    messageLevel = ( env ? atoi( env ) : 0 ) + 1;
+    messageLevel = MAX( 1, messageLevel );
 
+    if( messageLock == NULL )
         messageLock = tr_lockNew( );
-
-        initialized = TRUE;
-    }
 }
 
 FILE*
 tr_getLog( void )
 {
-    static int    initialized = FALSE;
+    static tr_bool initialized = FALSE;
     static FILE * file = NULL;
 
     if( !initialized )
@@ -104,7 +124,6 @@ tr_getLog( void )
 void
 tr_setMessageLevel( int level )
 {
-    tr_msgInit( );
     tr_lockLock( messageLock );
 
     messageLevel = MAX( 0, level );
@@ -116,7 +135,6 @@ int
 tr_getMessageLevel( void )
 {
     int ret;
-    tr_msgInit( );
     tr_lockLock( messageLock );
 
     ret = messageLevel;
@@ -128,7 +146,6 @@ tr_getMessageLevel( void )
 void
 tr_setMessageQueuing( tr_bool enabled )
 {
-    tr_msgInit( );
     tr_lockLock( messageLock );
 
     messageQueuing = enabled;
@@ -140,7 +157,6 @@ tr_bool
 tr_getMessageQueuing( void )
 {
     int ret;
-    tr_msgInit( );
     tr_lockLock( messageLock );
 
     ret = messageQueuing;
@@ -153,13 +169,14 @@ tr_msg_list *
 tr_getQueuedMessages( void )
 {
     tr_msg_list * ret;
-    tr_msgInit( );
     tr_lockLock( messageLock );
 
     ret = messageQueue;
     messageQueue = NULL;
     messageQueueTail = &messageQueue;
 
+    messageQueueCount = 0;
+
     tr_lockUnlock( messageLock );
     return ret;
 }
@@ -200,15 +217,15 @@ char*
 tr_getLogTimeStr( char * buf, int buflen )
 {
     char           tmp[64];
-    time_t         now;
     struct tm      now_tm;
     struct timeval tv;
+    time_t         seconds;
     int            milliseconds;
 
-    now = time( NULL );
     gettimeofday( &tv, NULL );
 
-    tr_localtime_r( &now, &now_tm );
+    seconds = tv.tv_sec;
+    tr_localtime_r( &seconds, &now_tm );
     strftime( tmp, sizeof( tmp ), "%H:%M:%S", &now_tm );
     milliseconds = (int)( tv.tv_usec / 1000 );
     tr_snprintf( buf, buflen, "%s.%03d", tmp, milliseconds );
@@ -216,26 +233,6 @@ tr_getLogTimeStr( char * buf, int buflen )
     return buf;
 }
 
-void
-tr_assertImpl( const char * file, int line, const char * test, const char * fmt, ... )
-{
-    char buf[64];
-    fprintf( stderr, "[%s] Transmission %s Assertion \"%s\" failed at %s:%d.  ",
-                     tr_getLogTimeStr( buf, sizeof( buf ) ),
-                      LONG_VERSION_STRING, test, file, line );
-    if( fmt && *fmt ) {
-        va_list args;
-        fputc( '(', stderr );
-        va_start( args, fmt );
-        vfprintf( stderr, fmt, args );
-        va_end( args );
-        fputs( ")  ", stderr );
-    }
-    fputs( "Please report this bug at <http://trac.transmissionbt.com/newticket>; Thank you.\n", stderr );
-    abort( );
-}
-
-
 tr_bool
 tr_deepLoggingIsActive( void )
 {
@@ -283,15 +280,6 @@ tr_deepLog( const char  * file,
 /***
 ****
 ***/
-    
-
-int
-tr_msgLoggingIsActive( int level )
-{
-    tr_msgInit( );
-
-    return messageLevel >= level;
-}
 
 void
 tr_msg( const char * file, int line,
@@ -299,60 +287,71 @@ tr_msg( const char * file, int line,
         const char * fmt, ... )
 {
     const int err = errno; /* message logging shouldn't affect errno */
-    FILE * fp;
-    tr_msgInit( );
-    tr_lockLock( messageLock );
+    char buf[1024];
+    va_list ap;
 
-    fp = tr_getLog( );
+    if( messageLock != NULL )
+        tr_lockLock( messageLock );
 
-    if( messageLevel >= level )
+    /* build the text message */
+    *buf = '\0';
+    va_start( ap, fmt );
+    evutil_vsnprintf( buf, sizeof( buf ), fmt, ap );
+    va_end( ap );
+
+    OutputDebugString( buf );
+
+    if( *buf )
     {
-        char buf[MAX_STACK_ARRAY_SIZE];
-        va_list ap;
-
-        /* build the text message */
-        *buf = '\0';
-        va_start( ap, fmt );
-        evutil_vsnprintf( buf, sizeof( buf ), fmt, ap );
-        va_end( ap );
-
-        OutputDebugString( buf );
-
-        if( *buf )
+        if( messageQueuing )
         {
-            if( messageQueuing )
-            {
-                tr_msg_list * newmsg;
-                newmsg = tr_new0( tr_msg_list, 1 );
-                newmsg->level = level;
-                newmsg->when = time( NULL );
-                newmsg->message = tr_strdup( buf );
-                newmsg->file = file;
-                newmsg->line = line;
-                newmsg->name = tr_strdup( name );
+            tr_msg_list * newmsg;
+            newmsg = tr_new0( tr_msg_list, 1 );
+            newmsg->level = level;
+            newmsg->when = tr_time( );
+            newmsg->message = tr_strdup( buf );
+            newmsg->file = file;
+            newmsg->line = line;
+            newmsg->name = tr_strdup( name );
 
-                *messageQueueTail = newmsg;
-                messageQueueTail = &newmsg->next;
+            *messageQueueTail = newmsg;
+            messageQueueTail = &newmsg->next;
+            ++messageQueueCount;
+
+            if( messageQueueCount > TR_MAX_MSG_LOG )
+            {
+                tr_msg_list * old = messageQueue;
+                messageQueue = old->next;
+                old->next = NULL;
+                tr_freeMessageList(old);
+
+                --messageQueueCount;
+
+                assert( messageQueueCount == TR_MAX_MSG_LOG );
             }
+        }
+        else
+        {
+            char timestr[64];
+            FILE * fp;
+
+            fp = tr_getLog( );
+            if( fp == NULL )
+                fp = stderr;
+
+            tr_getLogTimeStr( timestr, sizeof( timestr ) );
+
+            if( name )
+                fprintf( fp, "[%s] %s: %s\n", timestr, name, buf );
             else
-            {
-                char timestr[64];
-
-                if( fp == NULL )
-                    fp = stderr;
-
-                tr_getLogTimeStr( timestr, sizeof( timestr ) );
-
-                if( name )
-                    fprintf( fp, "[%s] %s: %s\n", timestr, name, buf );
-                else
-                    fprintf( fp, "[%s] %s\n", timestr, buf );
-                fflush( fp );
-            }
+                fprintf( fp, "[%s] %s\n", timestr, buf );
+            fflush( fp );
         }
     }
 
-    tr_lockUnlock( messageLock );
+    if( messageLock != NULL )
+        tr_lockUnlock( messageLock );
+
     errno = err;
 }
 
@@ -422,6 +421,7 @@ tr_set_compare( const void * va,
 const char*
 tr_strip_positional_args( const char* str )
 {
+    const char * in = str;
     static size_t bufsize = 0;
     static char * buf = NULL;
     const size_t  len = strlen( str );
@@ -448,7 +448,7 @@ tr_strip_positional_args( const char* str )
     }
     *out = '\0';
 
-    return buf;
+    return strcmp( buf, in ) ? buf : in;
 }
 
 #endif
@@ -457,38 +457,26 @@ tr_strip_positional_args( const char* str )
 ***
 **/
 
-tr_bool
-tr_isTimeval( const struct timeval * tv )
-{
-    return tv && ( tv->tv_sec >= 0 )
-              && ( tv->tv_usec >= 0 )
-              && ( tv->tv_usec < 1000000 );
-}
-
 void
-tr_timevalMsec( uint64_t milliseconds, struct timeval * setme )
-{
-    const uint64_t microseconds = milliseconds * 1000;
-    assert( setme != NULL );
-    setme->tv_sec  = microseconds / 1000000;
-    setme->tv_usec = microseconds % 1000000;
-    assert( tr_isTimeval( setme ) );
-}
-
-void
-tr_timevalSet( struct timeval * setme, int seconds, int microseconds )
-{
-    setme->tv_sec = seconds;
-    setme->tv_usec = microseconds;
-    assert( tr_isTimeval( setme ) );
-}
-
-void
-tr_timerAdd( struct event * timer, int seconds, int milliseconds )
+tr_timerAdd( struct event * timer, int seconds, int microseconds )
 {
     struct timeval tv;
-    tr_timevalSet( &tv, seconds, milliseconds );
-    event_add( timer, &tv );
+    tv.tv_sec = seconds;
+    tv.tv_usec = microseconds;
+
+    assert( tv.tv_sec >= 0 );
+    assert( tv.tv_usec >= 0 );
+    assert( tv.tv_usec < 1000000 );
+
+    evtimer_add( timer, &tv );
+}
+
+void
+tr_timerAddMsec( struct event * timer, int msec )
+{
+    const int seconds =  msec / 1000;
+    const int usec = (msec%1000) * 1000;
+    tr_timerAdd( timer, seconds, usec );
 }
 
 /**
@@ -784,8 +772,6 @@ tr_strstrip( char * str )
         while( len && isspace( str[len - 1] ) )
             --len;
 
-        str[len] = '\0';
-
         for( pos = 0; pos < len && isspace( str[pos] ); )
             ++pos;
 
@@ -811,7 +797,7 @@ tr_date( void )
 }
 
 void
-tr_wait( uint64_t delay_milliseconds )
+tr_wait_msec( uint64_t delay_milliseconds )
 {
 #ifdef WIN32
     Sleep( (DWORD)delay_milliseconds );
@@ -825,10 +811,7 @@ tr_wait( uint64_t delay_milliseconds )
 ***/
 
 int
-tr_snprintf( char *       buf,
-             size_t       buflen,
-             const char * fmt,
-             ... )
+tr_snprintf( char * buf, size_t buflen, const char * fmt, ... )
 {
     int     len;
     va_list args;
@@ -903,29 +886,43 @@ tr_getRatio( double numerator,
 }
 
 void
-tr_sha1_to_hex( char *          out,
-                const uint8_t * sha1 )
+tr_sha1_to_hex( char * out, const uint8_t * sha1 )
 {
+    int i;
     static const char hex[] = "0123456789abcdef";
-    int               i;
 
-    for( i = 0; i < 20; i++ )
+    for( i=0; i<20; ++i )
     {
-        unsigned int val = *sha1++;
+        const unsigned int val = *sha1++;
         *out++ = hex[val >> 4];
         *out++ = hex[val & 0xf];
     }
+
     *out = '\0';
 }
 
+void
+tr_hex_to_sha1( uint8_t * out, const char * in )
+{
+    int i;
+    static const char hex[] = "0123456789abcdef";
+
+    for( i=0; i<20; ++i )
+    {
+        const int hi = strchr( hex, tolower( *in++ ) ) - hex;
+        const int lo = strchr( hex, tolower( *in++ ) ) - hex;
+        *out++ = (uint8_t)( (hi<<4) | lo );
+    }
+}
+
 /***
 ****
 ***/
 
-int
-tr_httpIsValidURL( const char * url )
+static tr_bool
+isValidURLChars( const char * url )
 {
-    const char *        c;
+    const char * c;
     static const char * rfc2396_valid_chars =
         "abcdefghijklmnopqrstuvwxyz" /* lowalpha */
         "ABCDEFGHIJKLMNOPQRSTUVWXYZ" /* upalpha */
@@ -942,15 +939,55 @@ tr_httpIsValidURL( const char * url )
         if( !strchr( rfc2396_valid_chars, *c ) )
             return FALSE;
 
-    return !tr_httpParseURL( url, -1, NULL, NULL, NULL );
+    return TRUE;
+}
+
+/** @brief return TRUE if the url is a http or https url that Transmission understands */
+tr_bool
+tr_urlIsValidTracker( const char * url )
+{
+    tr_bool valid;
+    char * scheme = NULL;
+
+    valid = isValidURLChars( url ) 
+         && !tr_urlParse( url, -1, &scheme, NULL, NULL, NULL )
+         && ( scheme != NULL )
+         && ( !strcmp(scheme,"http") || !strcmp(scheme,"https") );
+
+    tr_free( scheme );
+    return valid;
+}
+
+/** @brief return TRUE if the url is a http or https or ftp or sftp url that Transmission understands */
+tr_bool
+tr_urlIsValid( const char * url )
+{
+    tr_bool valid;
+    char * scheme = NULL;
+
+    valid = isValidURLChars( url ) 
+         && !tr_urlParse( url, -1, &scheme, NULL, NULL, NULL )
+         && ( scheme != NULL )
+         && ( !strcmp(scheme,"http") || !strcmp(scheme,"https") || !strcmp(scheme,"ftp") || !strcmp(scheme,"sftp") );
+
+    tr_free( scheme );
+    return valid;
+}
+
+tr_bool
+tr_addressIsIP( const char * address )
+{
+    tr_address tempAddr;
+    return tr_pton(address, &tempAddr) != NULL;
 }
 
 int
-tr_httpParseURL( const char * url_in,
-                 int          len,
-                 char **      setme_host,
-                 int *        setme_port,
-                 char **      setme_path )
+tr_urlParse( const char * url_in,
+             int          len,
+             char **      setme_protocol,
+             char **      setme_host,
+             int *        setme_port,
+             char **      setme_path )
 {
     int          err;
     int          port = 0;
@@ -967,8 +1004,7 @@ tr_httpParseURL( const char * url_in,
         *pch = '\0';
         protocol = tmp;
         pch += 3;
-/*fprintf( stderr, "protocol is [%s]... what's left is [%s]\n", protocol, pch
-  );*/
+/*fprintf( stderr, "protocol is [%s]... what's left is [%s]\n", protocol, pch);*/
         if( ( n = strcspn( pch, ":/" ) ) )
         {
             const int havePort = pch[n] == ':';
@@ -988,21 +1024,24 @@ tr_httpParseURL( const char * url_in,
         }
     }
 
-    err = !host || !path || !protocol
-          || ( strcmp( protocol, "http" ) && strcmp( protocol, "https" ) );
+    err = !host || !path || !protocol;
 
     if( !err && !port )
     {
+        if( !strcmp( protocol, "ftp" ) ) port = 21;
+        if( !strcmp( protocol, "sftp" ) ) port = 22;
         if( !strcmp( protocol, "http" ) ) port = 80;
         if( !strcmp( protocol, "https" ) ) port = 443;
     }
 
     if( !err )
     {
+        if( setme_protocol ) *setme_protocol = tr_strdup( protocol );
+
         if( setme_host ){ ( (char*)host )[-3] = ':'; *setme_host =
-                              tr_strdup( protocol ); }
-        if( setme_path ){ ( (char*)path )[-1] = '/'; *setme_path =
-                              tr_strdup( path - 1 ); }
+                              tr_strdup( host ); }
+        if( setme_path ){ if( path[0] == '/' ) *setme_path = tr_strdup( path );
+                          else { ( (char*)path )[-1] = '/'; *setme_path = tr_strdup( path - 1 ); } }
         if( setme_port ) *setme_port = port;
     }
 
@@ -1019,29 +1058,34 @@ tr_httpParseURL( const char * url_in,
 #include <openssl/buffer.h>
 
 char *
-tr_base64_encode( const void * input,
-                  int          length,
-                  int *        setme_len )
+tr_base64_encode( const void * input, int length, int * setme_len )
 {
-    char *    ret;
-    BIO *     b64;
-    BIO *     bmem;
-    BUF_MEM * bptr;
+    int retlen = 0;
+    char * ret = NULL;
 
-    if( length < 1 )
-        length = strlen( input );
+    if( input != NULL )
+    {
+        BIO * b64;
+        BIO * bmem;
+        BUF_MEM * bptr;
+
+        if( length < 1 )
+            length = strlen( input );
+
+        bmem = BIO_new( BIO_s_mem( ) );
+        b64 = BIO_new( BIO_f_base64( ) );
+        b64 = BIO_push( b64, bmem );
+        BIO_write( b64, input, length );
+        (void) BIO_flush( b64 );
+        BIO_get_mem_ptr( b64, &bptr );
+        ret = tr_strndup( bptr->data, bptr->length );
+        retlen = bptr->length;
+        BIO_free_all( b64 );
+    }
 
-    bmem = BIO_new( BIO_s_mem( ) );
-    b64 = BIO_new( BIO_f_base64( ) );
-    b64 = BIO_push( b64, bmem );
-    BIO_write( b64, input, length );
-    (void) BIO_flush( b64 );
-    BIO_get_mem_ptr( b64, &bptr );
-    ret = tr_strndup( bptr->data, bptr->length );
     if( setme_len )
-        *setme_len = bptr->length;
+        *setme_len = retlen;
 
-    BIO_free_all( b64 );
     return ret;
 }
 
@@ -1081,18 +1125,6 @@ tr_base64_decode( const void * input,
     return ret;
 }
 
-int
-tr_ptr2int( void* v )
-{
-    return (intptr_t)v;
-}
-
-void*
-tr_int2ptr( int i )
-{
-    return (void*)(intptr_t)i;
-}
-
 /***
 ****
 ***/
@@ -1107,32 +1139,25 @@ tr_lowerBound( const void * key,
 {
     size_t first = 0;
     const char * cbase = base;
+    tr_bool exact = FALSE;
 
-    while( nmemb )
+    while( nmemb != 0 )
     {
         const size_t half = nmemb / 2;
         const size_t middle = first + half;
         const int c = compar( key, cbase + size*middle );
 
-        if( c < 0 )
-        {
+        if( c <= 0 ) {
+            if( c == 0 )
+                exact = TRUE;
+            nmemb = half;
+        } else {
             first = middle + 1;
             nmemb = nmemb - half - 1;
         }
-        else if( !c )
-        {
-            if( exact_match )
-                *exact_match = TRUE;
-            return middle;
-        }
-        else
-        {
-            nmemb = half;
-        }
     }
 
-    if( exact_match )
-        *exact_match = FALSE;
+    *exact_match = exact;
 
     return first;
 }
@@ -1246,7 +1271,7 @@ compareInt( const void * va, const void * vb )
  * Given a string like "1-4" or "1-4,6,9,14-51", this allocates and returns an
  * array of setmeCount ints of all the values in the array.
  * For example, "5-8" will return [ 5, 6, 7, 8 ] and setmeCount will be 4.
- * It's the caller's responsibility to call tr_free() on the returned array. 
+ * It's the caller's responsibility to call tr_free() on the returned array.
  * If a fragment of the string can't be parsed, NULL is returned.
  */
 int*
@@ -1324,20 +1349,13 @@ tr_parseNumberRange( const char * str_in, int len, int * setmeCount )
 ****
 ***/
 
-static void
-printf_double_without_rounding( char * buf, int buflen, double d, int places )
+double
+tr_truncd( double x, int decimal_places )
 {
-    char * pch;
-    char tmp[128];
-    int len;
-    tr_snprintf( tmp, sizeof( tmp ), "%'.64f", d );
-    pch = tmp;
-    while( isdigit( *pch ) ) ++pch; /* walk to the decimal point */
-    ++pch; /* walk over the decimal point */
-    pch += places;
-    len = MIN( buflen - 1, pch - tmp );
-    memcpy( buf, tmp, len );
-    buf[len] = '\0';
+    static const int multiplier[] = { 1, 10, 100, 1000, 10000, 100000, 1000000, 10000000 };
+    const int64_t i = multiplier[decimal_places];
+    double x2 = (int64_t)(x*i);
+    return x2 / i;
 }
 
 char*
@@ -1348,9 +1366,9 @@ tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity )
     else if( (int)ratio == TR_RATIO_INF )
         tr_strlcpy( buf, infinity, buflen );
     else if( ratio < 10.0 )
-        printf_double_without_rounding( buf, buflen, ratio, 2 );
+        tr_snprintf( buf, buflen, "%.2f", tr_truncd( ratio, 2 ) );
     else if( ratio < 100.0 )
-        printf_double_without_rounding( buf, buflen, ratio, 1 );
+        tr_snprintf( buf, buflen, "%.1f", tr_truncd( ratio, 1 ) );
     else
         tr_snprintf( buf, buflen, "%'.0f", ratio );
     return buf;
@@ -1361,14 +1379,14 @@ tr_strratio( char * buf, size_t buflen, double ratio, const char * infinity )
 ***/
 
 int
-tr_moveFile( const char * oldpath, const char * newpath )
+tr_moveFile( const char * oldpath, const char * newpath, tr_bool * renamed )
 {
     int in;
     int out;
     char * buf;
     struct stat st;
-    size_t bytesLeft;
-    size_t buflen;
+    off_t bytesLeft;
+    off_t buflen;
 
     /* make sure the old file exists */
     if( stat( oldpath, &st ) ) {
@@ -1391,20 +1409,24 @@ tr_moveFile( const char * oldpath, const char * newpath )
             return i;
     }
 
-    /* they  might be on the same filesystem... */
-    if( !rename( oldpath, newpath ) )
-        return 0;
+    /* they might be on the same filesystem... */
+    {
+        const int i = rename( oldpath, newpath );
+        if( renamed != NULL )
+            *renamed = i == 0;
+        if( !i )
+            return 0;
+    }
 
     /* copy the file */
     in = tr_open_file_for_scanning( oldpath );
-    tr_preallocate_file( newpath, bytesLeft );
     out = tr_open_file_for_writing( newpath );
     buflen = stat( newpath, &st ) ? 4096 : st.st_blksize;
-    buf = tr_new( char, buflen );
+    buf = tr_valloc( buflen );
     while( bytesLeft > 0 )
     {
         ssize_t bytesWritten;
-        const size_t bytesThisPass = MIN( bytesLeft, buflen );
+        const off_t bytesThisPass = MIN( bytesLeft, buflen );
         const int numRead = read( in, buf, bytesThisPass );
         if( numRead < 0 )
             break;
@@ -1424,3 +1446,41 @@ tr_moveFile( const char * oldpath, const char * newpath )
     unlink( oldpath );
     return 0;
 }
+
+/***
+****
+***/
+
+void*
+tr_valloc( size_t bufLen )
+{
+    size_t allocLen;
+    void * buf = NULL;
+    static size_t pageSize = 0;
+
+    if( !pageSize ) {
+#ifdef HAVE_GETPAGESIZE
+        pageSize = getpagesize();
+#else /* guess */
+        pageSize = 4096;
+#endif
+    }
+
+    allocLen = pageSize;
+    while( allocLen < bufLen )
+        allocLen += pageSize;
+
+#ifdef HAVE_POSIX_MEMALIGN
+    if( !buf )
+        posix_memalign( &buf, pageSize, allocLen );
+#endif
+#ifdef HAVE_VALLOC
+    if( !buf )
+        buf = valloc( allocLen );
+#endif
+    if( !buf )
+        buf = malloc( allocLen );
+
+    tr_dbg( "tr_valloc(%zu) allocating %zu bytes", bufLen, allocLen );
+    return buf;
+}
