@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: verify.h 7664 2009-01-11 17:46:51Z charles $
+ * $Id: verify.h 9868 2010-01-04 21:00:47Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -17,6 +17,11 @@
 #ifndef TR_VERIFY_H
 #define TR_VERIFY_H 1
 
+/**
+ * @addtogroup file_io File IO
+ * @{
+ */
+
 typedef void ( *tr_verify_done_cb )( tr_torrent * tor );
 
 void tr_verifyAdd( tr_torrent *      tor,
@@ -24,6 +29,10 @@ void tr_verifyAdd( tr_torrent *      tor,
 
 void tr_verifyRemove( tr_torrent * tor );
 
-int tr_verifyInProgress( const tr_torrent * tor );
+tr_bool tr_verifyInProgress( const tr_torrent * tor );
+
+void tr_verifyClose( tr_session * );
+
+/* @} */
 
 #endif
