@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008 Charles Kerr <charles@rebelbase.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,17 +7,15 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: webseed.c 7455 2008-12-22 00:51:14Z charles $
+ * $Id: webseed.c 10031 2010-01-28 13:33:40Z charles $
  */
 
-#include <assert.h>
 #include <string.h> /* strlen */
 
 #include <event.h>
 
 #include "transmission.h"
 #include "inout.h"
-#include "list.h"
 #include "ratecontrol.h"
 #include "torrent.h"
 #include "utils.h"
@@ -40,7 +38,7 @@ struct tr_webseed
     uint32_t            pieceOffset;
     uint32_t            byteCount;
 
-    tr_ratecontrol    * rateDown;
+    tr_ratecontrol      rateDown;
 
     tr_session        * session;
 
@@ -51,7 +49,7 @@ struct tr_webseed
 ****
 ***/
 
-static const tr_peer_event blankEvent = { 0, 0, 0, 0, 0.0f, 0, 0, 0 };
+static const tr_peer_event blankEvent = { 0, 0, 0, 0, 0.0f, 0, 0, 0, 0 };
 
 static void
 publish( tr_webseed *    w,
@@ -64,9 +62,11 @@ publish( tr_webseed *    w,
 static void
 fireNeedReq( tr_webseed * w )
 {
+#if 0
     tr_peer_event e = blankEvent;
     e.eventType = TR_PEER_NEED_REQ;
     publish( w, &e );
+#endif
 }
 
 static void
@@ -106,38 +106,8 @@ makeURL( tr_webseed *    w,
     evbuffer_add( out, url, url_len );
 
     /* if url ends with a '/', add the torrent name */
-    if( url[url_len - 1] == '/' )
-    {
-        const char * str = file->name;
-
-        /* this is like curl_escape() but doesn't munge the
-         * '/' directory separators in the path */
-        while( str && *str )
-        {
-            switch( *str )
-            {
-                case ',': case '-': case '.': case '/':
-                case '0': case '1': case '2': case '3': case '4':
-                case '5': case '6': case '7': case '8': case '9':
-                case 'a': case 'b': case 'c': case 'd': case 'e':
-                case 'f': case 'g': case 'h': case 'i': case 'j':
-                case 'k': case 'l': case 'm': case 'n': case 'o':
-                case 'p': case 'q': case 'r': case 's': case 't':
-                case 'u': case 'v': case 'w': case 'x': case 'y': case 'z':
-                case 'A': case 'B': case 'C': case 'D': case 'E':
-                case 'F': case 'G': case 'H': case 'I': case 'J':
-                case 'K': case 'L': case 'M': case 'N': case 'O':
-                case 'P': case 'Q': case 'R': case 'S': case 'T':
-                case 'U': case 'V': case 'W': case 'X': case 'Y': case 'Z':
-                    evbuffer_add( out, str, 1 );
-                    break;
-                default:
-                    evbuffer_add_printf( out, "%%%02X", *str );
-                    break;
-            }
-            str++;
-        }
-    }
+    if( url[url_len - 1] == '/' && file->name )
+        tr_http_escape( out, file->name, strlen(file->name), FALSE );
 
     ret = tr_strndup( EVBUFFER_DATA( out ), EVBUFFER_LENGTH( out ) );
     evbuffer_free( out );
@@ -169,7 +139,7 @@ webResponseFunc( tr_session    * session,
         if( !w->dead )
         {
             fireClientGotData( w, response_byte_count );
-            tr_rcTransferred( w->rateDown, response_byte_count );
+            tr_rcTransferred( &w->rateDown, response_byte_count );
         }
 
         if( EVBUFFER_LENGTH( w->content ) < w->byteCount )
@@ -252,11 +222,11 @@ tr_webseedIsActive( const tr_webseed * w )
 }
 
 int
-tr_webseedGetSpeed( const tr_webseed * w, float * setme_KiBs )
+tr_webseedGetSpeed( const tr_webseed * w, uint64_t now, float * setme_KiBs )
 {
     const int isActive = tr_webseedIsActive( w );
 
-    *setme_KiBs = isActive ? tr_rcRate( w->rateDown ) : 0.0f;
+    *setme_KiBs = isActive ? tr_rcRate( &w->rateDown, now ) : 0.0f;
     return isActive;
 }
 
@@ -275,10 +245,10 @@ tr_webseedNew( struct tr_torrent * torrent,
     memcpy( w->hash, torrent->info.hash, SHA_DIGEST_LENGTH );
     w->session = torrent->session;
     w->content = evbuffer_new( );
-    w->rateDown = tr_rcInit( );
     w->url = tr_strdup( url );
     w->callback = callback;
     w->callback_userdata = callback_userdata;
+    tr_rcConstruct( &w->rateDown );
 /*fprintf( stderr, "w->callback_userdata is %p\n", w->callback_userdata );*/
     return w;
 }
@@ -295,7 +265,7 @@ tr_webseedFree( tr_webseed * w )
         else
         {
             evbuffer_free( w->content );
-            tr_rcClose( w->rateDown );
+            tr_rcDestruct( &w->rateDown );
             tr_free( w->url );
             tr_free( w );
         }
