@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: AddWindowController.m 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id: AddWindowController.m 10345 2010-03-10 03:45:15Z livings124 $
  *
- * Copyright (c) 2008-2009 Transmission authors and contributors
+ * Copyright (c) 2008-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,16 +23,23 @@
  *****************************************************************************/
 
 #import "AddWindowController.h"
-#import "FileOutlineController.h"
 #import "Controller.h"
+#import "ExpandedPathToIconTransformer.h"
+#import "FileOutlineController.h"
 #import "GroupsController.h"
 #import "NSStringAdditions.h"
-#import "ExpandedPathToIconTransformer.h"
+#import "Torrent.h"
 
 #define UPDATE_SECONDS 1.0
 
+#define POPUP_PRIORITY_HIGH 0
+#define POPUP_PRIORITY_NORMAL 1
+#define POPUP_PRIORITY_LOW 2
+
 @interface AddWindowController (Private)
 
+- (void) updateFiles;
+
 - (void) confirmAdd;
 
 - (void) setDestinationPath: (NSString *) destination;
@@ -49,22 +56,25 @@
 @implementation AddWindowController
 
 - (id) initWithTorrent: (Torrent *) torrent destination: (NSString *) path lockDestination: (BOOL) lockDestination
-    controller: (Controller *) controller deleteTorrent: (torrentFileState) deleteTorrent
+    controller: (Controller *) controller torrentFile: (NSString *) torrentFile
+    deleteTorrent: (BOOL) deleteTorrent canToggleDelete: (BOOL) canToggleDelete
 {
     if ((self = [super initWithWindowNibName: @"AddWindow"]))
     {
         fTorrent = torrent;
-        if (path)
-            fDestination = [[path stringByExpandingTildeInPath] retain];
+        fDestination = [[path stringByExpandingTildeInPath] retain];
         fLockDestination = lockDestination;
         
         fController = controller;
         
-        fDeleteTorrent = deleteTorrent == TORRENT_FILE_DELETE || (deleteTorrent == TORRENT_FILE_DEFAULT
-                            && [[NSUserDefaults standardUserDefaults] boolForKey: @"DeleteOriginalTorrent"]);
-        fDeleteEnable = deleteTorrent == TORRENT_FILE_DEFAULT;
+        fTorrentFile = [[torrentFile stringByExpandingTildeInPath] retain];
+        
+        fDeleteTorrent = deleteTorrent;
+        fDeleteEnable = canToggleDelete;
         
         fGroupValue = [torrent groupValue];
+        
+        [fVerifyIndicator setUsesThreadedAnimation: YES];
     }
     return self;
 }
@@ -83,16 +93,23 @@
     [[self window] setTitle: name];
     [fNameField setStringValue: name];
     
-    NSImage * icon = [[fTorrent icon] copy];
-    [icon setFlipped: NO];
-    [fIconView setImage: icon];
-    [icon release];
+    [fIconView setImage: [fTorrent icon]];
     
     [self updateStatusField: nil];
     
     [self setGroupsMenu];
     [fGroupPopUp selectItemWithTag: fGroupValue];
     
+    NSInteger priorityIndex;
+    switch ([fTorrent priority])
+    {
+        case TR_PRI_HIGH: priorityIndex = POPUP_PRIORITY_HIGH; break;
+        case TR_PRI_NORMAL: priorityIndex = POPUP_PRIORITY_NORMAL; break;
+        case TR_PRI_LOW: priorityIndex = POPUP_PRIORITY_LOW; break;
+        default: NSAssert1(NO, @"Unknown priority for adding torrent: %d", [fTorrent priority]);
+    }
+    [fPriorityPopUp selectItemAtIndex: priorityIndex];
+    
     [fStartCheck setState: [[NSUserDefaults standardUserDefaults] boolForKey: @"AutoStartDownload"] ? NSOnState : NSOffState];
     
     [fDeleteCheck setState: fDeleteTorrent ? NSOnState : NSOffState];
@@ -106,8 +123,9 @@
         [fLocationImageView setImage: nil];
     }
     
-    fTimer = [NSTimer scheduledTimerWithTimeInterval: UPDATE_SECONDS target: fFileController
-                selector: @selector(reloadData) userInfo: nil repeats: YES];
+    fTimer = [NSTimer scheduledTimerWithTimeInterval: UPDATE_SECONDS target: self
+                selector: @selector(updateFiles) userInfo: nil repeats: YES];
+    [self updateFiles];
 }
 
 - (void) windowDidLoad
@@ -124,6 +142,7 @@
     [fTimer invalidate];
     
     [fDestination release];
+    [fTorrentFile release];
     
     [super dealloc];
 }
@@ -192,7 +211,20 @@
 - (void) verifyLocalData: (id) sender
 {
     [fTorrent resetCache];
-    [fFileController reloadData];
+    [self updateFiles];
+}
+
+- (void) changePriority: (id) sender
+{
+    tr_priority_t priority;
+    switch ([sender indexOfSelectedItem])
+    {
+        case POPUP_PRIORITY_HIGH: priority = TR_PRI_HIGH; break;
+        case POPUP_PRIORITY_NORMAL: priority = TR_PRI_NORMAL; break;
+        case POPUP_PRIORITY_LOW: priority = TR_PRI_LOW; break;
+        default: NSAssert1(NO, @"Unknown priority tag for adding torrent: %d", [sender tag]);
+    }
+    [fTorrent setPriority: priority];
 }
 
 - (void) updateStatusField: (NSNotification *) notification
@@ -230,6 +262,30 @@
 
 @implementation AddWindowController (Private)
 
+- (void) updateFiles
+{
+    [fTorrent update];
+    
+    [fFileController reloadData];
+    
+    if ([fTorrent isChecking])
+    {
+        const BOOL waiting = [fTorrent isCheckingWaiting];
+        [fVerifyIndicator setIndeterminate: waiting];
+        if (!waiting)
+            [fVerifyIndicator setDoubleValue: [fTorrent checkingProgress]];
+        else
+            [fVerifyIndicator startAnimation: self];
+        
+        [fVerifyIndicator setHidden: NO];
+    }
+    else
+    {
+        [fVerifyIndicator stopAnimation: self];
+        [fVerifyIndicator setHidden: YES];
+    }
+}
+
 - (void) confirmAdd
 {
     [fTimer invalidate];
@@ -238,8 +294,8 @@
     [fTorrent setWaitToStart: [fStartCheck state] == NSOnState];
     [fTorrent setGroupValue: fGroupValue];
     
-    if ([fDeleteCheck state] == NSOnState)
-        [fTorrent trashTorrent];
+    if (fTorrentFile && [fDeleteCheck state] == NSOnState)
+        [Torrent trashFile: fTorrentFile];
     
     [fFileController setTorrent: nil]; //avoid a crash when window tries to update
     
@@ -255,7 +311,7 @@
         [fDestination release];
         fDestination = [destination retain];
         
-        [fTorrent changeDownloadFolder: fDestination];
+        [fTorrent changeDownloadFolderBeforeUsing: fDestination];
     }
     
     [fLocationField setStringValue: [fDestination stringByAbbreviatingWithTildeInPath]];
