@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: CreatorWindowController.m 8675 2009-06-11 23:02:54Z livings124 $
+ * $Id: CreatorWindowController.m 10240 2010-02-20 15:57:34Z charles $
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,7 +24,7 @@
 
 #import "CreatorWindowController.h"
 #import "NSStringAdditions.h"
-#import "utils.h" //tr_httpIsValidURL
+#import "utils.h" //tr_urlIsValidTracker
 
 #define TRACKER_ADD_TAG 0
 #define TRACKER_REMOVE_TAG 1
@@ -32,7 +32,6 @@
 @interface CreatorWindowController (Private)
 
 + (NSString *) chooseFile;
-- (void) updateEnableOpenCheckForTrackers;
 - (void) locationSheetClosed: (NSSavePanel *) openPanel returnCode: (NSInteger) code contextInfo: (void *) info;
 
 - (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo;
@@ -126,7 +125,7 @@
         //remove potentially invalid addresses
         for (NSInteger i = [fTrackers count]-1; i >= 0; i--)
         {
-            if (!tr_httpIsValidURL([[fTrackers objectAtIndex: i] UTF8String]))
+            if (!tr_urlIsValidTracker([[fTrackers objectAtIndex: i] UTF8String]))
                 [fTrackers removeObjectAtIndex: i];
         }
     }
@@ -142,7 +141,7 @@
     [fNameField setStringValue: name];
     [fNameField setToolTip: fPath];
     
-    BOOL multifile = !fInfo->isSingleFile;
+    const BOOL multifile = !fInfo->isSingleFile;
     
     NSImage * icon = [[NSWorkspace sharedWorkspace] iconForFileType: multifile
                         ? NSFileTypeForHFSTypeCode('fldr') : [fPath pathExtension]];
@@ -162,15 +161,12 @@
     }
     [fStatusField setStringValue: statusString];
     
-    NSString * piecesCountString;
-    int piecesCount = fInfo->pieceCount;
-    if (piecesCount == 1)
-        piecesCountString = NSLocalizedString(@"1 piece", "Create torrent -> info");
+    if (fInfo->pieceCount == 1)
+        [fPiecesField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"1 piece, %@", "Create torrent -> info"),
+                                                            [NSString stringForFileSize: fInfo->pieceSize]]];
     else
-        piecesCountString = [NSString stringWithFormat: NSLocalizedString(@"%d pieces", "Create torrent -> info"),
-                                                                piecesCount];
-    [fPiecesField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@, %@ each", "Create torrent -> info"),
-                                        piecesCountString, [NSString stringForFileSize: fInfo->pieceSize]]];
+        [fPiecesField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%d pieces, %@ each", "Create torrent -> info"),
+                                                            fInfo->pieceCount, [NSString stringForFileSize: fInfo->pieceSize]]];
     
     fLocation = [[[[fDefaults stringForKey: @"CreatorLocation"] stringByExpandingTildeInPath] stringByAppendingPathComponent:
                     [name stringByAppendingPathExtension: @"torrent"]] retain];
@@ -181,8 +177,7 @@
     if ([fDefaults objectForKey: @"CreatorPrivate"])
         [fPrivateCheck setState: [fDefaults boolForKey: @"CreatorPrivate"] ? NSOnState : NSOffState];
     
-    fOpenTorrent = [fDefaults boolForKey: @"CreatorOpen"];
-    [self updateEnableOpenCheckForTrackers];
+    [fOpenCheck setState: [fDefaults boolForKey: @"CreatorOpen"] ? NSOnState : NSOffState];
 }
 
 - (void) dealloc
@@ -200,11 +195,6 @@
     [super dealloc];
 }
 
-- (void) toggleOpenCheck: (id) sender
-{
-    fOpenTorrent = [fOpenCheck state] == NSOnState;
-}
-
 - (void) setLocation: (id) sender
 {
     NSSavePanel * panel = [NSSavePanel savePanel];
@@ -223,13 +213,26 @@
 
 - (void) create: (id) sender
 {
-    if ([fTrackers count] == 0 && [fDefaults boolForKey: @"WarningCreatorBlankAddress"])
+    //make sure the trackers are no longer being verified
+    if ([fTrackerTable editedRow] != -1)
+        [[self window] endEditingFor: fTrackerTable];
+    
+    const BOOL isPrivate = [fPrivateCheck state] == NSOnState;
+    if ([fTrackers count] == 0
+        && [fDefaults boolForKey: isPrivate ? @"WarningCreatorPrivateBlankAddress" : @"WarningCreatorBlankAddress"])
     {
         NSAlert * alert = [[NSAlert alloc] init];
-        [alert setMessageText: NSLocalizedString(@"There is no tracker address.", "Create torrent -> blank address -> title")];
-        [alert setInformativeText: NSLocalizedString(@"The torrent file will not be able to be opened."
-            " A torrent file with no tracker address is only useful when you plan to upload the file to a tracker website"
-            " that will add the address for you.", "Create torrent -> blank address -> message")];
+        [alert setMessageText: NSLocalizedString(@"There are no tracker addresses.", "Create torrent -> blank address -> title")];
+        
+        NSString * infoString = isPrivate
+                    ? NSLocalizedString(@"A transfer marked as private with no tracker addresses will be unable to connect to peers."
+                        " The torrent file will only be useful if you plan to upload the file to a tracker website"
+                        " that will add the addresses for you.", "Create torrent -> blank address -> message")
+                    : NSLocalizedString(@"The transfer will not contact trackers for peers,"
+                        " and will have to rely solely on DHT and PEX to download and seed.",
+                        "Create torrent -> blank address -> message");
+        
+        [alert setInformativeText: infoString];
         [alert addButtonWithTitle: NSLocalizedString(@"Create", "Create torrent -> blank address -> button")];
         [alert addButtonWithTitle: NSLocalizedString(@"Cancel", "Create torrent -> blank address -> button")];
         [alert setShowsSuppressionButton: YES];
@@ -279,16 +282,14 @@
         
         [fTrackerTable deselectAll: self];
         [fTrackerTable reloadData];
-        
-        [self updateEnableOpenCheckForTrackers];
     }
     else
     {
         [fTrackers addObject: @""];
         [fTrackerTable reloadData];
         
-        NSInteger row = [fTrackers count] - 1;
-        [fTrackerTable selectRow: row byExtendingSelection: NO];
+        const NSInteger row = [fTrackers count] - 1;
+        [fTrackerTable selectRowIndexes: [NSIndexSet indexSetWithIndex: row] byExtendingSelection: NO];
         [fTrackerTable editColumn: 0 row: row withEvent: nil select: YES];
     }
 }
@@ -303,16 +304,13 @@
     if ([tracker rangeOfString: @"://"].location == NSNotFound)
         tracker = [@"http://" stringByAppendingString: tracker];
     
-    if (!tr_httpIsValidURL([tracker UTF8String]))
+    if (!tr_urlIsValidTracker([tracker UTF8String]))
     {
         NSBeep();
         [fTrackers removeObjectAtIndex: row];
     }
     else
-    {
         [fTrackers replaceObjectAtIndex: row withObject: tracker];
-        [self updateEnableOpenCheckForTrackers];
-    }
     
     [fTrackerTable deselectAll: self];
     [fTrackerTable reloadData];
@@ -344,13 +342,6 @@
     return success ? [[panel filenames] objectAtIndex: 0] : nil;
 }
 
-- (void) updateEnableOpenCheckForTrackers
-{
-    BOOL hasTracker = [fTrackers count] > 0;
-    [fOpenCheck setEnabled: hasTracker];
-    [fOpenCheck setState: (fOpenTorrent && hasTracker) ? NSOnState : NSOffState];
-}
-
 - (void) locationSheetClosed: (NSSavePanel *) panel returnCode: (NSInteger) code contextInfo: (void *) info
 {
     if (code == NSOKButton)
@@ -366,7 +357,11 @@
 - (void) createBlankAddressAlertDidEnd: (NSAlert *) alert returnCode: (NSInteger) returnCode contextInfo: (void *) contextInfo
 {
     if ([[alert suppressionButton] state] == NSOnState)
-        [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorBlankAddress"];
+    {
+        [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorBlankAddress"]; //set regardless of private/public
+        if ([fPrivateCheck state] == NSOnState)
+            [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningCreatorPrivateBlankAddress"];
+    }
     
     [alert release];
     
@@ -376,6 +371,24 @@
 
 - (void) createReal
 {
+    //check if the location currently exists
+    if (![[NSFileManager defaultManager] fileExistsAtPath: [fLocation stringByDeletingLastPathComponent]])
+    {
+        NSAlert * alert = [[[NSAlert alloc] init] autorelease];
+        [alert addButtonWithTitle: NSLocalizedString(@"OK", "Create torrent -> directory doesn't exist warning -> button")];
+        [alert setMessageText: NSLocalizedString(@"The chosen torrent file location does not exist.",
+                                                "Create torrent -> directory doesn't exist warning -> title")];
+        [alert setInformativeText: [NSString stringWithFormat:
+                NSLocalizedString(@"The directory \"%@\" does not currently exist. "
+                    "Create this directory or choose a different one to create the torrent file.",
+                    "Create torrent -> directory doesn't exist warning -> warning"),
+                    [fLocation stringByDeletingLastPathComponent]]];
+        [alert setAlertStyle: NSWarningAlertStyle];
+        
+        [alert beginSheetModalForWindow: [self window] modalDelegate: self didEndSelector: nil contextInfo: nil];
+        return;
+    }
+    
     //check if a file with the same name and location already exists
     if ([[NSFileManager defaultManager] fileExistsAtPath: fLocation])
     {
@@ -401,12 +414,15 @@
     tr_tracker_info * trackerInfo = tr_new0(tr_tracker_info, [fTrackers count]);
     
     for (NSUInteger i = 0; i < [fTrackers count]; i++)
+    {
         trackerInfo[i].announce = (char *)[[fTrackers objectAtIndex: i] UTF8String];
+        trackerInfo[i].tier = i;
+    }
     
     //store values
     [fDefaults setObject: fTrackers forKey: @"CreatorTrackers"];
     [fDefaults setBool: [fPrivateCheck state] == NSOnState forKey: @"CreatorPrivate"];
-    [fDefaults setBool: fOpenTorrent forKey: @"CreatorOpen"];
+    [fDefaults setBool: [fOpenCheck state] == NSOnState forKey: @"CreatorOpen"];
     [fDefaults setObject: [fLocation stringByDeletingLastPathComponent] forKey: @"CreatorLocation"];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"BeginCreateTorrentFile" object: fLocation userInfo: nil];
@@ -429,7 +445,7 @@
         switch (fInfo->result)
         {
             case TR_MAKEMETA_OK:
-                if (fOpenTorrent && [fTrackers count] > 0)
+                if ([fDefaults boolForKey: @"CreatorOpen"])
                 {
                     NSDictionary * dict = [[NSDictionary alloc] initWithObjectsAndKeys: fLocation, @"File",
                                             [fPath stringByDeletingLastPathComponent], @"Path", nil];
@@ -475,6 +491,7 @@
             [fProgressView setHidden: YES];
             
             NSWindow * window = [self window];
+            [window setFrameAutosaveName: @""];
             
             NSRect windowRect = [window frame];
             CGFloat difference = [fProgressView frame].size.height - [[window contentView] frame].size.height;
