@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: DragOverlayView.m 7013 2008-11-01 22:08:02Z livings124 $
+ * $Id: DragOverlayView.m 10029 2010-01-28 03:48:21Z livings124 $
  *
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,10 +23,9 @@
  *****************************************************************************/
 
 #import "DragOverlayView.h"
-#import "NSBezierPathAdditions.h"
 
-#define PADDING 10.0f
-#define ICON_WIDTH 64.0f
+#define PADDING 10.0
+#define ICON_WIDTH 64.0
 
 @implementation DragOverlayView
 
@@ -34,26 +33,13 @@
 {
     if ((self = [super initWithFrame: frame]))
     {
-        //create badge
-        NSRect badgeRect = NSMakeRect(0.0f, 0.0f, 325.0f, 84.0f);
-        NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: badgeRect radius: 15.0f];
-        
-        fBackBadge = [[NSImage alloc] initWithSize: badgeRect.size];
-        [fBackBadge lockFocus];
-        
-        [[NSColor colorWithCalibratedWhite: 0.0f alpha: 0.75f] set];
-        [bp fill];
-        
-        [fBackBadge unlockFocus];
-        
         //create attributes
         NSShadow * stringShadow = [[NSShadow alloc] init];
-        [stringShadow setShadowOffset: NSMakeSize(2.0f, -2.0f)];
-        [stringShadow setShadowBlurRadius: 4.0f];
+        [stringShadow setShadowOffset: NSMakeSize(2.0, -2.0)];
+        [stringShadow setShadowBlurRadius: 4.0];
         
-        NSFont * bigFont = [[NSFontManager sharedFontManager] convertFont:
-                                [NSFont fontWithName: @"Lucida Grande" size: 18.0f] toHaveTrait: NSBoldFontMask],
-                * smallFont = [NSFont fontWithName: @"Lucida Grande" size: 14.0f];
+        NSFont * bigFont = [NSFont boldSystemFontOfSize: 18.0],
+                * smallFont = [NSFont systemFontOfSize: 14.0];
         
         NSMutableParagraphStyle * paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
         [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
@@ -76,7 +62,6 @@
 
 - (void) dealloc
 {
-    [fBackBadge release];
     [fBadge release];
     
     [fMainLineAttributes release];
@@ -88,26 +73,32 @@
 - (void) setOverlay: (NSImage *) icon mainLine: (NSString *) mainLine subLine: (NSString *) subLine
 {
     [fBadge release];
-    fBadge = [fBackBadge copy];
-    NSSize badgeSize = [fBadge size];
     
+    //create badge
+    const NSRect badgeRect = NSMakeRect(0.0, 0.0, 325.0, 84.0);
+    
+    fBadge = [[NSImage alloc] initWithSize: badgeRect.size];
     [fBadge lockFocus];
     
+    NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: badgeRect xRadius: 15.0 yRadius: 15.0];
+    [[NSColor colorWithCalibratedWhite: 0.0 alpha: 0.75] set];
+    [bp fill];
+    
     //place icon
-    [icon drawInRect: NSMakeRect(PADDING, (badgeSize.height - ICON_WIDTH) * 0.5f, ICON_WIDTH, ICON_WIDTH) fromRect: NSZeroRect
-            operation: NSCompositeSourceOver fraction: 1.0f];
+    [icon drawInRect: NSMakeRect(PADDING, (NSHeight(badgeRect) - ICON_WIDTH) * 0.5, ICON_WIDTH, ICON_WIDTH) fromRect: NSZeroRect
+            operation: NSCompositeSourceOver fraction: 1.0];
     
     //place main text
-    NSSize mainLineSize = [mainLine sizeWithAttributes: fMainLineAttributes];
-    NSSize subLineSize = [subLine sizeWithAttributes: fSubLineAttributes];
+    const NSSize mainLineSize = [mainLine sizeWithAttributes: fMainLineAttributes];
+    const NSSize subLineSize = [subLine sizeWithAttributes: fSubLineAttributes];
     
-    NSRect lineRect = NSMakeRect(PADDING + ICON_WIDTH + 5.0f,
-                        (badgeSize.height + (subLineSize.height + 2.0f - mainLineSize.height)) * 0.5f,
-                        badgeSize.width - (PADDING + ICON_WIDTH + 2.0f) - PADDING, mainLineSize.height);
+    NSRect lineRect = NSMakeRect(PADDING + ICON_WIDTH + 5.0,
+                        (NSHeight(badgeRect) + (subLineSize.height + 2.0 - mainLineSize.height)) * 0.5,
+                        NSWidth(badgeRect) - (PADDING + ICON_WIDTH + 2.0) - PADDING, mainLineSize.height);
     [mainLine drawInRect: lineRect withAttributes: fMainLineAttributes];
     
     //place sub text
-    lineRect.origin.y -= subLineSize.height + 2.0f;
+    lineRect.origin.y -= subLineSize.height + 2.0;
     lineRect.size.height = subLineSize.height;
     [subLine drawInRect: lineRect withAttributes: fSubLineAttributes];
     
@@ -120,10 +111,10 @@
 {
     if (fBadge)
     {
-        NSRect frame = [self frame];
-        NSSize imageSize = [fBadge size];
-        [fBadge compositeToPoint: NSMakePoint((frame.size.width - imageSize.width) * 0.5f,
-                    (frame.size.height - imageSize.height) * 0.5f) operation: NSCompositeSourceOver];
+        const NSRect frame = [self frame];
+        const NSSize imageSize = [fBadge size];
+        [fBadge compositeToPoint: NSMakePoint((NSWidth(frame) - imageSize.width) * 0.5,
+                    (NSHeight(frame) - imageSize.height) * 0.5) operation: NSCompositeSourceOver];
     }
 }
 
