@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: StatsWindowController.m 5671 2008-04-23 12:44:11Z livings124 $
+ * $Id: StatsWindowController.m 9844 2010-01-01 21:12:04Z livings124 $
  *
- * Copyright (c) 2007-2008 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,7 +24,6 @@
 
 #import "StatsWindowController.h"
 #import "NSStringAdditions.h"
-#import "NSApplicationAdditions.h"
 
 #define UPDATE_SECONDS 1.0
 
@@ -33,15 +32,15 @@
 - (void) updateStats;
 
 - (void) performResetStats;
-- (void) resetSheetClosed: (NSAlert *) alert returnCode: (int) code contextInfo: (void *) info;
+- (void) resetSheetClosed: (NSAlert *) alert returnCode: (NSInteger) code contextInfo: (void *) info;
 
 @end
 
 @implementation StatsWindowController
 
 StatsWindowController * fStatsWindowInstance = nil;
-tr_handle * fLib;
-+ (StatsWindowController *) statsWindow: (tr_handle *) lib
+tr_session * fLib;
++ (StatsWindowController *) statsWindow: (tr_session *) lib
 {
     if (!fStatsWindowInstance)
     {
@@ -73,7 +72,7 @@ tr_handle * fLib;
                                             stringByAppendingString: @":"]];
     
     //size all elements
-    float oldWidth = [fUploadedLabelField frame].size.width;
+    const CGFloat oldWidth = [fUploadedLabelField frame].size.width;
     
     [fUploadedLabelField sizeToFit];
     [fDownloadedLabelField sizeToFit];
@@ -81,7 +80,7 @@ tr_handle * fLib;
     [fTimeLabelField sizeToFit];
     [fNumOpenedLabelField sizeToFit];
     
-    float maxWidth = MAX([fUploadedLabelField frame].size.width, [fDownloadedLabelField frame].size.width);
+    CGFloat maxWidth = MAX([fUploadedLabelField frame].size.width, [fDownloadedLabelField frame].size.width);
     maxWidth = MAX(maxWidth, [fRatioLabelField frame].size.width);
     maxWidth = MAX(maxWidth, [fTimeLabelField frame].size.width);
     maxWidth = MAX(maxWidth, [fNumOpenedLabelField frame].size.width);
@@ -112,7 +111,7 @@ tr_handle * fLib;
     [[self window] setFrame: windowRect display: YES];
     
     //resize reset button
-    float oldButtonWidth = [fResetButton frame].size.width;
+    const CGFloat oldButtonWidth = [fResetButton frame].size.width;
     
     [fResetButton setTitle: NSLocalizedString(@"Reset", "Stats window -> reset button")];
     [fResetButton sizeToFit];
@@ -142,20 +141,21 @@ tr_handle * fLib;
     NSAlert * alert = [[NSAlert alloc] init];
     [alert setMessageText: NSLocalizedString(@"Are you sure you want to reset usage statistics?", "Stats reset -> title")];
     [alert setInformativeText: NSLocalizedString(@"This will clear the global statistics displayed by Transmission."
-                                " Individual transfer statistics will not be effected.", "Stats reset -> message")];
+                                " Individual transfer statistics will not be affected.", "Stats reset -> message")];
     [alert setAlertStyle: NSWarningAlertStyle];
     [alert addButtonWithTitle: NSLocalizedString(@"Reset", "Stats reset -> button")];
     [alert addButtonWithTitle: NSLocalizedString(@"Cancel", "Stats reset -> button")];
-    
-    if ([NSApp isOnLeopardOrBetter])
-        [alert setShowsSuppressionButton: YES];
-    else
-        [alert addButtonWithTitle: NSLocalizedString(@"Don't Alert Again", "Stats reset -> button")];
+    [alert setShowsSuppressionButton: YES];
     
     [alert beginSheetModalForWindow: [self window] modalDelegate: self
         didEndSelector: @selector(resetSheetClosed:returnCode:contextInfo:) contextInfo: nil];
 }
 
+- (NSString *) windowFrameAutosaveName
+{
+    return @"StatsWindow";
+}
+
 @end
 
 @implementation StatsWindowController (Private)
@@ -163,23 +163,23 @@ tr_handle * fLib;
 - (void) updateStats
 {
     tr_session_stats statsAll, statsSession;
-    tr_getCumulativeSessionStats(fLib, &statsAll);
-    tr_getSessionStats(fLib, &statsSession);
+    tr_sessionGetCumulativeStats(fLib, &statsAll);
+    tr_sessionGetStats(fLib, &statsSession);
     
     [fUploadedField setStringValue: [NSString stringForFileSize: statsSession.uploadedBytes]];
-    [fUploadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fUploadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                     statsSession.uploadedBytes]];
     [fUploadedAllField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@ total", "stats total"),
                                         [NSString stringForFileSize: statsAll.uploadedBytes]]];
-    [fUploadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fUploadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                     statsAll.uploadedBytes]];
     
     [fDownloadedField setStringValue: [NSString stringForFileSize: statsSession.downloadedBytes]];
-    [fDownloadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fDownloadedField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                     statsSession.downloadedBytes]];
     [fDownloadedAllField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%@ total", "stats total"),
                                         [NSString stringForFileSize: statsAll.downloadedBytes]]];
-    [fDownloadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%u bytes", "stats -> bytes"),
+    [fDownloadedAllField setToolTip: [NSString stringWithFormat: NSLocalizedString(@"%llu bytes", "stats -> bytes"),
                                         statsAll.downloadedBytes]];
     
     [fRatioField setStringValue: [NSString stringForRatio: statsSession.ratio]];
@@ -196,21 +196,21 @@ tr_handle * fLib;
     if (statsAll.sessionCount == 1)
         [fNumOpenedField setStringValue: NSLocalizedString(@"1 time", "stats window -> times opened")];
     else
-        [fNumOpenedField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%d times", "stats window -> times opened"),
-                                        statsAll.sessionCount]];
+        [fNumOpenedField setStringValue: [NSString stringWithFormat: NSLocalizedString(@"%llu times", "stats window -> times opened"),
+                                            statsAll.sessionCount]];
 }
 
 - (void) performResetStats
 {
-    tr_clearSessionStats(fLib);
+    tr_sessionClearStats(fLib);
     [self updateStats];
 }
 
-- (void) resetSheetClosed: (NSAlert *) alert returnCode: (int) code contextInfo: (void *) info
+- (void) resetSheetClosed: (NSAlert *) alert returnCode: (NSInteger) code contextInfo: (void *) info
 {
     [[alert window] orderOut: nil];
     
-    if (([NSApp isOnLeopardOrBetter] ? [[alert suppressionButton] state] == NSOnState : code == NSAlertThirdButtonReturn))
+    if ([[alert suppressionButton] state] == NSOnState)
         [[NSUserDefaults standardUserDefaults] setBool: NO forKey: @"WarningResetStats"];
     
     if (code == NSAlertFirstButtonReturn)
