@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentCell.m 8260 2009-04-20 02:45:27Z livings124 $
+ * $Id: TorrentCell.m 10331 2010-03-08 04:18:55Z livings124 $
  *
- * Copyright (c) 2006-2009 Transmission authors and contributors
+ * Copyright (c) 2006-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,39 +23,42 @@
  *****************************************************************************/
 
 #import "TorrentCell.h"
-#import "TorrentTableView.h"
 #import "GroupsController.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "ProgressGradients.h"
+#import "Torrent.h"
+#import "TorrentTableView.h"
 
-#define BAR_HEIGHT 12.0f
+#define BAR_HEIGHT 12.0
 
-#define IMAGE_SIZE_REG 32.0f
-#define IMAGE_SIZE_MIN 16.0f
+#define IMAGE_SIZE_REG 32.0
+#define IMAGE_SIZE_MIN 16.0
+#define ERROR_IMAGE_SIZE 20.0
 
-#define NORMAL_BUTTON_WIDTH 14.0f
-#define ACTION_BUTTON_WIDTH 16.0f
+#define NORMAL_BUTTON_WIDTH 14.0
+#define ACTION_BUTTON_WIDTH 16.0
 
-#define PRIORITY_ICON_WIDTH 14.0f
-#define PRIORITY_ICON_HEIGHT 14.0f
+#define PRIORITY_ICON_WIDTH 14.0
+#define PRIORITY_ICON_HEIGHT 14.0
 
 //ends up being larger than font height
-#define HEIGHT_TITLE 16.0f
-#define HEIGHT_STATUS 12.0f
+#define HEIGHT_TITLE 16.0
+#define HEIGHT_STATUS 12.0
 
-#define PADDING_HORIZONTAL 3.0f
-#define PADDING_BETWEEN_IMAGE_AND_TITLE 5.0f
-#define PADDING_BETWEEN_IMAGE_AND_BAR 7.0f
-#define PADDING_BETWEEN_TITLE_AND_PRIORITY 4.0f
-#define PADDING_ABOVE_TITLE 4.0f
-#define PADDING_ABOVE_MIN_STATUS 4.0f
-#define PADDING_BETWEEN_TITLE_AND_MIN_STATUS 2.0f
-#define PADDING_BETWEEN_TITLE_AND_PROGRESS 1.0f
-#define PADDING_BETWEEN_PROGRESS_AND_BAR 2.0f
-#define PADDING_BETWEEN_TITLE_AND_BAR_MIN 3.0f
-#define PADDING_BETWEEN_BAR_AND_STATUS 2.0f
+#define PADDING_HORIZONTAL 3.0
+#define PADDING_BETWEEN_IMAGE_AND_TITLE 5.0
+#define PADDING_BETWEEN_IMAGE_AND_BAR 7.0
+#define PADDING_BETWEEN_TITLE_AND_PRIORITY 4.0
+#define PADDING_ABOVE_TITLE 4.0
+#define PADDING_ABOVE_MIN_STATUS 4.0
+#define PADDING_BETWEEN_TITLE_AND_MIN_STATUS 2.0
+#define PADDING_BETWEEN_TITLE_AND_PROGRESS 1.0
+#define PADDING_BETWEEN_PROGRESS_AND_BAR 2.0
+#define PADDING_BETWEEN_TITLE_AND_BAR_MIN 3.0
+#define PADDING_BETWEEN_BAR_AND_STATUS 2.0
 
-#define PIECES_TOTAL_PERCENT 0.6f
+#define PIECES_TOTAL_PERCENT 0.6
 
 #define MAX_PIECES (18*18)
 
@@ -66,18 +69,24 @@
 - (void) drawPiecesBar: (NSRect) barRect;
 
 - (NSRect) rectForMinimalStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds;
-- (NSRect) rectForTitleWithString: (NSAttributedString *) string basedOnMinimalStatusRect: (NSRect) statusRect
-            inBounds: (NSRect) bounds;
-- (NSRect) rectForProgressWithString: (NSAttributedString *) string inBounds: (NSRect) bounds;
-- (NSRect) rectForStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds;
+- (NSRect) rectForTitleWithString: (NSAttributedString *) string basedOnMinimalStatusRect: (NSRect) statusRect inBounds: (NSRect) bounds;
+- (NSRect) rectForProgressWithStringInBounds: (NSRect) bounds;
+- (NSRect) rectForStatusWithStringInBounds: (NSRect) bounds;
+- (NSRect) barRectForBounds: (NSRect) bounds;
 
-- (NSAttributedString *) attributedTitleWithColor: (NSColor *) color;
-- (NSAttributedString *) attributedStatusString: (NSString *) string withColor: (NSColor *) color;
+- (NSRect) controlButtonRectForBounds: (NSRect) bounds;
+- (NSRect) revealButtonRectForBounds: (NSRect) bounds;
+- (NSRect) actionButtonRectForBounds: (NSRect) bounds;
+
+- (NSAttributedString *) attributedTitle;
+- (NSAttributedString *) attributedStatusString: (NSString *) string;
 
 - (NSString *) buttonString;
 - (NSString *) statusString;
 - (NSString *) minimalStatusString;
 
+- (void) drawImage: (NSImage *) image inRect: (NSRect) rect; //use until 10.5 dropped
+
 @end
 
 @implementation TorrentCell
@@ -91,134 +100,29 @@
         
         NSMutableParagraphStyle * paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
         [paragraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
-    
-        fTitleAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                                [NSFont messageFontOfSize: 12.0f], NSFontAttributeName,
-                                paragraphStyle, NSParagraphStyleAttributeName, nil];
-        fStatusAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
-                                [NSFont messageFontOfSize: 9.0f], NSFontAttributeName,
-                                paragraphStyle, NSParagraphStyleAttributeName, nil];
+        
+        fTitleAttributes = [[NSMutableDictionary alloc] initWithCapacity: 3];
+        [fTitleAttributes setObject: [NSFont messageFontOfSize: 12.0] forKey: NSFontAttributeName];
+        [fTitleAttributes setObject: paragraphStyle forKey: NSParagraphStyleAttributeName];
+        
+        fStatusAttributes = [[NSMutableDictionary alloc] initWithCapacity: 3];
+        [fStatusAttributes setObject: [NSFont messageFontOfSize: 9.0] forKey: NSFontAttributeName];
+        [fStatusAttributes setObject: paragraphStyle forKey: NSParagraphStyleAttributeName];
+        
         [paragraphStyle release];
         
-        fBluePieceColor = [[NSColor colorWithCalibratedRed: 0.0f green: 0.4f blue: 0.8f alpha: 1.0f] retain];
-        fBarBorderColor = [[NSColor colorWithCalibratedWhite: 0.0f alpha: 0.2f] retain];
-        
-        fHighPriorityImage = [[NSImage imageNamed: @"PriorityHigh.png"] copy];
-        [fHighPriorityImage setFlipped: YES];
-        
-        fLowPriorityImage = [[NSImage imageNamed: @"PriorityLow.png"] copy];
-        [fLowPriorityImage setFlipped: YES];
+        fBluePieceColor = [[NSColor colorWithCalibratedRed: 0.0 green: 0.4 blue: 0.8 alpha: 1.0] retain];
+        fBarBorderColor = [[NSColor colorWithCalibratedWhite: 0.0 alpha: 0.2] retain];
     }
 	return self;
 }
 
 - (NSRect) iconRectForBounds: (NSRect) bounds
 {
-    CGFloat imageSize = [fDefaults boolForKey: @"SmallView"] ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG;
+    const CGFloat imageSize = [fDefaults boolForKey: @"SmallView"] ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG;
     
-    NSRect result = bounds;
-    result.origin.x += PADDING_HORIZONTAL;
-    result.origin.y += floorf((result.size.height - imageSize) * 0.5f);
-    result.size = NSMakeSize(imageSize, imageSize);
-    
-    return result;
-}
-
-- (NSRect) titleRectForBounds: (NSRect) bounds
-{
-    return [self rectForTitleWithString: [self attributedTitleWithColor: nil]
-            basedOnMinimalStatusRect: [self minimalStatusRectForBounds: bounds] inBounds: bounds];
-}
-
-- (NSRect) minimalStatusRectForBounds: (NSRect) bounds
-{
-    if (![fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
-    return [self rectForMinimalStatusWithString: [self attributedStatusString: [self minimalStatusString] withColor: nil]
-            inBounds: bounds];
-}
-
-- (NSRect) progressRectForBounds: (NSRect) bounds
-{
-    if ([fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
-    return [self rectForProgressWithString: [self attributedStatusString: [[self representedObject] progressString] withColor: nil]
-            inBounds: bounds];
-}
-
-- (NSRect) barRectForBounds: (NSRect) bounds
-{
-    BOOL minimal = [fDefaults boolForKey: @"SmallView"];
-    
-    NSRect result = bounds;
-    result.size.height = BAR_HEIGHT;
-    result.origin.x += (minimal ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG) + PADDING_BETWEEN_IMAGE_AND_BAR;
-    
-    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE;
-    if (minimal)
-        result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
-    else
-        result.origin.y += PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS + PADDING_BETWEEN_PROGRESS_AND_BAR;
-    
-    result.size.width = round(NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL - 2.0f * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH));
-    
-    return result;
-}
-
-- (NSRect) statusRectForBounds: (NSRect) bounds
-{
-    if ([fDefaults boolForKey: @"SmallView"])
-        return NSZeroRect;
-    
-    return [self rectForStatusWithString: [self attributedStatusString: [self statusString] withColor: nil] inBounds: bounds];
-}
-
-- (NSRect) controlButtonRectForBounds: (NSRect) bounds
-{
-    NSRect result = bounds;
-    result.size.height = NORMAL_BUTTON_WIDTH;
-    result.size.width = NORMAL_BUTTON_WIDTH;
-    result.origin.x = NSMaxX(bounds) - 2.0f * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH);
-    
-    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5f;
-    if ([fDefaults boolForKey: @"SmallView"])
-        result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
-    else
-        result.origin.y += PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS + PADDING_BETWEEN_PROGRESS_AND_BAR;
-    
-    return result;
-}
-
-- (NSRect) revealButtonRectForBounds: (NSRect) bounds
-{
-    NSRect result = bounds;
-    result.size.height = NORMAL_BUTTON_WIDTH;
-    result.size.width = NORMAL_BUTTON_WIDTH;
-    result.origin.x = NSMaxX(bounds) - (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH);
-    
-    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5f;
-    if ([fDefaults boolForKey: @"SmallView"])
-        result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
-    else
-        result.origin.y += PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS + PADDING_BETWEEN_PROGRESS_AND_BAR;
-    
-    return result;
-}
-
-- (NSRect) actionButtonRectForBounds: (NSRect) bounds
-{
-    NSRect result = [self iconRectForBounds: bounds];
-    if (![fDefaults boolForKey: @"SmallView"])
-    {
-        result.origin.x += (result.size.width - ACTION_BUTTON_WIDTH) * 0.5f;
-        result.origin.y += (result.size.height - ACTION_BUTTON_WIDTH) * 0.5f;
-        result.size.width = ACTION_BUTTON_WIDTH;
-        result.size.height = ACTION_BUTTON_WIDTH;
-    }
-    
-    return result;
+    return NSMakeRect(NSMinX(bounds) + PADDING_HORIZONTAL, floor(NSMidY(bounds) - imageSize * 0.5),
+                        imageSize, imageSize);
 }
 
 - (NSUInteger) hitTestForEvent: (NSEvent *) event inRect: (NSRect) cellFrame ofView: (NSView *) controlView
@@ -245,11 +149,11 @@
     
     NSPoint point = [controlView convertPoint: [event locationInWindow] fromView: nil];
     
-    NSRect controlRect= [self controlButtonRectForBounds: cellFrame];
-    BOOL checkControl = NSMouseInRect(point, controlRect, [controlView isFlipped]);
+    const NSRect controlRect= [self controlButtonRectForBounds: cellFrame];
+    const BOOL checkControl = NSMouseInRect(point, controlRect, [controlView isFlipped]);
     
-    NSRect revealRect = [self revealButtonRectForBounds: cellFrame];
-    BOOL checkReveal = NSMouseInRect(point, revealRect, [controlView isFlipped]);
+    const NSRect revealRect = [self revealButtonRectForBounds: cellFrame];
+    const BOOL checkReveal = NSMouseInRect(point, revealRect, [controlView isFlipped]);
     
     [(TorrentTableView *)controlView removeButtonTrackingAreas];
     
@@ -259,7 +163,7 @@
         
         if (checkControl)
         {
-            BOOL inControlButton = NSMouseInRect(point, controlRect, [controlView isFlipped]);
+            const BOOL inControlButton = NSMouseInRect(point, controlRect, [controlView isFlipped]);
             if (fMouseDownControlButton != inControlButton)
             {
                 fMouseDownControlButton = inControlButton;
@@ -268,7 +172,7 @@
         }
         else if (checkReveal)
         {
-            BOOL inRevealButton = NSMouseInRect(point, revealRect, [controlView isFlipped]);
+            const BOOL inRevealButton = NSMouseInRect(point, revealRect, [controlView isFlipped]);
             if (fMouseDownRevealButton != inRevealButton)
             {
                 fMouseDownRevealButton = inRevealButton;
@@ -297,7 +201,21 @@
         fMouseDownRevealButton = NO;
         [controlView setNeedsDisplayInRect: cellFrame];
         
-        [[self representedObject] revealData];
+        if ([NSApp isOnSnowLeopardOrBetter])
+        {
+            NSString * location = [[self representedObject] dataLocation];
+            if (location)
+            {
+                NSURL * file = [NSURL fileURLWithPath: location];
+                [[NSWorkspace sharedWorkspace] activateFileViewerSelectingURLs: [NSArray arrayWithObject: file]];
+            }
+        }
+        else
+        {
+            NSString * location = [[self representedObject] dataLocation];
+            if (location)
+                [[NSWorkspace sharedWorkspace] selectFile: location inFileViewerRootedAtPath: nil];
+        }
     }
     else;
     
@@ -317,7 +235,7 @@
     if (NSMouseInRect(mouseLocation, controlButtonRect, [controlView isFlipped]))
     {
         controlOptions |= NSTrackingAssumeInside;
-        [(TorrentTableView *)controlView setControlButtonHover: [[userInfo objectForKey: @"Row"] intValue]];
+        [(TorrentTableView *)controlView setControlButtonHover: [[userInfo objectForKey: @"Row"] integerValue]];
     }
     
     NSMutableDictionary * controlInfo = [userInfo mutableCopy];
@@ -334,7 +252,7 @@
     if (NSMouseInRect(mouseLocation, revealButtonRect, [controlView isFlipped]))
     {
         revealOptions |= NSTrackingAssumeInside;
-        [(TorrentTableView *)controlView setRevealButtonHover: [[userInfo objectForKey: @"Row"] intValue]];
+        [(TorrentTableView *)controlView setRevealButtonHover: [[userInfo objectForKey: @"Row"] integerValue]];
     }
     
     NSMutableDictionary * revealInfo = [userInfo mutableCopy];
@@ -350,7 +268,7 @@
     if (NSMouseInRect(mouseLocation, actionButtonRect, [controlView isFlipped]))
     {
         actionOptions |= NSTrackingAssumeInside;
-        [(TorrentTableView *)controlView setActionButtonHover: [[userInfo objectForKey: @"Row"] intValue]];
+        [(TorrentTableView *)controlView setActionButtonHover: [[userInfo objectForKey: @"Row"] integerValue]];
     }
     
     NSMutableDictionary * actionInfo = [userInfo mutableCopy];
@@ -388,56 +306,52 @@
     const BOOL minimal = [fDefaults boolForKey: @"SmallView"];
     
     //group coloring
-    NSRect iconRect = [self iconRectForBounds: cellFrame];
+    const NSRect iconRect = [self iconRectForBounds: cellFrame];
     
-    NSInteger groupValue = [torrent groupValue];
+    const NSInteger groupValue = [torrent groupValue];
     if (groupValue != -1)
     {
-        NSRect groupRect = NSInsetRect(iconRect, -1.0f, -2.0f);
+        NSRect groupRect = NSInsetRect(iconRect, -1.0, -2.0);
         if (!minimal)
         {
-            groupRect.size.height--;
-            groupRect.origin.y--;
+            groupRect.size.height -= 1.0;
+            groupRect.origin.y -= 1.0;
         }
-        const CGFloat radius = minimal ? 3.0f : 6.0f;
+        const CGFloat radius = minimal ? 3.0 : 6.0;
         
         NSColor * groupColor = [[GroupsController groups] colorForIndex: groupValue],
-                * darkGroupColor = [groupColor blendedColorWithFraction: 0.2f ofColor: [NSColor whiteColor]];
+                * darkGroupColor = [groupColor blendedColorWithFraction: 0.2 ofColor: [NSColor whiteColor]];
         
         //border
         NSBezierPath * bp = [NSBezierPath bezierPathWithRoundedRect: groupRect xRadius: radius yRadius: radius];
         [darkGroupColor set];
-        [bp setLineWidth: 2.0f];
+        [bp setLineWidth: 2.0];
         [bp stroke];
         
         //inside
         bp = [NSBezierPath bezierPathWithRoundedRect: groupRect xRadius: radius yRadius: radius];
-        NSGradient * gradient = [[NSGradient alloc] initWithStartingColor: [groupColor blendedColorWithFraction: 0.7f
+        NSGradient * gradient = [[NSGradient alloc] initWithStartingColor: [groupColor blendedColorWithFraction: 0.7
                                     ofColor: [NSColor whiteColor]] endingColor: darkGroupColor];
-        [gradient drawInBezierPath: bp angle: 90.0f];
+        [gradient drawInBezierPath: bp angle: 90.0];
         [gradient release];
     }
     
-    //error image
-    const BOOL error = [torrent isError];
-    if (error && !fErrorImage)
-    {
-        fErrorImage = [NSImage imageNamed: @"Error.png"];
-        [fErrorImage setFlipped: YES];
-    }
+    const BOOL error = [torrent isAnyErrorOrWarning];
     
     //icon
     if (!minimal || !(!fTracking && fHoverAction)) //don't show in minimal mode when hovered over
     {
-        NSImage * icon = (minimal && error) ? fErrorImage : [torrent icon];
-        [icon drawInRect: iconRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0f];
+        NSImage * icon = (minimal && error) ? [NSImage imageNamed: [NSApp isOnSnowLeopardOrBetter] ? NSImageNameCaution : @"Error.png"]
+                                            : [torrent icon];
+        [self drawImage: icon inRect: iconRect];
     }
     
+    //error badge
     if (error && !minimal)
     {
-        NSRect errorRect = NSMakeRect(NSMaxX(iconRect) - IMAGE_SIZE_MIN, NSMaxY(iconRect) - IMAGE_SIZE_MIN,
-                                        IMAGE_SIZE_MIN, IMAGE_SIZE_MIN);
-        [fErrorImage drawInRect: errorRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0f];
+        NSRect errorRect = NSMakeRect(NSMaxX(iconRect) - ERROR_IMAGE_SIZE, NSMaxY(iconRect) - ERROR_IMAGE_SIZE,
+                                        ERROR_IMAGE_SIZE, ERROR_IMAGE_SIZE);
+        [self drawImage: [NSImage imageNamed: [NSApp isOnSnowLeopardOrBetter] ? NSImageNameCaution : @"Error.png"] inRect: errorRect];
     }
     
     //text color
@@ -450,37 +364,45 @@
         statusColor = [NSColor darkGrayColor];
     }
     
+    [fTitleAttributes setObject: titleColor forKey: NSForegroundColorAttributeName];
+    [fStatusAttributes setObject: statusColor forKey: NSForegroundColorAttributeName];
+    
     //minimal status
     NSRect minimalStatusRect;
     if (minimal)
     {
-        NSAttributedString * minimalString = [self attributedStatusString: [self minimalStatusString] withColor: statusColor];
+        NSAttributedString * minimalString = [self attributedStatusString: [self minimalStatusString]];
         minimalStatusRect = [self rectForMinimalStatusWithString: minimalString inBounds: cellFrame];
         
         [minimalString drawInRect: minimalStatusRect];
     }
     
     //title
-    NSAttributedString * titleString = [self attributedTitleWithColor: titleColor];
+    NSAttributedString * titleString = [self attributedTitle];
     NSRect titleRect = [self rectForTitleWithString: titleString basedOnMinimalStatusRect: minimalStatusRect inBounds: cellFrame];
     [titleString drawInRect: titleRect];
     
     //priority icon
     if ([torrent priority] != TR_PRI_NORMAL)
     {
-        NSImage * priorityImage = [torrent priority] == TR_PRI_HIGH ? fHighPriorityImage : fLowPriorityImage;
+        NSImage * priorityImage = [torrent priority] == TR_PRI_HIGH ? [NSImage imageNamed: @"PriorityHigh.png"]
+                                                                    : [NSImage imageNamed: @"PriorityLow.png"];
+        //take line out completely when 10.6-only
+        priorityImage = [NSApp isOnSnowLeopardOrBetter] ? [priorityImage retain] : [priorityImage copy];
         
         NSRect priorityRect = NSMakeRect(NSMaxX(titleRect) + PADDING_BETWEEN_TITLE_AND_PRIORITY,
-                                titleRect.origin.y - (PRIORITY_ICON_HEIGHT - titleRect.size.height) / 2.0,
-                                PRIORITY_ICON_WIDTH, PRIORITY_ICON_HEIGHT);
-        [priorityImage drawInRect: priorityRect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+                                        NSMidY(titleRect) - PRIORITY_ICON_HEIGHT  * 0.5,
+                                        PRIORITY_ICON_WIDTH, PRIORITY_ICON_HEIGHT);
+        
+        [self drawImage: priorityImage inRect: priorityRect];
+        [priorityImage release];
     }
     
     //progress
     if (!minimal)
     {
-        NSAttributedString * progressString = [self attributedStatusString: [torrent progressString] withColor: statusColor];
-        NSRect progressRect = [self rectForProgressWithString: progressString inBounds: cellFrame];
+        NSAttributedString * progressString = [self attributedStatusString: [torrent progressString]];
+        NSRect progressRect = [self rectForProgressWithStringInBounds: cellFrame];
         
         [progressString drawInRect: progressRect];
     }
@@ -510,9 +432,7 @@
             controlImage = [NSImage imageNamed: [@"Resume" stringByAppendingString: controlImageSuffix]];
     }
     
-    [controlImage setFlipped: YES];
-    [controlImage drawInRect: [self controlButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
-        fraction: 1.0f];
+    [self drawImage: controlImage inRect: [self controlButtonRectForBounds: cellFrame]];
     
     //reveal button
     NSString * revealImageString;
@@ -524,9 +444,7 @@
         revealImageString = @"RevealOff.png";
     
     NSImage * revealImage = [NSImage imageNamed: revealImageString];
-    [revealImage setFlipped: YES];
-    [revealImage drawInRect: [self revealButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
-        fraction: 1.0f];
+    [self drawImage: revealImage inRect: [self revealButtonRectForBounds: cellFrame]];
     
     //action button
     NSString * actionImageString;
@@ -540,16 +458,14 @@
     if (actionImageString)
     {
         NSImage * actionImage = [NSImage imageNamed: actionImageString];
-        [actionImage setFlipped: YES];
-        [actionImage drawInRect: [self actionButtonRectForBounds: cellFrame] fromRect: NSZeroRect operation: NSCompositeSourceOver
-            fraction: 1.0f];
+        [self drawImage: actionImage inRect: [self actionButtonRectForBounds: cellFrame]];
     }
     
     //status
     if (!minimal)
     {
-        NSAttributedString * statusString = [self attributedStatusString: [self statusString] withColor: statusColor];
-        [statusString drawInRect: [self rectForStatusWithString: statusString inBounds: cellFrame]];
+        NSAttributedString * statusString = [self attributedStatusString: [self statusString]];
+        [statusString drawInRect: [self rectForStatusWithStringInBounds: cellFrame]];
     }
 }
 
@@ -559,13 +475,12 @@
 
 - (void) drawBar: (NSRect) barRect
 {
-    CGFloat piecesBarPercent = [(TorrentTableView *)[self controlView] piecesBarPercent];
-    if (piecesBarPercent > 0.0f)
+    const CGFloat piecesBarPercent = [(TorrentTableView *)[self controlView] piecesBarPercent];
+    if (piecesBarPercent > 0.0)
     {
-        NSRect regularBarRect = barRect, piecesBarRect = barRect;
-        piecesBarRect.size.height *= PIECES_TOTAL_PERCENT * piecesBarPercent;
-        regularBarRect.size.height -= piecesBarRect.size.height;
-        piecesBarRect.origin.y += regularBarRect.size.height;
+        NSRect piecesBarRect, regularBarRect;
+        NSDivideRect(barRect, &piecesBarRect, &regularBarRect, floor(NSHeight(barRect) * PIECES_TOTAL_PERCENT * piecesBarPercent),
+                    NSMaxYEdge);
         
         [self drawRegularBar: regularBarRect];
         [self drawPiecesBar: piecesBarRect];
@@ -578,115 +493,92 @@
     }
     
     [fBarBorderColor set];
-    [NSBezierPath strokeRect: NSInsetRect(barRect, 0.5f, 0.5f)];
+    [NSBezierPath strokeRect: NSInsetRect(barRect, 0.5, 0.5)];
 }
 
 - (void) drawRegularBar: (NSRect) barRect
 {
     Torrent * torrent = [self representedObject];
     
-    NSInteger leftWidth = barRect.size.width;
-    CGFloat progress = [torrent progress];
+    NSRect haveRect, missingRect;
+    NSDivideRect(barRect, &haveRect, &missingRect, round([torrent progress] * NSWidth(barRect)), NSMinXEdge);
     
-    if (progress < 1.0f)
+    if (!NSIsEmptyRect(haveRect))
     {
-        CGFloat rightProgress = 1.0f - progress, progressLeft = [torrent progressLeft];
-        NSInteger rightWidth = leftWidth * rightProgress;
-        leftWidth -= rightWidth;
-        
-        if (progressLeft < rightProgress)
-        {
-            NSInteger rightNoIncludeWidth = rightWidth * ((rightProgress - progressLeft) / rightProgress);
-            rightWidth -= rightNoIncludeWidth;
-            
-            NSRect noIncludeRect = barRect;
-            noIncludeRect.origin.x += barRect.size.width - rightNoIncludeWidth;
-            noIncludeRect.size.width = rightNoIncludeWidth;
-            
-            [[ProgressGradients progressLightGrayGradient] drawInRect: noIncludeRect angle: 90];
-        }
-        
-        if (rightWidth > 0)
-        {
-            if ([torrent isActive] && ![torrent allDownloaded] && ![torrent isChecking]
-                && [fDefaults boolForKey: @"DisplayProgressBarAvailable"])
-            {
-                NSInteger notAvailableWidth = ceil(rightWidth * [torrent notAvailableDesired]);
-                if (notAvailableWidth > 0)
-                {
-                    rightWidth -= notAvailableWidth;
-                    
-                    NSRect notAvailableRect = barRect;
-                    notAvailableRect.origin.x += leftWidth + rightWidth;
-                    notAvailableRect.size.width = notAvailableWidth;
-                    
-                    [[ProgressGradients progressRedGradient] drawInRect: notAvailableRect angle: 90];
-                }
-            }
-            
-            if (rightWidth > 0)
-            {
-                NSRect includeRect = barRect;
-                includeRect.origin.x += leftWidth;
-                includeRect.size.width = rightWidth;
-                
-                [[ProgressGradients progressWhiteGradient] drawInRect: includeRect angle: 90];
-            }
-        }
-    }
-    
-    if (leftWidth > 0)
-    {
-        NSRect completeRect = barRect;
-        completeRect.size.width = leftWidth;
-        
         if ([torrent isActive])
         {
             if ([torrent isChecking])
-                [[ProgressGradients progressYellowGradient] drawInRect: completeRect angle: 90];
+                [[ProgressGradients progressYellowGradient] drawInRect: haveRect angle: 90];
             else if ([torrent isSeeding])
             {
-                NSInteger ratioLeftWidth = leftWidth * (1.0f - [torrent progressStopRatio]);
-                leftWidth -= ratioLeftWidth;
+                NSRect ratioHaveRect, ratioRemainingRect;
+                NSDivideRect(haveRect, &ratioHaveRect, &ratioRemainingRect, round([torrent progressStopRatio] * NSWidth(haveRect)),
+                            NSMinXEdge);
                 
-                if (ratioLeftWidth > 0)
-                {
-                    NSRect ratioLeftRect = barRect;
-                    ratioLeftRect.origin.x += leftWidth;
-                    ratioLeftRect.size.width = ratioLeftWidth;
-                    
-                    [[ProgressGradients progressLightGreenGradient] drawInRect: ratioLeftRect angle: 90];
-                }
-                
-                if (leftWidth > 0)
-                {
-                    completeRect.size.width = leftWidth;
-                    
-                    [[ProgressGradients progressGreenGradient] drawInRect: completeRect angle: 90];
-                }
+                [[ProgressGradients progressGreenGradient] drawInRect: ratioHaveRect angle: 90];
+                [[ProgressGradients progressLightGreenGradient] drawInRect: ratioRemainingRect angle: 90];
             }
             else
-                [[ProgressGradients progressBlueGradient] drawInRect: completeRect angle: 90];
+                [[ProgressGradients progressBlueGradient] drawInRect: haveRect angle: 90];
         }
         else
         {
             if ([torrent waitingToStart])
             {
-                if ([torrent progressLeft] <= 0.0f)
-                    [[ProgressGradients progressDarkGreenGradient] drawInRect: completeRect angle: 90];
+                if ([torrent allDownloaded])
+                    [[ProgressGradients progressDarkGreenGradient] drawInRect: haveRect angle: 90];
                 else
-                    [[ProgressGradients progressDarkBlueGradient] drawInRect: completeRect angle: 90];
+                    [[ProgressGradients progressDarkBlueGradient] drawInRect: haveRect angle: 90];
             }
             else
-                [[ProgressGradients progressGrayGradient] drawInRect: completeRect angle: 90];
+                [[ProgressGradients progressGrayGradient] drawInRect: haveRect angle: 90];
         }
     }
+    
+    if (![torrent allDownloaded])
+    {
+        const CGFloat widthRemaining = round(NSWidth(barRect) * [torrent progressLeft]);
+        
+        NSRect wantedRect;
+        NSDivideRect(missingRect, &wantedRect, &missingRect, widthRemaining, NSMinXEdge);
+        
+        //not-available section
+        if ([torrent isActive] && ![torrent isChecking] && [torrent availableDesired] < 1.0
+            && [fDefaults boolForKey: @"DisplayProgressBarAvailable"])
+        {
+            NSRect unavailableRect;
+            NSDivideRect(wantedRect, &wantedRect, &unavailableRect, round(NSWidth(wantedRect) * [torrent availableDesired]),
+                        NSMinXEdge);
+            
+            [[ProgressGradients progressRedGradient] drawInRect: unavailableRect angle: 90];
+        }
+        
+        //remaining section
+        [[ProgressGradients progressWhiteGradient] drawInRect: wantedRect angle: 90];
+    }
+    
+    //unwanted section
+    if (!NSIsEmptyRect(missingRect))
+    {
+        if (![torrent isMagnet])
+            [[ProgressGradients progressLightGrayGradient] drawInRect: missingRect angle: 90];
+        else
+            [[ProgressGradients progressRedGradient] drawInRect: missingRect angle: 90];
+    }
 }
 
 - (void) drawPiecesBar: (NSRect) barRect
 {
     Torrent * torrent = [self representedObject];
     
+    //fill an all-white bar for magnet links 
+    if ([torrent isMagnet]) 
+    {
+        [[NSColor whiteColor] set]; 
+        NSRectFill(barRect); 
+        return;
+    }
+    
     NSInteger pieceCount = MIN([torrent pieceCount], MAX_PIECES);
     float * piecesPercent = malloc(pieceCount * sizeof(float));
     [torrent getAmountFinished: piecesPercent size: pieceCount];
@@ -727,78 +619,137 @@
 
 - (NSRect) rectForMinimalStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
 {
-    NSRect result = bounds;
+    NSRect result;
     result.size = [string size];
     
-    result.origin.x += bounds.size.width - result.size.width - PADDING_HORIZONTAL;
-    result.origin.y += PADDING_ABOVE_MIN_STATUS;
+    result.origin.x = NSMaxX(bounds) - (NSWidth(result) + PADDING_HORIZONTAL);
+    result.origin.y = NSMinY(bounds) + PADDING_ABOVE_MIN_STATUS;
     
     return result;
 }
 
-- (NSRect) rectForTitleWithString: (NSAttributedString *) string basedOnMinimalStatusRect: (NSRect) statusRect
-            inBounds: (NSRect) bounds
+- (NSRect) rectForTitleWithString: (NSAttributedString *) string basedOnMinimalStatusRect: (NSRect) statusRect inBounds: (NSRect) bounds
 {
     const BOOL minimal = [fDefaults boolForKey: @"SmallView"];
     
-    NSRect result = bounds;
-    result.origin.y += PADDING_ABOVE_TITLE;
-    result.origin.x += PADDING_HORIZONTAL + (minimal ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG) + PADDING_BETWEEN_IMAGE_AND_TITLE;
+    NSRect result;
+    result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE;
+    result.origin.x = NSMinX(bounds) + PADDING_HORIZONTAL
+                        + (minimal ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG) + PADDING_BETWEEN_IMAGE_AND_TITLE;
     
-    result.size = [string size];
-    result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL
-                - (minimal ? PADDING_BETWEEN_TITLE_AND_MIN_STATUS + statusRect.size.width : 0.0)
-                - ([[self representedObject] priority] != TR_PRI_NORMAL ? PRIORITY_ICON_WIDTH + PADDING_BETWEEN_TITLE_AND_PRIORITY: 0.0));
+    result.size.height = HEIGHT_TITLE;
+    result.size.width = NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONTAL;
+    if (minimal)
+        result.size.width -= PADDING_BETWEEN_TITLE_AND_MIN_STATUS + NSWidth(statusRect);
+    if ([[self representedObject] priority] != TR_PRI_NORMAL)
+    {
+        result.size.width -= PRIORITY_ICON_WIDTH + PADDING_BETWEEN_TITLE_AND_PRIORITY;
+        result.size.width = MIN(NSWidth(result), [string size].width); //only need to force it smaller for the priority icon
+    }
     
     return result;
 }
 
-- (NSRect) rectForProgressWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
+- (NSRect) rectForProgressWithStringInBounds: (NSRect) bounds
 {
-    NSRect result = bounds;
-    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS;
-    result.origin.x += PADDING_HORIZONTAL + IMAGE_SIZE_REG + PADDING_BETWEEN_IMAGE_AND_TITLE;
+    NSRect result;
+    result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS;
+    result.origin.x = NSMinX(bounds) + PADDING_HORIZONTAL + IMAGE_SIZE_REG + PADDING_BETWEEN_IMAGE_AND_TITLE;
     
-    result.size = [string size];
-    result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL);
+    result.size.height = HEIGHT_STATUS;
+    result.size.width = NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONTAL;
     
     return result;
 }
 
-- (NSRect) rectForStatusWithString: (NSAttributedString *) string inBounds: (NSRect) bounds
+- (NSRect) rectForStatusWithStringInBounds: (NSRect) bounds
 {
-    NSRect result = bounds;
-    result.origin.y += PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS
+    NSRect result;
+    result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE + HEIGHT_TITLE + PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS
                         + PADDING_BETWEEN_PROGRESS_AND_BAR + BAR_HEIGHT + PADDING_BETWEEN_BAR_AND_STATUS;
-    result.origin.x += PADDING_HORIZONTAL + IMAGE_SIZE_REG + PADDING_BETWEEN_IMAGE_AND_TITLE;
+    result.origin.x = NSMinX(bounds) + PADDING_HORIZONTAL + IMAGE_SIZE_REG + PADDING_BETWEEN_IMAGE_AND_TITLE;
     
-    result.size = [string size];
-    result.size.width = MIN(result.size.width, NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL);
+    result.size.height = HEIGHT_STATUS;
+    result.size.width = NSMaxX(bounds) - NSMinX(result) - PADDING_HORIZONTAL;
     
     return result;
 }
 
-- (NSAttributedString *) attributedTitleWithColor: (NSColor *) color
+- (NSRect) barRectForBounds: (NSRect) bounds
 {
-    if (color)
-        [fTitleAttributes setObject: color forKey: NSForegroundColorAttributeName];
+    const BOOL minimal = [fDefaults boolForKey: @"SmallView"];
     
+    NSRect result;
+    result.size.height = BAR_HEIGHT;
+    result.origin.x = NSMinX(bounds) + (minimal ? IMAGE_SIZE_MIN : IMAGE_SIZE_REG) + PADDING_BETWEEN_IMAGE_AND_BAR;
+    
+    result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE + HEIGHT_TITLE;
+    if (minimal)
+        result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
+    else
+        result.origin.y += PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS + PADDING_BETWEEN_PROGRESS_AND_BAR;
+    
+    result.size.width = floor(NSMaxX(bounds) - result.origin.x - PADDING_HORIZONTAL - 2.0 * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH));
+    
+    return result;
+}
+
+- (NSRect) controlButtonRectForBounds: (NSRect) bounds
+{
+    NSRect result;
+    result.size.height = NORMAL_BUTTON_WIDTH;
+    result.size.width = NORMAL_BUTTON_WIDTH;
+    result.origin.x = NSMaxX(bounds) - 2.0 * (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH);
+    
+    result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5;
+    if ([fDefaults boolForKey: @"SmallView"])
+        result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
+    else
+        result.origin.y += PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS + PADDING_BETWEEN_PROGRESS_AND_BAR;
+    
+    return result;
+}
+
+- (NSRect) revealButtonRectForBounds: (NSRect) bounds
+{
+    NSRect result;
+    result.size.height = NORMAL_BUTTON_WIDTH;
+    result.size.width = NORMAL_BUTTON_WIDTH;
+    result.origin.x = NSMaxX(bounds) - (PADDING_HORIZONTAL + NORMAL_BUTTON_WIDTH);
+    
+    result.origin.y = NSMinY(bounds) + PADDING_ABOVE_TITLE + HEIGHT_TITLE - (NORMAL_BUTTON_WIDTH - BAR_HEIGHT) * 0.5;
+    if ([fDefaults boolForKey: @"SmallView"])
+        result.origin.y += PADDING_BETWEEN_TITLE_AND_BAR_MIN;
+    else
+        result.origin.y += PADDING_BETWEEN_TITLE_AND_PROGRESS + HEIGHT_STATUS + PADDING_BETWEEN_PROGRESS_AND_BAR;
+    
+    return result;
+}
+
+- (NSRect) actionButtonRectForBounds: (NSRect) bounds
+{
+    const NSRect iconRect = [self iconRectForBounds: bounds];
+    
+    //in minimal view the rect will be the icon rect, but avoid the extra defaults lookup with some cheap math
+    return NSMakeRect(NSMidX(iconRect) - ACTION_BUTTON_WIDTH * 0.5, NSMidY(iconRect) - ACTION_BUTTON_WIDTH * 0.5,
+                        ACTION_BUTTON_WIDTH, ACTION_BUTTON_WIDTH);
+}
+
+- (NSAttributedString *) attributedTitle
+{
     NSString * title = [[self representedObject] name];
     return [[[NSAttributedString alloc] initWithString: title attributes: fTitleAttributes] autorelease];
 }
 
-- (NSAttributedString *) attributedStatusString: (NSString *) string withColor: (NSColor *) color
+- (NSAttributedString *) attributedStatusString: (NSString *) string
 {
-    if (color)
-        [fStatusAttributes setObject: color forKey: NSForegroundColorAttributeName];
-    
     return [[[NSAttributedString alloc] initWithString: string attributes: fStatusAttributes] autorelease];
 }
 
 - (NSString *) buttonString
 {
     if (fMouseDownRevealButton || (!fTracking && fHoverReveal))
-        return NSLocalizedString(@"Reveal the data file in Finder", "Torrent cell -> button info");
+        return NSLocalizedString(@"Show the data file in Finder", "Torrent cell -> button info");
     else if (fMouseDownControlButton || (!fTracking && fHoverControl))
     {
         Torrent * torrent = [self representedObject];
@@ -841,4 +792,15 @@
     }
 }
 
+- (void) drawImage: (NSImage *) image inRect: (NSRect) rect
+{
+    if ([NSApp isOnSnowLeopardOrBetter])
+        [image drawInRect: rect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0 respectFlipped: YES hints: nil];
+    else
+    {
+        [image setFlipped: YES];
+        [image drawInRect: rect fromRect: NSZeroRect operation: NSCompositeSourceOver fraction: 1.0];
+    }
+}
+
 @end
