@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentTableView.h 7007 2008-11-01 15:22:32Z livings124 $
+ * $Id: TorrentTableView.h 9844 2010-01-01 21:12:04Z livings124 $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,8 +24,9 @@
 
 #import <Cocoa/Cocoa.h>
 #import <transmission.h>
-#import "Controller.h"
 
+@class Controller;
+@class Torrent;
 @class TorrentCell;
 
 #define GROUP_SEPARATOR_HEIGHT 18.0
@@ -45,7 +46,8 @@
     NSInteger fMouseControlRow, fMouseRevealRow, fMouseActionRow, fActionPushedRow;
     NSArray * fSelectedValues;
     
-    IBOutlet NSMenu * fActionMenu, * fUploadMenu, * fDownloadMenu, * fRatioMenu;
+    IBOutlet NSMenu * fActionMenu, * fUploadMenu, * fDownloadMenu, * fRatioMenu, * fPriorityMenu;
+    IBOutlet NSMenuItem * fGlobalLimitItem;
     Torrent * fMenuTorrent;
     
     CGFloat fPiecesBarPercent;
@@ -76,12 +78,14 @@
 
 - (void) setQuickLimitMode: (id) sender;
 - (void) setQuickLimit: (id) sender;
+- (void) setGlobalLimit: (id) sender;
 
 - (void) setQuickRatioMode: (id) sender;
 - (void) setQuickRatio: (id) sender;
 
+- (void) setPriority: (id) sender;
+
 - (void) checkFile: (id) sender;
-- (void) moveDataFile: (id) sender;
 
 - (void) togglePiecesBar;
 - (CGFloat) piecesBarPercent;
