@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: TorrentTableView.m 7080 2008-11-09 12:12:29Z livings124 $
+ * $Id: TorrentTableView.m 10206 2010-02-15 14:56:14Z livings124 $
  *
- * Copyright (c) 2005-2008 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,12 +23,12 @@
  *****************************************************************************/
 
 #import "TorrentTableView.h"
-#import "TorrentCell.h"
-#import "Torrent.h"
-#import "TorrentGroup.h"
+#import "Controller.h"
 #import "FileListNode.h"
-#import "QuickLookController.h"
 #import "NSApplicationAdditions.h"
+#import "Torrent.h"
+#import "TorrentCell.h"
+#import "TorrentGroup.h"
 
 #define MAX_GROUP 999999
 
@@ -36,23 +36,18 @@
 #define ACTION_MENU_UNLIMITED_TAG 102
 #define ACTION_MENU_LIMIT_TAG 103
 
-#define GROUP_SPEED_IMAGE_COLUMN_WIDTH 8.0f
-#define GROUP_RATIO_IMAGE_COLUMN_WIDTH 10.0f
+#define ACTION_MENU_PRIORITY_HIGH_TAG 101
+#define ACTION_MENU_PRIORITY_NORMAL_TAG 102
+#define ACTION_MENU_PRIORITY_LOW_TAG 103
 
 #define TOGGLE_PROGRESS_SECONDS 0.175
 
 @interface TorrentTableView (Private)
 
-- (BOOL) pointInControlRect: (NSPoint) point;
-- (BOOL) pointInRevealRect: (NSPoint) point;
-- (BOOL) pointInActionRect: (NSPoint) point;
-
 - (BOOL) pointInGroupStatusRect: (NSPoint) point;
 
 - (void) setGroupStatusColumns;
 
-- (void) createFileMenu: (NSMenu *) menu forFiles: (NSArray *) files;
-
 @end
 
 @implementation TorrentTableView
@@ -65,24 +60,6 @@
         
         fTorrentCell = [[TorrentCell alloc] init];
         
-        if (![NSApp isOnLeopardOrBetter])
-        {
-            NSTableColumn * groupColumn = [self tableColumnWithIdentifier: @"Group"];
-            [self setOutlineTableColumn: groupColumn];
-            
-            //remove all unnecessary columns
-            for (NSInteger i = [[self tableColumns] count]-1; i >= 0; i--)
-            {
-                NSTableColumn * column = [[self tableColumns] objectAtIndex: i];
-                if (column != groupColumn)
-                    [self removeTableColumn: column];
-            }
-            
-            [self sizeLastColumnToFit];
-            
-            [groupColumn setDataCell: fTorrentCell];
-        }
-        
         NSData * groupData = [fDefaults dataForKey: @"CollapsedGroups"];
         if (groupData)
             fCollapsedGroups = [[NSUnarchiver unarchiveObjectWithData: groupData] mutableCopy];
@@ -96,7 +73,7 @@
         
         [self setDelegate: self];
         
-        fPiecesBarPercent = [fDefaults boolForKey: @"PiecesBar"] ? 1.0f : 0.0f;
+        fPiecesBarPercent = [fDefaults boolForKey: @"PiecesBar"] ? 1.0 : 0.0;
     }
     
     return self;
@@ -160,7 +137,7 @@
 
 - (NSCell *) outlineView: (NSOutlineView *) outlineView dataCellForTableColumn: (NSTableColumn *) tableColumn item: (id) item
 {
-    BOOL group = ![item isKindOfClass: [Torrent class]];
+    const BOOL group = ![item isKindOfClass: [Torrent class]];
     if (!tableColumn)
         return !group ? fTorrentCell : nil;
     else
@@ -174,13 +151,10 @@
     {
         [cell setRepresentedObject: item];
         
-        NSInteger row = [self rowForItem: item];
-        if ([NSApp isOnLeopardOrBetter])
-        {
-            [cell setControlHover: row == fMouseControlRow];
-            [cell setRevealHover: row == fMouseRevealRow];
-            [cell setActionHover: row == fMouseActionRow];
-        }
+        const NSInteger row = [self rowForItem: item];
+        [cell setControlHover: row == fMouseControlRow];
+        [cell setRevealHover: row == fMouseRevealRow];
+        [cell setActionHover: row == fMouseActionRow];
         [cell setActionPushed: row == fActionPushedRow];
     }
     else
@@ -189,13 +163,18 @@
         if ([ident isEqualToString: @"UL Image"] || [ident isEqualToString: @"DL Image"])
         {
             //ensure arrows are white only when selected
-            NSImage * image = [cell image];
-            BOOL template = [cell backgroundStyle] == NSBackgroundStyleLowered;
-            if ([image isTemplate] != template)
+            if ([NSApp isOnSnowLeopardOrBetter])
+                [[cell image] setTemplate: [cell backgroundStyle] == NSBackgroundStyleLowered];
+            else
             {
-                [image setTemplate: template];
-                [cell setImage: nil];
-                [cell setImage: image];
+                NSImage * image = [cell image];
+                const BOOL template = [cell backgroundStyle] == NSBackgroundStyleLowered;
+                if ([image isTemplate] != template)
+                {
+                    [image setTemplate: template];
+                    [cell setImage: nil];
+                    [cell setImage: image];
+                }
             }
         }
     }
@@ -203,7 +182,7 @@
 
 - (NSRect) frameOfCellAtColumn: (NSInteger) column row: (NSInteger) row
 {
-    if (column == -1 || ![NSApp isOnLeopardOrBetter])
+    if (column == -1)
         return [self rectOfRow: row];
     else
     {
@@ -259,7 +238,7 @@
         if (![[self itemAtRow: row] isKindOfClass: [Torrent class]])
             continue;
         
-        NSDictionary * userInfo = [NSDictionary dictionaryWithObject: [NSNumber numberWithInt: row] forKey: @"Row"];
+        NSDictionary * userInfo = [NSDictionary dictionaryWithObject: [NSNumber numberWithInteger: row] forKey: @"Row"];
         TorrentCell * cell = (TorrentCell *)[self preparedCellAtColumn: -1 row: row];
         [cell addTrackingAreasForView: self inRect: [self rectOfRow: row] withUserInfo: userInfo mouseLocation: mouseLocation];
     }
@@ -271,12 +250,7 @@
     fMouseRevealRow = -1;
     fMouseActionRow = -1;
     
-    if (![NSApp isOnLeopardOrBetter])
-        return;
-    
-    NSEnumerator * enumerator = [[self trackingAreas] objectEnumerator];
-    NSTrackingArea * area;
-    while ((area = [enumerator nextObject]))
+    for (NSTrackingArea * area in [self trackingAreas])
     {
         if ([area owner] == self && [[area userInfo] objectForKey: @"Row"])
             [self removeTrackingArea: area];
@@ -304,7 +278,6 @@
         [self setNeedsDisplayInRect: [self rectOfRow: row]];
 }
 
-//when Leopard-only, use these variables instead of pointInActionRect:, etc.
 - (void) mouseEntered: (NSEvent *) event
 {
     NSDictionary * dict = (NSDictionary *)[event userData];
@@ -312,7 +285,7 @@
     NSNumber * row;
     if ((row = [dict objectForKey: @"Row"]))
     {
-        NSInteger rowVal = [row intValue];
+        NSInteger rowVal = [row integerValue];
         NSString * type = [dict objectForKey: @"Type"];
         if ([type isEqualToString: @"Action"])
             fMouseActionRow = rowVal;
@@ -340,7 +313,7 @@
         else
             fMouseRevealRow = -1;
         
-        [self setNeedsDisplayInRect: [self rectOfRow: [row intValue]]];
+        [self setNeedsDisplayInRect: [self rectOfRow: [row integerValue]]];
     }
 }
 
@@ -376,6 +349,7 @@
 - (void) mouseDown: (NSEvent *) event
 {
     NSPoint point = [self convertPoint: [event locationInWindow] fromView: nil];
+    const NSInteger row = [self rowAtPoint: point];
     
     //check to toggle group status before anything else
     if ([self pointInGroupStatusRect: point])
@@ -386,7 +360,7 @@
         return;
     }
     
-    BOOL pushed = [self pointInControlRect: point] || [self pointInRevealRect: point] || [self pointInActionRect: point];
+    const BOOL pushed = row != -1 && (fMouseActionRow == row || fMouseRevealRow == row || fMouseControlRow == row);
     
     //if pushing a button, don't change the selected rows
     if (pushed)
@@ -398,10 +372,8 @@
     fSelectedValues = nil;
     
     //avoid weird behavior when showing menu by doing this after mouse down
-    if ([self pointInActionRect: point])
+    if (row != -1 && fMouseActionRow == row)
     {
-        NSInteger row = [self rowAtPoint: point];
-        
         fActionPushedRow = row;
         [self setNeedsDisplayInRect: [self rectOfRow: row]]; //ensure button is pushed down
         
@@ -413,7 +385,6 @@
     else if (!pushed && [event clickCount] == 2) //double click
     {
         id item = nil;
-        NSInteger row = [self rowAtPoint: point];
         if (row != -1)
             item = [self itemAtRow: row];
         
@@ -434,26 +405,21 @@
 {
     NSMutableIndexSet * indexSet = [NSMutableIndexSet indexSet];
     
-    NSEnumerator * enumerator = [values objectEnumerator];
-    id item;
-    while ((item = [enumerator nextObject]))
+    for (id item in values)
     {
         if ([item isKindOfClass: [Torrent class]])
         {
-            NSInteger index = [self rowForItem: item];
+            const NSInteger index = [self rowForItem: item];
             if (index != -1)
                 [indexSet addIndex: index];
         }
         else
         {
-            NSInteger group = [item groupIndex];
+            const NSInteger group = [item groupIndex];
             for (NSInteger i = 0; i < [self numberOfRows]; i++)
             {
-                if ([indexSet containsIndex: i])
-                    continue;
-                
                 id tableItem = [self itemAtRow: i];
-                if (![tableItem isKindOfClass: [Torrent class]] && group == [tableItem groupIndex])
+                if ([tableItem isKindOfClass: [TorrentGroup class]] && group == [tableItem groupIndex])
                 {
                     [indexSet addIndex: i];
                     break;
@@ -490,7 +456,8 @@
         {
             NSArray * groupTorrents = [item torrents];
             [torrents addObjectsFromArray: groupTorrents];
-            i += [groupTorrents count];
+            if ([self isItemExpanded: item])
+                i +=[groupTorrents count];
         }
     }
     
@@ -527,19 +494,10 @@
     
     if (firstChar == 'f' && [event modifierFlags] & NSAlternateKeyMask && [event modifierFlags] & NSCommandKeyMask)
         [fController focusFilterField];
+    else if (firstChar == ' ')
+        [fController toggleQuickLook: nil];
     else
-    {
-        //handle quicklook
-        if (firstChar == ' ')
-            [[QuickLookController quickLook] toggleQuickLook];
-        else if (firstChar == NSRightArrowFunctionKey)
-            [[QuickLookController quickLook] pressRight];
-        else if (firstChar == NSLeftArrowFunctionKey)
-            [[QuickLookController quickLook] pressLeft];
-        else;
-        
         [super keyDown: event];
-    }
 }
 
 - (NSRect) iconRectForRow: (NSInteger) row
@@ -547,11 +505,12 @@
     return [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
 }
 
+#warning catch string urls?
 - (void) paste: (id) sender
 {
     NSURL * url;
     if ((url = [NSURL URLFromPasteboard: [NSPasteboard generalPasteboard]]))
-        [fController openURL: url];
+        [fController openURL: [url absoluteString]];
 }
 
 - (BOOL) validateMenuItem: (NSMenuItem *) menuItem
@@ -570,7 +529,7 @@
         [fController stopTorrents: [NSArray arrayWithObject: torrent]];
     else
     {
-        if ([[NSApp currentEvent] modifierFlags] & NSAlternateKeyMask)
+        if (([NSApp isOnSnowLeopardOrBetter] ? [NSEvent modifierFlags] : [[NSApp currentEvent] modifierFlags]) & NSAlternateKeyMask)
             [fController resumeTorrentsNoWait: [NSArray arrayWithObject: torrent]];
         else if ([torrent waitingToStart])
             [fController stopTorrents: [NSArray arrayWithObject: torrent]];
@@ -581,30 +540,35 @@
 
 - (void) displayTorrentMenuForEvent: (NSEvent *) event
 {
-    NSInteger row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
+    const NSInteger row = [self rowAtPoint: [self convertPoint: [event locationInWindow] fromView: nil]];
     if (row < 0)
         return;
     
-    NSInteger numberOfNonFileItems = [fActionMenu numberOfItems];
-    
     //update file action menu
     fMenuTorrent = [[self itemAtRow: row] retain];
-    [self createFileMenu: fActionMenu forFiles: [fMenuTorrent fileList]];
+    
+    //update global limit check
+    [fGlobalLimitItem setState: [fMenuTorrent usesGlobalSpeedLimit] ? NSOnState : NSOffState];
     
     //place menu below button
     NSRect rect = [fTorrentCell iconRectForBounds: [self rectOfRow: row]];
     NSPoint location = rect.origin;
-    location.y += rect.size.height + 5.0f;
-    location = [self convertPoint: location toView: nil];
+    location.y += rect.size.height + 5.0;
     
-    NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
-        modifierFlags: [event modifierFlags] timestamp: [event timestamp] windowNumber: [event windowNumber]
-        context: [event context] eventNumber: [event eventNumber] clickCount: [event clickCount] pressure: [event pressure]];
-    
-    [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
-    
-    for (NSInteger i = [fActionMenu numberOfItems]-1; i >= numberOfNonFileItems; i--)
-        [fActionMenu removeItemAtIndex: i];
+    if ([NSApp isOnSnowLeopardOrBetter])
+    {
+        location = [self convertPoint: location toView: self];
+        [fActionMenu popUpMenuPositioningItem: nil atLocation: location inView: self];
+    }
+    else
+    {
+        location = [self convertPoint: location toView: nil];
+        NSEvent * newEvent = [NSEvent mouseEventWithType: [event type] location: location
+            modifierFlags: [event modifierFlags] timestamp: [event timestamp] windowNumber: [event windowNumber]
+            context: [event context] eventNumber: [event eventNumber] clickCount: [event clickCount] pressure: [event pressure]];
+        
+        [NSMenu popUpContextMenu: fActionMenu withEvent: newEvent forView: self];
+    }
     
     [fMenuTorrent release];
     fMenuTorrent = nil;
@@ -619,9 +583,10 @@
     if (menu == fUploadMenu || menu == fDownloadMenu)
     {
         NSMenuItem * item;
-        if ([menu numberOfItems] == 4)
+        if ([menu numberOfItems] == 3)
         {
-            const NSInteger speedLimitActionValue[] = { 0, 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500, 750, -1 };
+            const NSInteger speedLimitActionValue[] = { 0, 5, 10, 20, 30, 40, 50, 75, 100, 150, 200, 250, 500,
+                                                        750, 1000, 1500, 2000, -1 };
             
             for (NSInteger i = 0; speedLimitActionValue[i] != -1; i++)
             {
@@ -635,28 +600,25 @@
             }
         }
         
-        BOOL upload = menu == fUploadMenu;
-        NSInteger mode = [fMenuTorrent speedMode: upload];
+        const BOOL upload = menu == fUploadMenu;
+        const BOOL limit = [fMenuTorrent usesSpeedLimit: upload];
         
         item = [menu itemWithTag: ACTION_MENU_LIMIT_TAG];
-        [item setState: mode == TR_SPEEDLIMIT_SINGLE ? NSOnState : NSOffState];
+        [item setState: limit ? NSOnState : NSOffState];
         [item setTitle: [NSString stringWithFormat: NSLocalizedString(@"Limit (%d KB/s)",
                             "torrent action menu -> upload/download limit"), [fMenuTorrent speedLimit: upload]]];
         
         item = [menu itemWithTag: ACTION_MENU_UNLIMITED_TAG];
-        [item setState: mode == TR_SPEEDLIMIT_UNLIMITED ? NSOnState : NSOffState];
-        
-        item = [menu itemWithTag: ACTION_MENU_GLOBAL_TAG];
-        [item setState: mode == TR_SPEEDLIMIT_GLOBAL ? NSOnState : NSOffState];
+        [item setState: !limit ? NSOnState : NSOffState];
     }
     else if (menu == fRatioMenu)
     {
         NSMenuItem * item;
         if ([menu numberOfItems] == 4)
         {
-            const CGFloat ratioLimitActionValue[] = { 0.25f, 0.5f, 0.75f, 1.0f, 1.5f, 2.0f, 3.0f, -1.0f };
+            const CGFloat ratioLimitActionValue[] = { 0.25, 0.5, 0.75, 1.0, 1.5, 2.0, 3.0, -1.0 };
             
-            for (NSInteger i = 0; ratioLimitActionValue[i] != -1.0f; i++)
+            for (NSInteger i = 0; ratioLimitActionValue[i] != -1.0; i++)
             {
                 item = [[NSMenuItem alloc] initWithTitle: [NSString localizedStringWithFormat: @"%.2f", ratioLimitActionValue[i]]
                         action: @selector(setQuickRatio:) keyEquivalent: @""];
@@ -667,40 +629,46 @@
             }
         }
         
-        NSInteger mode = [fMenuTorrent ratioSetting];
+        const tr_ratiolimit mode = [fMenuTorrent ratioSetting];
         
         item = [menu itemWithTag: ACTION_MENU_LIMIT_TAG];
-        [item setState: mode == NSOnState ? NSOnState : NSOffState];
+        [item setState: mode == TR_RATIOLIMIT_SINGLE ? NSOnState : NSOffState];
         [item setTitle: [NSString localizedStringWithFormat: NSLocalizedString(@"Stop at Ratio (%.2f)",
             "torrent action menu -> ratio stop"), [fMenuTorrent ratioLimit]]];
         
         item = [menu itemWithTag: ACTION_MENU_UNLIMITED_TAG];
-        [item setState: mode == NSOffState ? NSOnState : NSOffState];
+        [item setState: mode == TR_RATIOLIMIT_UNLIMITED ? NSOnState : NSOffState];
         
         item = [menu itemWithTag: ACTION_MENU_GLOBAL_TAG];
-        [item setState: mode == NSMixedState ? NSOnState : NSOffState];
+        [item setState: mode == TR_RATIOLIMIT_GLOBAL ? NSOnState : NSOffState];
     }
-    else //assume the menu is part of the file list
+    else if (menu == fPriorityMenu)
     {
-        if ([menu numberOfItems] > 0)
-            return;
+        const tr_priority_t priority = [fMenuTorrent priority];
         
-        NSMenu * supermenu = [menu supermenu];
-        [self createFileMenu: menu forFiles: [(FileListNode *)[[supermenu itemAtIndex: [supermenu indexOfItemWithSubmenu: menu]]
-                                                representedObject] children]];
+        NSMenuItem * item = [menu itemWithTag: ACTION_MENU_PRIORITY_HIGH_TAG];
+        [item setState: priority == TR_PRI_HIGH ? NSOnState : NSOffState];
+        
+        item = [menu itemWithTag: ACTION_MENU_PRIORITY_NORMAL_TAG];
+        [item setState: priority == TR_PRI_NORMAL ? NSOnState : NSOffState];
+        
+        item = [menu itemWithTag: ACTION_MENU_PRIORITY_LOW_TAG];
+        [item setState: priority == TR_PRI_LOW ? NSOnState : NSOffState];
     }
 }
 
 //alternating rows - first row after group row is white
 - (void) highlightSelectionInClipRect: (NSRect) clipRect
 {
-    NSColor * altColor = [[NSColor controlAlternatingRowBackgroundColors] objectAtIndex: 1];
-    [altColor set];
-    
     NSRect visibleRect = clipRect;
     NSRange rows = [self rowsInRect: visibleRect];
     BOOL start = YES;
     
+    const CGFloat totalRowHeight = [self rowHeight] + [self intercellSpacing].height;
+    
+    NSRect gridRects[(NSInteger)(ceil(visibleRect.size.height / totalRowHeight / 2.0)) + 1]; //add one if partial rows at top and bottom
+    NSInteger rectNum = 0;
+    
     if (rows.length > 0)
     {
         //determine what the first row color should be
@@ -729,78 +697,74 @@
             }
             
             if (!start && ![self isRowSelected: i])
-                NSRectFill([self rectOfRow: i]);
+                gridRects[rectNum++] = [self rectOfRow: i];
             
             start = !start;
         }
         
-        CGFloat newY = NSMaxY([self rectOfRow: i-1]);
+        const CGFloat newY = NSMaxY([self rectOfRow: i-1]);
         visibleRect.size.height -= newY - visibleRect.origin.y;
         visibleRect.origin.y = newY;
     }
-        
+    
+    const NSInteger numberBlankRows = ceil(visibleRect.size.height / totalRowHeight);
+    
     //remaining visible rows continue alternating
-    const CGFloat height = [self rowHeight] + [self intercellSpacing].height;
-    const NSInteger numberOfRects = ceil(visibleRect.size.height / height);
-    
-    visibleRect.size.height = height;
+    visibleRect.size.height = totalRowHeight;
     if (start)
-        visibleRect.origin.y += height;
+        visibleRect.origin.y += totalRowHeight;
     
-    for (NSInteger i = start ? 1 : 0; i < numberOfRects; i += 2)
+    for (NSInteger i = start ? 1 : 0; i < numberBlankRows; i += 2)
     {
-        NSRectFill(visibleRect);
-        visibleRect.origin.y += 2.0 * height;
+        gridRects[rectNum++] = visibleRect;
+        visibleRect.origin.y += 2.0 * totalRowHeight;
     }
     
+    NSAssert([[NSColor controlAlternatingRowBackgroundColors] count] >= 2, @"There should be 2 alternating row colors");
+    
+    [[[NSColor controlAlternatingRowBackgroundColors] objectAtIndex: 1] set];
+    NSRectFillList(gridRects, rectNum);
+    
     [super highlightSelectionInClipRect: clipRect];
 }
 
 - (void) setQuickLimitMode: (id) sender
 {
-    NSInteger mode;
-    switch ([sender tag])
-    {
-        case ACTION_MENU_UNLIMITED_TAG:
-            mode = TR_SPEEDLIMIT_UNLIMITED;
-            break;
-        case ACTION_MENU_LIMIT_TAG:
-            mode = TR_SPEEDLIMIT_SINGLE;
-            break;
-        case ACTION_MENU_GLOBAL_TAG:
-            mode = TR_SPEEDLIMIT_GLOBAL;
-            break;
-        default:
-            return;
-    }
-    
-    [fMenuTorrent setSpeedMode: mode upload: [sender menu] == fUploadMenu];
+    const BOOL limit = [sender tag] == ACTION_MENU_LIMIT_TAG;
+    [fMenuTorrent setUseSpeedLimit: limit upload: [sender menu] == fUploadMenu];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
 }
 
 - (void) setQuickLimit: (id) sender
 {
-    BOOL upload = [sender menu] == fUploadMenu;
-    [fMenuTorrent setSpeedMode: TR_SPEEDLIMIT_SINGLE upload: upload];
+    const BOOL upload = [sender menu] == fUploadMenu;
+    [fMenuTorrent setUseSpeedLimit: YES upload: upload];
     [fMenuTorrent setSpeedLimit: [[sender representedObject] intValue] upload: upload];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
 }
 
+- (void) setGlobalLimit: (id) sender
+{
+    [fMenuTorrent setUseGlobalSpeedLimit: [sender state] != NSOnState];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
+}
+
 - (void) setQuickRatioMode: (id) sender
 {
-    NSInteger mode;
+    tr_ratiolimit mode;
     switch ([sender tag])
     {
         case ACTION_MENU_UNLIMITED_TAG:
-            mode = NSOffState;
+            mode = TR_RATIOLIMIT_UNLIMITED;
             break;
         case ACTION_MENU_LIMIT_TAG:
-            mode = NSOnState;
+            mode = TR_RATIOLIMIT_SINGLE;
             break;
         case ACTION_MENU_GLOBAL_TAG:
-            mode = NSMixedState;
+            mode = TR_RATIOLIMIT_GLOBAL;
             break;
         default:
             return;
@@ -813,12 +777,35 @@
 
 - (void) setQuickRatio: (id) sender
 {
-    [fMenuTorrent setRatioSetting: NSOnState];
+    [fMenuTorrent setRatioSetting: TR_RATIOLIMIT_SINGLE];
     [fMenuTorrent setRatioLimit: [[sender representedObject] floatValue]];
     
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateOptions" object: nil];
 }
 
+- (void) setPriority: (id) sender
+{
+    tr_priority_t priority;
+    switch ([sender tag])
+    {
+        case ACTION_MENU_PRIORITY_HIGH_TAG:
+            priority = TR_PRI_HIGH;
+            break;
+        case ACTION_MENU_PRIORITY_NORMAL_TAG:
+            priority = TR_PRI_NORMAL;
+            break;
+        case ACTION_MENU_PRIORITY_LOW_TAG:
+            priority = TR_PRI_LOW;
+            break;
+        default:
+            NSAssert1(NO, @"Unknown priority: %d", [sender tag]);
+    }
+    
+    [fMenuTorrent setPriority: priority];
+    
+    [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateUI" object: nil];
+}
+
 - (void) checkFile: (id) sender
 {
     NSIndexSet * indexSet = [(FileListNode *)[sender representedObject] indexes];
@@ -827,11 +814,6 @@
     [[NSNotificationCenter defaultCenter] postNotificationName: @"UpdateStats" object: nil];
 }
 
-- (void) moveDataFile: (id) sender
-{
-    [fController moveDataFiles: [NSArray arrayWithObject: fMenuTorrent]];
-}
-
 - (void) togglePiecesBar
 {
     //stop previous animation
@@ -839,7 +821,7 @@
         [fPiecesBarAnimation release];
     
     NSMutableArray * progressMarks = [NSMutableArray arrayWithCapacity: 16];
-    for (NSAnimationProgress i = 0.0625f; i <= 1.0f; i += 0.0625f)
+    for (NSAnimationProgress i = 0.0625; i <= 1.0; i += 0.0625)
         [progressMarks addObject: [NSNumber numberWithFloat: i]];
     
     fPiecesBarAnimation = [[NSAnimation alloc] initWithDuration: TOGGLE_PROGRESS_SECONDS animationCurve: NSAnimationEaseIn];
@@ -866,7 +848,7 @@
         if ([fDefaults boolForKey: @"PiecesBar"])
             fPiecesBarPercent = progress;
         else
-            fPiecesBarPercent = 1.0f - progress;
+            fPiecesBarPercent = 1.0 - progress;
         
         [self reloadData];
     }
@@ -881,33 +863,6 @@
 
 @implementation TorrentTableView (Private)
 
-- (BOOL) pointInControlRect: (NSPoint) point
-{
-    NSInteger row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]])
-        return NO;
-    
-    return NSPointInRect(point, [fTorrentCell controlButtonRectForBounds: [self rectOfRow: row]]);
-}
-
-- (BOOL) pointInRevealRect: (NSPoint) point
-{
-    NSInteger row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]])
-        return NO;
-    
-    return NSPointInRect(point, [fTorrentCell revealButtonRectForBounds: [self rectOfRow: row]]);
-}
-
-- (BOOL) pointInActionRect: (NSPoint) point
-{
-    NSInteger row = [self rowAtPoint: point];
-    if (row < 0 || ![[self itemAtRow: row] isKindOfClass: [Torrent class]])
-        return NO;
-    
-    return NSPointInRect(point, [fTorrentCell iconRectForBounds: [self rectOfRow: row]]);
-}
-
 - (BOOL) pointInGroupStatusRect: (NSPoint) point
 {
     NSInteger row = [self rowAtPoint: point];
@@ -921,60 +876,10 @@
 
 - (void) setGroupStatusColumns
 {
-    BOOL ratio = [fDefaults boolForKey: @"DisplayGroupRowRatio"];
+    const BOOL ratio = [fDefaults boolForKey: @"DisplayGroupRowRatio"];
     
     [[self tableColumnWithIdentifier: @"DL"] setHidden: ratio];
     [[self tableColumnWithIdentifier: @"DL Image"] setHidden: ratio];
-    
-    //change size of image column
-    NSTableColumn * ulImageTableColumn = [self tableColumnWithIdentifier: @"UL Image"];
-    CGFloat oldWidth = [ulImageTableColumn width], newWidth = ratio ? GROUP_RATIO_IMAGE_COLUMN_WIDTH : GROUP_SPEED_IMAGE_COLUMN_WIDTH;
-    if (oldWidth != newWidth)
-    {
-        [ulImageTableColumn setWidth: newWidth];
-        
-        NSTableColumn * groupTableColumn = [self tableColumnWithIdentifier: @"Group"];
-        [groupTableColumn setWidth: [groupTableColumn width] - (newWidth - oldWidth)];
-    }
-}
-
-- (void) createFileMenu: (NSMenu *) menu forFiles: (NSArray *) files
-{
-    NSEnumerator * enumerator = [files objectEnumerator];
-    FileListNode * node;
-    while ((node = [enumerator nextObject]))
-    {
-        NSString * name = [node name];
-        
-        NSMenuItem * item = [[NSMenuItem alloc] initWithTitle: name action: @selector(checkFile:) keyEquivalent: @""];
-        
-        NSImage * icon;
-        if (![node isFolder])
-            icon = [[NSWorkspace sharedWorkspace] iconForFileType: [name pathExtension]];
-        else
-        {
-            NSMenu * itemMenu = [[NSMenu alloc] initWithTitle: name];
-            [itemMenu setAutoenablesItems: NO];
-            [item setSubmenu: itemMenu];
-            [itemMenu setDelegate: self];
-            [itemMenu release];
-            
-            icon = [[NSWorkspace sharedWorkspace] iconForFileType: NSFileTypeForHFSTypeCode('fldr')];
-        }
-        
-        [item setRepresentedObject: node];
-        
-        [icon setScalesWhenResized: YES];
-        [icon setSize: NSMakeSize(16.0, 16.0)];
-        [item setImage: icon];
-        
-        NSIndexSet * indexSet = [node indexes];
-        [item setState: [fMenuTorrent checkForFiles: indexSet]];
-        [item setEnabled: [fMenuTorrent canChangeDownloadCheckForFiles: indexSet]];
-        
-        [menu addItem: item];
-        [item release];
-    }
 }
 
 @end
