@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: app.cc 9868 2010-01-04 21:00:47Z charles $
+ * $Id: app.cc 10576 2010-05-01 04:15:33Z charles $
  */
 
 #include <cassert>
@@ -138,7 +138,7 @@ MyApp :: MyApp( int& argc, char ** argv ):
     if( username != 0 )
         myPrefs->set( Prefs::SESSION_REMOTE_USERNAME, username );
     if( password != 0 )
-        myPrefs->set( Prefs::SESSION_REMOTE_USERNAME, password );
+        myPrefs->set( Prefs::SESSION_REMOTE_PASSWORD, password );
     if( ( host != 0 ) || ( port != 0 ) || ( username != 0 ) || ( password != 0 ) )
         myPrefs->set( Prefs::SESSION_IS_REMOTE, true );
 
