@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,23 +7,27 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: mainwin.cc 8570 2009-05-31 19:33:48Z charles $
+ * $Id: mainwin.cc 9890 2010-01-05 23:47:50Z charles $
  */
 
 #include <cassert>
 #include <iostream>
 
 #include <QCheckBox>
+#include <QCloseEvent>
 #include <QDesktopServices>
 #include <QFileDialog>
+#include <QHBoxLayout>
+#include <QInputDialog>
 #include <QLabel>
+#include <QSignalMapper>
 #include <QSize>
 #include <QStyle>
-#include <QHBoxLayout>
 #include <QSystemTrayIcon>
 #include <QUrl>
-#include <QSignalMapper>
 
+#include <libtransmission/transmission.h>
+#include <libtransmission/utils.h>
 #include <libtransmission/version.h>
 
 #include "about.h"
@@ -109,7 +113,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     const QSize smallIconSize( i, i );
 
     // icons
-    ui.action_Add->setIcon( getStockIcon( "list-add", QStyle::SP_DialogOpenButton ) );
+    ui.action_AddFile->setIcon( getStockIcon( "list-add", QStyle::SP_DialogOpenButton ) );
     ui.action_New->setIcon( getStockIcon( "document-new", QStyle::SP_DesktopIcon ) );
     ui.action_Properties->setIcon( getStockIcon( "document-properties", QStyle::SP_DesktopIcon ) );
     ui.action_OpenFolder->setIcon( getStockIcon( "folder-open", QStyle::SP_DirOpenIcon ) );
@@ -151,13 +155,15 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_Announce, SIGNAL(triggered()), this, SLOT(reannounceSelected()) );
     connect( ui.action_StartAll, SIGNAL(triggered()), this, SLOT(startAll()));
     connect( ui.action_PauseAll, SIGNAL(triggered()), this, SLOT(pauseAll()));
-    connect( ui.action_Add, SIGNAL(triggered()), this, SLOT(openTorrent()));
+    connect( ui.action_AddFile, SIGNAL(triggered()), this, SLOT(openTorrent()));
+    connect( ui.action_AddURL, SIGNAL(triggered()), this, SLOT(openURL()));
     connect( ui.action_New, SIGNAL(triggered()), this, SLOT(newTorrent()));
     connect( ui.action_Preferences, SIGNAL(triggered()), myPrefsDialog, SLOT(show()));
     connect( ui.action_Statistics, SIGNAL(triggered()), myStatsDialog, SLOT(show()));
     connect( ui.action_About, SIGNAL(triggered()), myAboutDialog, SLOT(show()));
     connect( ui.action_Contents, SIGNAL(triggered()), this, SLOT(openHelp()));
     connect( ui.action_OpenFolder, SIGNAL(triggered()), this, SLOT(openFolder()));
+    connect( ui.action_CopyMagnetToClipboard, SIGNAL(triggered()), this, SLOT(copyMagnetLinkToClipboard()));
     connect( ui.action_SetLocation, SIGNAL(triggered()), this, SLOT(setLocation()));
     connect( ui.action_Properties, SIGNAL(triggered()), this, SLOT(openProperties()));
     connect( ui.action_SessionDialog, SIGNAL(triggered()), mySessionDialog, SLOT(show()));
@@ -165,17 +171,21 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
 
     QAction * sep2 = new QAction( this );
     sep2->setSeparator( true );
+    QAction * sep3 = new QAction( this );
+    sep3->setSeparator( true );
 
     // context menu
     QList<QAction*> actions;
     actions << ui.action_Properties
             << ui.action_OpenFolder
-            << ui.action_SetLocation
             << sep2
             << ui.action_Start
-            << ui.action_Pause
-            << ui.action_Verify
             << ui.action_Announce
+            << ui.action_Pause
+            << ui.action_CopyMagnetToClipboard
+            << sep3
+            << ui.action_Verify
+            << ui.action_SetLocation
             << sep
             << ui.action_Remove
             << ui.action_Delete;
@@ -213,7 +223,8 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     actionGroup->addAction( ui.action_SortByTracker );
 
     QMenu * menu = new QMenu( );
-    menu->addAction( ui.action_Add );
+    menu->addAction( ui.action_AddFile );
+    menu->addAction( ui.action_AddURL );
     menu->addSeparator( );
     menu->addAction( ui.action_ShowMainWindow );
     menu->addAction( ui.action_ShowMessageLog );
@@ -227,7 +238,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     myTrayIcon.setIcon( QApplication::windowIcon( ) );
 
     connect( &myPrefs, SIGNAL(changed(int)), this, SLOT(refreshPref(int)) );
-    connect( ui.action_ShowMainWindow, SIGNAL(toggled(bool)), this, SLOT(toggleWindows()));
+    connect( ui.action_ShowMainWindow, SIGNAL(toggled(bool)), this, SLOT(toggleWindows(bool)));
     connect( &myTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
              this, SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
 
@@ -285,6 +296,23 @@ TrMainWindow :: ~TrMainWindow( )
 *****
 ****/
 
+void
+TrMainWindow :: closeEvent( QCloseEvent * event )
+{
+    // if they're using a tray icon, close to the tray
+    // instead of exiting
+    if( !myPrefs.getBool( Prefs :: SHOW_TRAY_ICON ) )
+        event->accept( );
+    else {
+        toggleWindows( false );
+        event->ignore( );
+    }
+}
+
+/****
+*****
+****/
+
 void
 TrMainWindow :: onSessionSourceChanged( )
 {
@@ -444,7 +472,7 @@ TrMainWindow :: createStatusBar( )
         h->addWidget( l );
 
     h->addStretch( 1 );
-  
+
         a = new QActionGroup( this );
         a->addAction( ui.action_TotalRatio );
         a->addAction( ui.action_TotalTransfer );
@@ -460,13 +488,13 @@ TrMainWindow :: createStatusBar( )
         connect( ui.action_SessionRatio, SIGNAL(triggered()), this, SLOT(showSessionRatio()));
         connect( ui.action_SessionTransfer, SIGNAL(triggered()), this, SLOT(showSessionTransfer()));
         p = myStatsModeButton = new TrIconPushButton( this );
-        p->setIcon( getStockIcon( "view-refresh", QStyle::SP_BrowserReload ) );
+        p->setIcon( QIcon( ":/icons/ratio.png" ) );
         p->setFlat( true );
         p->setMenu( m );
-        h->addWidget( p );  
+        h->addWidget( p );
         l = myStatsLabel = new QLabel( this );
-        h->addWidget( l );  
-   
+        h->addWidget( l );
+
         w = new QWidget( this );
         w->setMinimumSize( HIG::PAD_BIG, 1 );
         w->setMaximumSize( HIG::PAD_BIG, 1 );
@@ -634,6 +662,13 @@ TrMainWindow :: openFolder( )
     QDesktopServices :: openUrl( QUrl::fromLocalFile( path ) );
 }
 
+void
+TrMainWindow :: copyMagnetLinkToClipboard( )
+{
+    const int id( *getSelectedTorrents().begin() );
+    mySession.copyMagnetLinkToClipboard( id );
+}
+
 void
 TrMainWindow :: openHelp( )
 {
@@ -641,7 +676,7 @@ TrMainWindow :: openHelp( )
     int major, minor;
     sscanf( SHORT_VERSION_STRING, "%d.%d", &major, &minor );
     char url[128];
-    snprintf( url, sizeof( url ), fmt, major, minor/10 );
+    tr_snprintf( url, sizeof( url ), fmt, major, minor/10 );
     QDesktopServices :: openUrl( QUrl( QString( url ) ) );
 }
 
@@ -700,7 +735,7 @@ TrMainWindow :: refreshStatusBar( )
         str = tr( "Down: %1, Up: %2" ).arg( Utils :: sizeToString( stats.downloadedBytes ) )
                                       .arg( Utils :: sizeToString( stats.uploadedBytes ) );
     }
-    else /* default is "total-ratio" */
+    else // default is "total-ratio"
     {
         str = tr( "Ratio: %1" ).arg( Utils :: ratioToString( mySession.getCumulativeStats().ratio ) );
     }
@@ -719,21 +754,23 @@ TrMainWindow :: refreshActionSensitivity( )
     const QItemSelectionModel * selectionModel( ui.listView->selectionModel( ) );
     const int rowCount( model->rowCount( ) );
 
-    /* count how many torrents are selected, paused, etc */
+    // count how many torrents are selected, paused, etc
     for( int row=0; row<rowCount; ++row ) {
         const QModelIndex modelIndex( model->index( row, 0 ) );
         assert( model == modelIndex.model( ) );
         const Torrent * tor( model->data( modelIndex, TorrentModel::TorrentRole ).value<const Torrent*>( ) );
-        const bool isSelected( selectionModel->isSelected( modelIndex ) );
-        const bool isPaused( tor->isPaused( ) );
-        if( isSelected )
-            ++selected;
-        if( isPaused )
-            ++ paused;
-        if( isSelected && isPaused )
-            ++selectedAndPaused;
-        if( tor->canManualAnnounce( ) )
-            ++canAnnounce;
+        if( tor ) {
+            const bool isSelected( selectionModel->isSelected( modelIndex ) );
+            const bool isPaused( tor->isPaused( ) );
+            if( isSelected )
+                ++selected;
+            if( isPaused )
+                ++ paused;
+            if( isSelected && isPaused )
+                ++selectedAndPaused;
+            if( tor->canManualAnnounce( ) )
+                ++canAnnounce;
+        }
     }
 
     const bool haveSelection( selected > 0 );
@@ -746,6 +783,7 @@ TrMainWindow :: refreshActionSensitivity( )
 
     const bool oneSelection( selected == 1 );
     ui.action_OpenFolder->setEnabled( oneSelection && mySession.isLocal( ) );
+    ui.action_CopyMagnetToClipboard->setEnabled( oneSelection );
 
     ui.action_SelectAll->setEnabled( selected < rowCount );
     ui.action_StartAll->setEnabled( paused > 0 );
@@ -883,16 +921,31 @@ TrMainWindow :: setStatusbarVisible( bool visible )
 **/
 
 void
-TrMainWindow :: toggleWindows( )
+TrMainWindow :: toggleWindows( bool doShow )
 {
-    setVisible( !isVisible( ) );
+    if( !doShow )
+    {
+        hide( );
+    }
+    else
+    {
+        if ( !isVisible( ) ) show( );
+        if ( isMinimized( ) ) showNormal( );
+        activateWindow( );
+        raise( );
+    }
 }
 
 void
 TrMainWindow :: trayActivated( QSystemTrayIcon::ActivationReason reason )
 {
     if( reason == QSystemTrayIcon::Trigger )
-        ui.action_ShowMainWindow->toggle( );
+    {
+        if( isMinimized ( ) )
+            toggleWindows( true );
+        else
+            ui.action_ShowMainWindow->toggle( );
+    }
 }
 
 
@@ -934,7 +987,7 @@ TrMainWindow :: refreshPref( int key )
         case Prefs::DSPEED_ENABLED:
             (myPrefs.get<bool>(key) ? myDlimitOnAction : myDlimitOffAction)->setChecked( true );
             break;
-     
+
         case Prefs::DSPEED:
             myDlimitOnAction->setText( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( myPrefs.get<int>(key) ) ) ) );
             break;
@@ -942,7 +995,7 @@ TrMainWindow :: refreshPref( int key )
         case Prefs::USPEED_ENABLED:
             (myPrefs.get<bool>(key) ? myUlimitOnAction : myUlimitOffAction)->setChecked( true );
             break;
-     
+
         case Prefs::USPEED:
             myUlimitOnAction->setText( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( myPrefs.get<int>(key) ) ) ) );
             break;
@@ -1003,12 +1056,19 @@ TrMainWindow :: refreshPref( int key )
             break;
 
         case Prefs :: ALT_SPEED_LIMIT_ENABLED:
-            b = myPrefs.getBool( key );
+        case Prefs :: ALT_SPEED_LIMIT_UP:
+        case Prefs :: ALT_SPEED_LIMIT_DOWN: {
+            b = myPrefs.getBool( Prefs :: ALT_SPEED_LIMIT_ENABLED );
             myAltSpeedButton->setChecked( b );
             myAltSpeedButton->setIcon( b ? mySpeedModeOnIcon : mySpeedModeOffIcon );
-            myAltSpeedButton->setToolTip( b ? tr( "Click to disable Speed Limit Mode" )
-                                            : tr( "Click to enable Speed Limit Mode" ) );
+            const QString fmt = b ? tr( "Click to disable Temporary Speed Limits\n(%1 down, %2 up)" )
+                                  : tr( "Click to enable Temporary Speed Limits\n(%1 down, %2 up)" );
+            const Speed d = Speed::fromKbps( myPrefs.getInt( Prefs::ALT_SPEED_LIMIT_DOWN ) );
+            const Speed u = Speed::fromKbps( myPrefs.getInt( Prefs::ALT_SPEED_LIMIT_UP ) );
+            myAltSpeedButton->setToolTip( fmt.arg( Utils::speedToString( d ) )
+                                             .arg( Utils::speedToString( u ) ) );
             break;
+        }
 
         default:
             break;
@@ -1022,8 +1082,8 @@ TrMainWindow :: refreshPref( int key )
 void
 TrMainWindow :: newTorrent( )
 {
-    MakeDialog * d = new MakeDialog( mySession, this );
-    d->show( );
+    MakeDialog * dialog = new MakeDialog( mySession, this );
+    dialog->show( );
 }
 
 void
@@ -1038,18 +1098,33 @@ TrMainWindow :: openTorrent( )
         myFileDialog->setFileMode( QFileDialog::ExistingFiles );
 
 
-        QCheckBox * button = new QCheckBox( tr( "Display &options dialog" ) );
+        QCheckBox * button = new QCheckBox( tr( "Show &options dialog" ) );
         button->setChecked( myPrefs.getBool( Prefs::OPTIONS_PROMPT ) );
         QGridLayout * layout = dynamic_cast<QGridLayout*>(myFileDialog->layout());
         layout->addWidget( button, layout->rowCount( ), 0, 1, -1, Qt::AlignLeft );
         myFileDialogOptionsCheck = button;
 
-        connect( myFileDialog, SIGNAL(filesSelected(const QStringList&)), this, SLOT(addTorrents(const QStringList&)));
+        connect( myFileDialog, SIGNAL(filesSelected(const QStringList&)),
+                 this, SLOT(addTorrents(const QStringList&)));
     }
 
     myFileDialog->show( );
 }
 
+void
+TrMainWindow :: openURL( )
+{
+    bool ok;
+    const QString key = QInputDialog::getText( this,
+                                               tr( "Add URL or Magnet Link" ),
+                                               tr( "Add URL or Magnet Link" ),
+                                               QLineEdit::Normal,
+                                               QString( ),
+                                               &ok );
+    if( ok && !key.isEmpty( ) )
+        mySession.addTorrent( key );
+}
+
 void
 TrMainWindow :: addTorrents( const QStringList& filenames )
 {
