@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: mainwin.h 8570 2009-05-31 19:33:48Z charles $
+ * $Id: mainwin.h 9868 2010-01-04 21:00:47Z charles $
  */
 
 #ifndef MAIN_WINDOW_H
@@ -50,6 +50,9 @@ class TrMainWindow: public QMainWindow
 {
         Q_OBJECT
 
+    private:
+        virtual void closeEvent( QCloseEvent * event );
+
     private:
         time_t myLastFullUpdateTime;
         QDialog * mySessionDialog;
@@ -107,18 +110,20 @@ class TrMainWindow: public QMainWindow
         void refreshTitle( );
         void refreshStatusBar( );
         void openTorrent( );
+        void openURL( );
         void newTorrent( );
         void trayActivated( QSystemTrayIcon::ActivationReason );
         void refreshPref( int key );
         void addTorrents( const QStringList& filenames );
         void openHelp( );
         void openFolder( );
+        void copyMagnetLinkToClipboard( );
         void setLocation( );
         void openProperties( );
         void toggleSpeedMode( );
         void dataReadProgress( );
         void dataSendProgress( );
-        void toggleWindows( );
+        void toggleWindows( bool doShow );
         void onSetPrefs( );
         void onSetPrefs( bool );
         void onSessionSourceChanged( );
