@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.cc 10077 2010-02-02 05:34:26Z charles $
+ * $Id: torrent.cc 10586 2010-05-01 04:46:08Z charles $
  */
 
 #include <cassert>
@@ -388,8 +388,8 @@ Torrent :: compareETA( const Torrent& that ) const
     const bool haveA( hasETA( ) );
     const bool haveB( that.hasETA( ) );
     if( haveA && haveB ) return getETA() - that.getETA();
-    if( haveA ) return -1;
-    if( haveB ) return 1;
+    if( haveA ) return 1;
+    if( haveB ) return -1;
     return 0;
 }
 
